BeginPackage["PDEPlot`", {"JLink`", "Utilities`FilterOptions`"}]

Options[PDEPlot] = {PlotRange->{{-1,1},{-1,1}}, ImageSize->{400,400}};

Options[NDSolvePlot] = {PlotRange->{{-1,1},{-1,1}}, ImageSize->{400,400}, Frames->100};

Begin["`Private`"]

NDSolvePlot[eqns_, dvars_, ivart_, ivarsp_, opts___?OptionQ] :=
JavaBlock[Catch[
    Module[{ivpd, frame, canvas, offscreen, g, mml, mouseDraggedFunc,	mouseReleasedFunc,	xpix, ypix, xmax, xmin, ymax, ymin, xmap, xmapinv, ymap, ymapinv, begin, end, flops},
        flops = Flatten[{opts, Options[NDSolvePlot]}];
        RangeParameters[{{xmin, xmax},{ymin, ymax}}, {xpix, ypix}, {xmap, ymap}, {xmapinv, ymapinv}, flops];
        frames = Frames /. flops;
        If[Not[PIntegerQ[frames]],
            Throw[NDSolve[eqns, dvars, ivart, ivarsp, FilterOptions[NDSolve, opts]]]
        ];
        ivpd = NDSolve`ProcessEquations[eqns, dvars, ivart, ivarsp, FilterOptions[NDSolve, opts]];
        If[Not[And[ListQ[ivpd], (Length[ivpd] == 1), System`Private`ValidQ[First[ivpd]]]],
            Throw[$Failed]];
        ivpd = First[ivpd];
        begin = First[ivpd@"CurrentTime"[]];
        end = Last[ivart];
        If[begin == end, end = ivart[[2]]];
        dtime = (end - begin)/frames;
        now = If[dtime < 0, "Backward", "Forward"];
        sol = NDSolve`ProcessSolutions[ivpd, now];
        ifun = sol[[1, 2, 0]];
        grid = ifun[[3,1]];
        {frame, canvas, offscreen, g, mml, mmml} = SetUpFrame["PlotSolution", {xpix, ypix},{None, nothing, None}];
        CheckAbort[
        Do[
            NDSolve`Iterate[ivpd, time];
            sol = NDSolve`ProcessSolutions[ivpd, now];
            ifun = sol[[1, 2, 0]];
            values = ifun[grid];
            g@clearRect[0, 0, xpix, ypix];
            g@drawPolyline[xmap[grid], ymap[values], Length[grid]];
            canvas@setImage[offscreen];
            canvas@repaintNow[],
            {time, begin, end, dtime}];
        frame@dispose[],
        frame@dispose[]];
        {NDSolve`ProcessSolutions[ivpd]}
    ]
]]

SetAttributes[PDEPlot, HoldFirst];
PDEPlot[ivpd_, dt_, opts___?OptionQ] :=
Catch[JavaBlock[
	Module[{frame, canvas, offscreen, g, mml, mouseDraggedFunc,	mouseReleasedFunc,	xpix, ypix, xmax, xmin, ymax, ymin, xmap, xmapinv, ymap, ymapinv, begin, end, flops},
    flops = Flatten[{opts, Options[PDEPlot]}];
    RangeParameters[{{xmin, xmax},{ymin, ymax}}, {xpix, ypix}, {xmap, ymap}, {xmapinv, ymapinv}, flops];
    mouseDraggedFunc[__] := Module[{now},
        now = ivpd[[2, -1, 1, 1]];
        NDSolve`Iterate[ivpd, now + dt];
        ShowSolution[ivpd]
	];
    ShowSolution[ivpdarg_] := 
    Module[{sol, ifun, grid, values},
        sol = NDSolve`ProcessSolutions[ivpdarg, Last];
        ifun = sol[[1,2,0]];
        grid = ifun[[3,1]];
        values = ifun[grid];
        g@clearRect[0, 0, xpix, ypix];
        g@drawPolyline[xmap[grid], ymap[values], Length[grid]];
        canvas@setImage[offscreen];
        canvas@repaintNow[];
    ];      
    {frame, canvas, offscreen, g, mml, mmml} = SetUpFrame["PlotSolution", {xpix, ypix},{None, mouseDraggedFunc, None}];
    ShowSolution[ivpd];
    DoModal[];
]]]

RangeParameters[{{xmin_, xmax_},{ymin_, ymax_}}, {xpix_, ypix_}, {xmap_, ymap_}, {xmapinv_, ymapinv_}, flops_] := 
Module[{temp, dx, dy},
    temp = N[PlotRange /. flops];
    If[Not[TensorDimensionQ[temp, {2, 2}, RealQ]],Throw[PlotRange[temp]]];
    {{xmin, xmax},{ymin, ymax}} = temp;
    temp = ImageSize /. flops;
    If[Not[TensorDimensionQ[temp, {2}, PIntegerQ]], Throw[ImageSize[temp]]];
    {xpix, ypix} = temp;
	dx = N[xpix/(xmax - xmin)];
	xmap = Function[Evaluate[Round[dx (# - xmin)]]];
    xmapinv = Function[Evaluate[xmin + #/dx]];
	dy = N[ypix/(ymax - ymin)];
	ymap = Function[Evaluate[ypix - Round[dy (# - ymin)]]];
    ymapinv = Function[Evaluate[ymin + (ypix - #)/dy]]
]

PIntegerQ[x_] := And[IntegerQ[x], x > 0]

RealQ[x_] := And[NumberQ[x], Not[Head[x] === Complex]]

TensorDimensionQ[x_, {dim_}, pred_] := And[VectorQ[x, pred], Length[x] == dim]

TensorDimensionQ[x_, {dim1_, dim2_}, pred_] := And[Dimensions[x] === {dim1, dim2}, MatrixQ[x, pred]]

SetUpFrame[title_, {xpix_, ypix_}, {pressed_, dragged_, released_}] := 
Module[{frame, mathCanvas, offscreen, g, mml, mmml},
    InstallJava[];
    frame = JavaNew["com.wolfram.jlink.MathFrame", title];
    frame@setLayout[JavaNew["java.awt.BorderLayout"]];
    frame@setLocation[1024 - xpix, 40];
    mathCanvas = JavaNew["com.wolfram.jlink.MathCanvas"];
    frame@add["Center", mathCanvas];
    frame@setSize[xpix, ypix];
    frame@layout[];
    JavaShow[frame];
    (* Now create the offscreen image and the graphics context
    for drawing into it. *)
    offscreen = mathCanvas@createImage[mathCanvas@getSize[]@width,
                      		 mathCanvas@getSize[]@height];
    g = offscreen@getGraphics[];
    If[Or[pressed =!= None, released =!= None],
        mml = JavaNew["com.wolfram.jlink.MathMouseListener"];
        If[pressed =!= None, 
            mml@setHandler["mousePressed", ToString[pressed]]];
        If[released =!= None, 
            mml@setHandler["mouseReleased", ToString[released]]];
        mathCanvas@addMouseListener[mml]
        ];
    If[dragged =!= None,
        mmml = JavaNew["com.wolfram.jlink.MathMouseMotionListener"];
        mmml@setHandler["mouseDragged", ToString[dragged]];
        mathCanvas@addMouseMotionListener[mmml]
        ];
    frame@setModal[];
    {frame, mathCanvas, offscreen, g, mml, mmml}]

End[]

EndPackage[]
