(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    308357,       6682]*)
(*NotebookOutlinePosition[    309549,       6721]*)
(*  CellTagsIndexPosition[    309395,       6713]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[TextData[Cell[BoxData[GridBox[{
        {
          ButtonBox[
            StyleBox["\[FirstPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageFirst"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"First Slide"], 
          ButtonBox[
            StyleBox["\[LeftPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPagePrevious"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Previous Slide"], 
          ButtonBox[
            StyleBox["\[RightPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageNext"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Next Slide"], 
          ButtonBox[
            StyleBox["\[LastPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageLast"]}],
            ButtonStyle->"SlideHyperlink"], " ", 
          ButtonBox[
            StyleBox[
              RowBox[{
                CounterBox["SlideShowNavigationBar"], \(\(\ \)\(of\)\(\ \)\), 
                
                CounterBox["SlideShowNavigationBar", "LastSlide"]}],
              "SR"],
            ButtonFrame->"None"]}
        }]]]], "SlideShowNavigationBar"],

Cell[CellGroupData[{

Cell["Dense Numerical Linear Algebra", "Section",
  CellTags->"DenseLinearAlgebra"],

Cell[TextData[{
  "The biggest advantage of using a linear algebra library based on BLAS is \
that the BLAS are simple enough to fine tune for individual platforms.  For \
the many platforms ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " supports, we use a variety of different BLAS libraries.  On this machine \
you'll be seeing the Intel Math Kernel Library (MKL)."
}], "Text"],

Cell["\<\
In the following slides, I'll demonstrate a couple of examples, then show \
some graphs.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[GridBox[{
        {
          ButtonBox[
            StyleBox["\[FirstPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageFirst"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"First Slide"], 
          ButtonBox[
            StyleBox["\[LeftPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPagePrevious"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Previous Slide"], 
          ButtonBox[
            StyleBox["\[RightPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageNext"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Next Slide"], 
          ButtonBox[
            StyleBox["\[LastPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageLast"]}],
            ButtonStyle->"SlideHyperlink"], " ", 
          ButtonBox[
            StyleBox[
              RowBox[{
                CounterBox["SlideShowNavigationBar"], \(\(\ \)\(of\)\(\ \)\), 
                
                CounterBox["SlideShowNavigationBar", "LastSlide"]}],
              "SR"],
            ButtonFrame->"None"]}
        }]]]], "SlideShowNavigationBar"],

Cell[TextData[{
  "The command ",
  StyleBox["R", "MR"],
  " uses the Parallel Computing Toolkit in order to perform the same \
operation in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 4.2 and 5.0"
}], "MathCaption"],

Cell[BoxData[
    \(R[$Version]\)], "Input",
  CellLabel->"In[16]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[GridBox[{
        {
          ButtonBox[
            StyleBox["\[FirstPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageFirst"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"First Slide"], 
          ButtonBox[
            StyleBox["\[LeftPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPagePrevious"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Previous Slide"], 
          ButtonBox[
            StyleBox["\[RightPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageNext"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Next Slide"], 
          ButtonBox[
            StyleBox["\[LastPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageLast"]}],
            ButtonStyle->"SlideHyperlink"], " ", 
          ButtonBox[
            StyleBox[
              RowBox[{
                CounterBox["SlideShowNavigationBar"], \(\(\ \)\(of\)\(\ \)\), 
                
                CounterBox["SlideShowNavigationBar", "LastSlide"]}],
              "SR"],
            ButtonFrame->"None"]}
        }]]]], "SlideShowNavigationBar"],

Cell["\<\
Matrix multiplication is probably the most important basic operation for \
linear algebra. \
\>", "MathCaption"],

Cell[BoxData[
    RowBox[{"R", "[", " ", 
      RowBox[{\(SeedRandom[1234]\), ";", 
        "\[IndentingNewLine]", \(A\  = \ 
          Table[\ Random[], \ {1000}, \ {1000}]\), ";", " ", 
        "\[IndentingNewLine]", 
        RowBox[{"Timing", "[", 
          StyleBox[" ",
            FontColor->RGBColor[1, 0, 0]], 
          RowBox[{
            StyleBox[\(A\  . \ A\),
              FontColor->RGBColor[1, 0, 0]], ";"}], " ", "]"}]}], " ", 
      "]"}]], "Input",
  CellLabel->"In[19]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[GridBox[{
        {
          ButtonBox[
            StyleBox["\[FirstPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageFirst"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"First Slide"], 
          ButtonBox[
            StyleBox["\[LeftPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPagePrevious"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Previous Slide"], 
          ButtonBox[
            StyleBox["\[RightPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageNext"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Next Slide"], 
          ButtonBox[
            StyleBox["\[LastPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageLast"]}],
            ButtonStyle->"SlideHyperlink"], " ", 
          ButtonBox[
            StyleBox[
              RowBox[{
                CounterBox["SlideShowNavigationBar"], \(\(\ \)\(of\)\(\ \)\), 
                
                CounterBox["SlideShowNavigationBar", "LastSlide"]}],
              "SR"],
            ButtonFrame->"None"]}
        }]]]], "SlideShowNavigationBar"],

Cell["\<\
Solving linear systems is another essential, though less basic, operation. \
\>", "MathCaption"],

Cell[BoxData[
    RowBox[{"R", "[", 
      RowBox[{\(b\  = \ Table[\ Random[], \ {10\^3}]\), ";", "  ", 
        "\[IndentingNewLine]", 
        RowBox[{"Timing", "[", " ", 
          RowBox[{
            StyleBox[\(LinearSolve[A, \ b]\),
              FontColor->RGBColor[1, 0, 0]], ";"}], " ", "]"}]}], " ", 
      "]"}]], "Input",
  CellLabel->"In[20]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[GridBox[{
        {
          ButtonBox[
            StyleBox["\[FirstPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageFirst"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"First Slide"], 
          ButtonBox[
            StyleBox["\[LeftPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPagePrevious"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Previous Slide"], 
          ButtonBox[
            StyleBox["\[RightPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageNext"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Next Slide"], 
          ButtonBox[
            StyleBox["\[LastPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageLast"]}],
            ButtonStyle->"SlideHyperlink"], " ", 
          ButtonBox[
            StyleBox[
              RowBox[{
                CounterBox["SlideShowNavigationBar"], \(\(\ \)\(of\)\(\ \)\), 
                
                CounterBox["SlideShowNavigationBar", "LastSlide"]}],
              "SR"],
            ButtonFrame->"None"]}
        }]]]], "SlideShowNavigationBar"],

Cell["\<\
We spent a lot of time determining what BLAS libraries to use and making sure \
that we were getting appropriate speedups on each platform.  Following are a \
couple of graphs which indicate the type of tests we did for comparisons.\
\>", "Text"],

Cell["\<\
Wolfram Research has done extensive testing to be sure that the BLAS library \
we choose for each platform is well optimized and reliable.  One way of \
analyzing such data is to show a graph of evaluate times versus input size \
for a class of matrices.  This shows a logarithmic plot of the time it takes \
to evaluate Dot as a function of matrix size on a 4 processor Itanium 2 \
machine.  \
\>", "Text"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .70902 
%%ImageSize: 850 602.664 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.47619 0.397834 0.47619 [
[ 0 0 0 0 ]
[ 1 .70902 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
% Start of sub-graphic
p
0.0238095 0.103532 0.97619 0.692136 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.000463897 0.361377 0.112308 [
[.02381 -0.0125 -6.4375 -18 ]
[.02381 -0.0125 6.4375 0 ]
[.25576 -0.0125 -15.3125 -18 ]
[.25576 -0.0125 15.3125 0 ]
[.48771 -0.0125 -19.75 -18 ]
[.48771 -0.0125 19.75 0 ]
[.71966 -0.0125 -19.75 -18 ]
[.71966 -0.0125 19.75 0 ]
[.9516 -0.0125 -19.75 -18 ]
[.9516 -0.0125 19.75 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -60.0625 -36 ]
[.5 -0.0125 60.0625 -18 ]
[-0.0125 .02445 -43.9375 -9 ]
[-0.0125 .02445 0 9 ]
[-0.0125 .13676 -35.0625 -9 ]
[-0.0125 .13676 0 9 ]
[-0.0125 .24907 -26.1875 -9 ]
[-0.0125 .24907 0 9 ]
[-0.0125 .36138 -12.875 -9 ]
[-0.0125 .36138 0 9 ]
[-0.0125 .47368 -21.75 -9 ]
[-0.0125 .47368 0 9 ]
[-0.0125 .58599 -30.625 -9 ]
[-0.0125 .58599 0 9 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -109.625 -9 ]
[-0.01875 .30902 -43.9375 9 ]
[ 0 .61803 .125 0 ]
[1.0125 .02445 0 -9 ]
[1.0125 .02445 43.9375 9 ]
[1.0125 .13676 0 -9 ]
[1.0125 .13676 35.0625 9 ]
[1.0125 .24907 0 -9 ]
[1.0125 .24907 26.1875 9 ]
[1.0125 .36138 0 -9 ]
[1.0125 .36138 12.875 9 ]
[1.0125 .47368 0 -9 ]
[1.0125 .47368 21.75 9 ]
[1.0125 .58599 0 -9 ]
[1.0125 .58599 30.625 9 ]
[ 1 0 .125 0 ]
[.5 .63053 -182.594 0 ]
[.5 .63053 182.594 18 ]
[ -0.0005 0 0 0 ]
[ 1.0005 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
.5 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .61803 L
s
.25576 0 m
.25576 .61803 L
s
.48771 0 m
.48771 .61803 L
s
.71966 0 m
.71966 .61803 L
s
.9516 0 m
.9516 .61803 L
s
0 .02445 m
1 .02445 L
s
0 .13676 m
1 .13676 L
s
0 .24907 m
1 .24907 L
s
0 .36138 m
1 .36138 L
s
0 .47368 m
1 .47368 L
s
0 .58599 m
1 .58599 L
s
0 g
.02381 0 m
.02381 .00625 L
s
gsave
.02381 -0.0125 -67.4375 -22 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
1.000 setlinewidth
grestore
.25576 0 m
.25576 .00625 L
s
gsave
.25576 -0.0125 -76.3125 -22 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(500) show
1.000 setlinewidth
grestore
.48771 0 m
.48771 .00625 L
s
gsave
.48771 -0.0125 -80.75 -22 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1000) show
1.000 setlinewidth
grestore
.71966 0 m
.71966 .00625 L
s
gsave
.71966 -0.0125 -80.75 -22 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1500) show
1.000 setlinewidth
grestore
.9516 0 m
.9516 .00625 L
s
gsave
.9516 -0.0125 -80.75 -22 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2000) show
1.000 setlinewidth
grestore
.125 Mabswid
.0702 0 m
.0702 .00375 L
s
.11659 0 m
.11659 .00375 L
s
.16298 0 m
.16298 .00375 L
s
.20937 0 m
.20937 .00375 L
s
.30215 0 m
.30215 .00375 L
s
.34854 0 m
.34854 .00375 L
s
.39493 0 m
.39493 .00375 L
s
.44132 0 m
.44132 .00375 L
s
.5341 0 m
.5341 .00375 L
s
.58049 0 m
.58049 .00375 L
s
.62688 0 m
.62688 .00375 L
s
.67327 0 m
.67327 .00375 L
s
.76605 0 m
.76605 .00375 L
s
.81243 0 m
.81243 .00375 L
s
.85882 0 m
.85882 .00375 L
s
.90521 0 m
.90521 .00375 L
s
.99799 0 m
.99799 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -121.062 -40 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Vector) show
125.938 17.000 moveto
(Length) show
179.125 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .02445 m
.00625 .02445 L
s
gsave
-0.0125 .02445 -104.938 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.001) show
102.938 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .13676 m
.00625 .13676 L
s
gsave
-0.0125 .13676 -96.0625 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.01) show
94.062 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .24907 m
.00625 .24907 L
s
gsave
-0.0125 .24907 -87.1875 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.1) show
85.188 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .36138 m
.00625 .36138 L
s
gsave
-0.0125 .36138 -73.875 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
71.875 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .47368 m
.00625 .47368 L
s
gsave
-0.0125 .47368 -82.75 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10) show
80.750 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .58599 m
.00625 .58599 L
s
gsave
-0.0125 .58599 -91.625 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(100) show
89.625 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.001 w
0 .05826 m
.00375 .05826 L
s
0 .07804 m
.00375 .07804 L
s
0 .09207 m
.00375 .09207 L
s
0 .10295 m
.00375 .10295 L
s
0 .11184 m
.00375 .11184 L
s
0 .11936 m
.00375 .11936 L
s
0 .12588 m
.00375 .12588 L
s
0 .13162 m
.00375 .13162 L
s
0 .17057 m
.00375 .17057 L
s
0 .19034 m
.00375 .19034 L
s
0 .20438 m
.00375 .20438 L
s
0 .21526 m
.00375 .21526 L
s
0 .22415 m
.00375 .22415 L
s
0 .23167 m
.00375 .23167 L
s
0 .23818 m
.00375 .23818 L
s
0 .24393 m
.00375 .24393 L
s
0 .28288 m
.00375 .28288 L
s
0 .30265 m
.00375 .30265 L
s
0 .31668 m
.00375 .31668 L
s
0 .32757 m
.00375 .32757 L
s
0 .33646 m
.00375 .33646 L
s
0 .34398 m
.00375 .34398 L
s
0 .35049 m
.00375 .35049 L
s
0 .35624 m
.00375 .35624 L
s
0 .39518 m
.00375 .39518 L
s
0 .41496 m
.00375 .41496 L
s
0 .42899 m
.00375 .42899 L
s
0 .43988 m
.00375 .43988 L
s
0 .44877 m
.00375 .44877 L
s
0 .45629 m
.00375 .45629 L
s
0 .4628 m
.00375 .4628 L
s
0 .46855 m
.00375 .46855 L
s
0 .50749 m
.00375 .50749 L
s
0 .52727 m
.00375 .52727 L
s
0 .5413 m
.00375 .5413 L
s
0 .55218 m
.00375 .55218 L
s
0 .56108 m
.00375 .56108 L
s
0 .5686 m
.00375 .5686 L
s
0 .57511 m
.00375 .57511 L
s
0 .58085 m
.00375 .58085 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -170.625 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Time) show
%%IncludeResource: font Mathematica2-Bold
%%IncludeFont: Mathematica2-Bold
/Mathematica2-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
104.688 17.000 moveto
(H) show
110.250 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(s) show
%%IncludeResource: font Mathematica2-Bold
%%IncludeFont: Mathematica2-Bold
/Mathematica2-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
119.125 17.000 moveto
(L) show
124.688 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.02381 .61178 m
.02381 .61803 L
s
.25576 .61178 m
.25576 .61803 L
s
.48771 .61178 m
.48771 .61803 L
s
.71966 .61178 m
.71966 .61803 L
s
.9516 .61178 m
.9516 .61803 L
s
.125 Mabswid
.0702 .61428 m
.0702 .61803 L
s
.11659 .61428 m
.11659 .61803 L
s
.16298 .61428 m
.16298 .61803 L
s
.20937 .61428 m
.20937 .61803 L
s
.30215 .61428 m
.30215 .61803 L
s
.34854 .61428 m
.34854 .61803 L
s
.39493 .61428 m
.39493 .61803 L
s
.44132 .61428 m
.44132 .61803 L
s
.5341 .61428 m
.5341 .61803 L
s
.58049 .61428 m
.58049 .61803 L
s
.62688 .61428 m
.62688 .61803 L
s
.67327 .61428 m
.67327 .61803 L
s
.76605 .61428 m
.76605 .61803 L
s
.81243 .61428 m
.81243 .61803 L
s
.85882 .61428 m
.85882 .61803 L
s
.90521 .61428 m
.90521 .61803 L
s
.99799 .61428 m
.99799 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .02445 m
1 .02445 L
s
gsave
1.0125 .02445 -61 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.001) show
102.938 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.99375 .13676 m
1 .13676 L
s
gsave
1.0125 .13676 -61 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.01) show
94.062 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.99375 .24907 m
1 .24907 L
s
gsave
1.0125 .24907 -61 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.1) show
85.188 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.99375 .36138 m
1 .36138 L
s
gsave
1.0125 .36138 -61 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
71.875 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.99375 .47368 m
1 .47368 L
s
gsave
1.0125 .47368 -61 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10) show
80.750 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.99375 .58599 m
1 .58599 L
s
gsave
1.0125 .58599 -61 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(100) show
89.625 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.001 w
.99625 .05826 m
1 .05826 L
s
.99625 .07804 m
1 .07804 L
s
.99625 .09207 m
1 .09207 L
s
.99625 .10295 m
1 .10295 L
s
.99625 .11184 m
1 .11184 L
s
.99625 .11936 m
1 .11936 L
s
.99625 .12588 m
1 .12588 L
s
.99625 .13162 m
1 .13162 L
s
.99625 .17057 m
1 .17057 L
s
.99625 .19034 m
1 .19034 L
s
.99625 .20438 m
1 .20438 L
s
.99625 .21526 m
1 .21526 L
s
.99625 .22415 m
1 .22415 L
s
.99625 .23167 m
1 .23167 L
s
.99625 .23818 m
1 .23818 L
s
.99625 .24393 m
1 .24393 L
s
.99625 .28288 m
1 .28288 L
s
.99625 .30265 m
1 .30265 L
s
.99625 .31668 m
1 .31668 L
s
.99625 .32757 m
1 .32757 L
s
.99625 .33646 m
1 .33646 L
s
.99625 .34398 m
1 .34398 L
s
.99625 .35049 m
1 .35049 L
s
.99625 .35624 m
1 .35624 L
s
.99625 .39518 m
1 .39518 L
s
.99625 .41496 m
1 .41496 L
s
.99625 .42899 m
1 .42899 L
s
.99625 .43988 m
1 .43988 L
s
.99625 .44877 m
1 .44877 L
s
.99625 .45629 m
1 .45629 L
s
.99625 .4628 m
1 .4628 L
s
.99625 .46855 m
1 .46855 L
s
.99625 .50749 m
1 .50749 L
s
.99625 .52727 m
1 .52727 L
s
.99625 .5413 m
1 .5413 L
s
.99625 .55218 m
1 .55218 L
s
.99625 .56108 m
1 .56108 L
s
.99625 .5686 m
1 .5686 L
s
.99625 .57511 m
1 .57511 L
s
.99625 .58085 m
1 .58085 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 .36138 m
1 .36138 L
s
gsave
.5 .63053 -243.594 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Evaluation) show
148.125 17.000 moveto
(Time) show
189.812 17.000 moveto
(on) show
213.750 17.000 moveto
(a) show
227.062 17.000 moveto
(900) show
258.125 17.000 moveto
(MHz) show
295.375 17.000 moveto
(4) show
304.250 17.000 moveto
%%IncludeResource: font Mathematica1-Bold
%%IncludeFont: Mathematica1-Bold
/Mathematica1-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
315.062 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(way) show
354.125 17.000 moveto
(Itanium) show
415.312 17.000 moveto
(2) show
424.188 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 .4 .4 r
.004 w
.08087 .03008 m
.08319 .01472 L
.08551 .02414 L
.09339 .04756 L
.13978 .1018 L
.14025 .10417 L
.14257 .09929 L
.14489 .11583 L
.18617 .14404 L
.23256 .18258 L
.25901 .19671 L
.26132 .19384 L
.26364 .19671 L
.27895 .2056 L
.32534 .22764 L
.37173 .25211 L
.41812 .26291 L
.46451 .27844 L
.49652 .2902 L
.49884 .2879 L
.50116 .29102 L
.5109 .29563 L
.55729 .30403 L
.60368 .31709 L
.65007 .32851 L
.69646 .33845 L
.74285 .34921 L
.78924 .35715 L
.83563 .36557 L
.88202 .37339 L
.92841 .38099 L
.97155 .39239 L
.97387 .38873 L
.97619 .39473 L
s
.4 .4 1 r
.08087 .04379 m
.08319 .0278 L
.08551 .04043 L
.09339 .05432 L
.13978 .12157 L
.14025 .12356 L
.14257 .12034 L
.14489 .12862 L
.18617 .16748 L
.23256 .20123 L
.25901 .21963 L
.26132 .21602 L
.26364 .22218 L
.27895 .23121 L
.32534 .25344 L
.37173 .27234 L
.41812 .29041 L
.46451 .30659 L
.49652 .31767 L
.49884 .31516 L
.50116 .32127 L
.5109 .32256 L
.55729 .33373 L
.60368 .34586 L
.65007 .35673 L
.69646 .36704 L
.74285 .37663 L
.78924 .38538 L
.83563 .39346 L
.88202 .40185 L
.92841 .40886 L
.97155 .41867 L
.97387 .41529 L
.97619 .42061 L
s
.4 .8 .4 r
.08087 .05213 m
.08319 .04899 L
.08551 .06237 L
.09339 .07652 L
.13978 .14887 L
.14025 .15024 L
.14257 .14624 L
.14489 .15538 L
.18617 .19608 L
.23256 .23093 L
.25901 .24935 L
.26132 .24643 L
.26364 .25387 L
.27895 .26108 L
.32534 .2841 L
.37173 .30388 L
.41812 .32181 L
.46451 .33753 L
.49652 .34927 L
.49884 .34625 L
.50116 .35177 L
.5109 .35326 L
.55729 .36513 L
.60368 .37681 L
.65007 .38815 L
.69646 .39828 L
.74285 .40874 L
.78924 .41717 L
.83563 .42572 L
.88202 .43398 L
.92841 .44116 L
.97155 .45042 L
.97387 .44706 L
.97619 .45189 L
s
.8 .4 .8 r
.08087 .16343 m
.08319 .1675 L
.08551 .17446 L
.09339 .19193 L
.13978 .2675 L
.14025 .26808 L
.14257 .27271 L
.14489 .27375 L
.18617 .32492 L
.23256 .37442 L
.25901 .39741 L
.26132 .3953 L
.26364 .40013 L
.27895 .41095 L
.32534 .43552 L
.37173 .45667 L
.41812 .47464 L
.46451 .49099 L
.49652 .50125 L
.49884 .50082 L
.50116 .50267 L
.5109 .50554 L
.55729 .51881 L
.60368 .53093 L
.65007 .54215 L
.69646 .55255 L
.74285 .56228 L
.78924 .57139 L
.83563 .57996 L
.88202 .58809 L
.92841 .59576 L
.97155 .60261 L
.97387 .60186 L
.97619 .60332 L
s
MathSubEnd
P
% End of sub-graphic
0 g
.22381 .01688 m
.22381 .08355 L
.89048 .08355 L
.89048 .01688 L
F
1 g
.22381 .01688 m
.22381 .08355 L
.89048 .08355 L
.89048 .01688 L
F
0 g
.001 w
[ ] 0 setdash
.22381 .01688 m
.89048 .01688 L
.89048 .08355 L
.22381 .08355 L
.22381 .01688 L
s
% Start of sub-graphic
p
0.22381 0.0168814 0.890476 0.083548 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0206612 0.206612 0.00423729 0.0423729 [
[ 0 0 0 0 ]
[ 1 .1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .1 L
0 .1 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0371901 0.00762712 0.243802 0.05 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.5 0.5 0.5 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
1 .4 .4 r
.21371 w
[ ] 0 setdash
0 .5 m
1 .5 L
s
MathSubEnd
P
% End of sub-graphic
0 g
gsave
.1405 .05339 -100.688 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
76.312 17.000 moveto
(Threads) show
138.375 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
% Start of sub-graphic
p
0.27686 0.00762712 0.483471 0.05 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.5 0.5 0.5 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
.4 .4 1 r
.21371 w
[ ] 0 setdash
0 .5 m
1 .5 L
s
MathSubEnd
P
% End of sub-graphic
0 g
gsave
.38017 .05339 -100.688 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
76.312 17.000 moveto
(Threads) show
138.375 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
% Start of sub-graphic
p
0.516529 0.00762712 0.72314 0.05 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.5 0.5 0.5 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
.4 .8 .4 r
.21371 w
[ ] 0 setdash
0 .5 m
1 .5 L
s
MathSubEnd
P
% End of sub-graphic
0 g
gsave
.61983 .05339 -96.25 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
76.312 17.000 moveto
(Thread) show
129.500 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
% Start of sub-graphic
p
0.756198 0.00762712 0.96281 0.05 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.5 0.5 0.5 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
.8 .4 .8 r
.21371 w
[ ] 0 setdash
0 .5 m
1 .5 L
s
MathSubEnd
P
% End of sub-graphic
0 g
gsave
.8595 .05339 -80.75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4.2) show
85.188 17.000 moveto
(.1) show
98.500 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
MathSubEnd
P
% End of sub-graphic
0 0 m
1 0 L
1 .70902 L
0 .70902 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[54]:=",
  ImageSize->{477.625, 338.625},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000;<0001nb000`400?l00000o`00003oo`3ooooo
0?ooolh0oooo003o0?ooool0oooocP3oool00?l0ooooo`3oooo>0?ooo`00o`3ooooo0?ooolh0oooo
003o0?ooool0oooocP3oool00?l0ooooo`3oooo>0?ooo`00o`3ooooo0?ooolh0oooo003o0?ooool0
oooocP3oool00?l0ooooo`3oooo>0?ooo`00o`3ooooo0?ooolh0oooo003o0?ooool0oooocP3oool0
0?l0ooooo`3oooo>0?ooo`00X03ooooo00000=h00000C`3oool00:00oooo00<000000?ooo`3oool0
o`3ooooJ0?ooo`030000003oool0oooo04d0oooo002P0?ooo`030000003oool0oooo0?l0oooofP3o
ool00`000000oooo0?ooo`1=0?ooo`00X03oool00`000000oooo0?ooo`3o0?ooomX0oooo00<00000
0?ooo`3oool0C@3oool00:00oooo00<000000?ooo`3oool0o`3ooooJ0?ooo`030000003oool0oooo
04d0oooo002P0?ooo`030000003oool0oooo0?l0oooofP3oool00`000000oooo0?ooo`1=0?ooo`00
X03oool00`000000oooo0?ooo`3o0?ooomX0oooo00<000000?ooo`3oool0C@3oool00:00oooo00<0
00000?ooo`3oool0o`3ooooJ0?ooo`030000003oool0oooo04d0oooo002P0?ooo`030000003oool0
oooo0?l0oooofP3oool00`000000oooo0?ooo`1=0?ooo`00X03oool00`000000oooo0?ooo`3o0?oo
omX0oooo00<000000?ooo`3oool0C@3oool00:00oooo00<000000?ooo`3oool0o`3ooooJ0?ooo`03
0000003oool0oooo04d0oooo002P0?ooo`030000003oool0oooo0?l0oooofP3oool00`000000oooo
0?ooo`1=0?ooo`00X03oool00`000000oooo0?ooo`0f0?oooa@0ofIVGP3ooolD06IVoeh0oooo501V
c6IN0?oooa@0c6K<>@3oool00`000000oooo0?ooo`1=0?ooo`00X03oool00`000000oooo0?ooo`0f
0?oooa@0ofIVGP3ooolD06IVoeh0oooo501Vc6IN0?oooa@0c6K<>@3oool00`000000oooo0?ooo`1=
0?ooo`00X03oool00`000000oooo0?ooo`0f0?oooa@0ofIVGP3ooolD06IVoeh0oooo501Vc6IN0?oo
oa@0c6K<>@3oool00`000000oooo0?ooo`1=0?ooo`00X03oool00`000000oooo0?ooo`0f0?oooa@0
ofIVGP3ooolD06IVoeh0oooo501Vc6IN0?oooa@0c6K<>@3oool00`000000oooo0?ooo`1=0?ooo`00
X03oool00`000000oooo0?ooo`3o0?ooomX0oooo00<000000?ooo`3oool0C@3oool00:00oooo00<0
00000?ooo`3oool0o`3ooooJ0?ooo`030000003oool0oooo04d0oooo002P0?ooo`030000003oool0
oooo0?l0oooofP3oool00`000000oooo0?ooo`1=0?ooo`00X03oool00`000000oooo0?ooo`3o0?oo
omX0oooo00<000000?ooo`3oool0C@3oool00:00oooo00<000000?ooo`3oool0o`3ooooJ0?ooo`03
0000003oool0oooo04d0oooo002P0?ooo`030000003oool0oooo0?l0oooofP3oool00`000000oooo
0?ooo`1=0?ooo`00X03oool00`000000oooo0?ooo`3o0?ooomX0oooo00<000000?ooo`3oool0C@3o
ool00:00oooo00<000000?ooo`3oool0o`3ooooJ0?ooo`030000003oool0oooo04d0oooo002P0?oo
o`030000003oool0oooo0?l0oooofP3oool00`000000oooo0?ooo`1=0?ooo`00X03oool00`000000
oooo0?ooo`3o0?ooomX0oooo00<000000?ooo`3oool0C@3oool00:00oooo00<000000?ooo`3oool0
o`3ooooJ0?ooo`030000003oool0oooo04d0oooo002P0?ooo`030000003oool0oooo0?l0oooofP3o
ool00`000000oooo0?ooo`1=0?ooo`00X03oool00`000000oooo0?ooo`3o0?ooomX0oooo00<00000
0?ooo`3oool0C@3oool00:00oooo00<000000?ooo`3oool0o`3ooooJ0?ooo`030000003oool0oooo
04d0oooo002P0?ooo`030000003oool0oooo0?l0oooofP3oool00`000000oooo0?ooo`1=0?ooo`00
X03oool00`000000oooo0?ooo`3o0?ooomX0oooo00<000000?ooo`3oool0C@3oool00:00oooo00<0
00000?ooo`3oool0o`3ooooJ0?ooo`030000003oool0oooo04d0oooo002P0?ooo`030000003oool0
oooo0?l0oooofP3oool00`000000oooo0?ooo`1=0?ooo`00X03oool00`000000oooo0?ooo`0>0?oo
o`<00000403oool3000000P0oooo0`0000050?ooo`<00000103oool3000000P0oooo1@0000070?oo
o`D000000P3oool3000000@0oooo100000020?ooo`<000001P3oool5000000H0oooo2`00000>0?oo
o`<00000203oool3000000D0oooo0`0000040?ooo`<00000203oool5000000L0oooo1@0000020?oo
o`<00000103oool400000080oooo0`0000060?ooo`D000004P3oool300000140oooo0`0000080?oo
o`<000001@3oool3000000@0oooo0`0000080?ooo`D000001`3oool500000080oooo0`0000040?oo
o`@000000P3oool3000002l0oooo0`0000040?ooo`<00000103oool;00000080oooo0`0000090?oo
o`<00000;03oool00`000000oooo0?ooo`1=0?ooo`00X03oool00`000000oooo0?ooo`0>0?ooo`<0
0000403oool3000000P0oooo0`0000050?ooo`<00000103oool3000000L0oooo200000040?ooo`X0
0000103oool:000000@0oooo2@0000040?ooo`/000003P3oool3000000P0oooo0`0000050?ooo`<0
0000103oool3000000L0oooo200000040?ooo`X00000103oool:000000@0oooo2@00000@0?ooo`<0
00004@3oool3000000P0oooo0`0000050?ooo`<00000103oool3000000L0oooo200000040?ooo`X0
0000103oool:000002l0oooo0`0000040?ooo`<00000103oool;00000080oooo0`0000090?ooo`<0
0000;03oool00`000000oooo0?ooo`1=0?ooo`00X03oool00`000000oooo0?ooo`0>0?ooo`<00000
403oool3000000P0oooo0`0000050?ooo`<00000103oool3000000H0oooo2@0000030?ooo`@00000
0`3oool4000000<0oooo2`0000040?ooo`<00000103oool3000000@0oooo2P00000>0?ooo`<00000
203oool3000000D0oooo0`0000040?ooo`<000001P3oool9000000<0oooo100000030?ooo`@00000
0`3oool;000000@0oooo0`0000040?ooo`<000003`3oool300000140oooo0`0000080?ooo`<00000
1@3oool3000000@0oooo0`0000060?ooo`T000000`3oool4000000<0oooo100000030?ooo`/00000
;`3oool3000000@0oooo0`0000050?ooo`X000000P3oool3000000T0oooo0`00000/0?ooo`030000
003oool0oooo04d0oooo002P0?ooo`030000003oool0oooo00L0oooo3000000>0?ooo`<00000203o
ool3000000D0oooo0`0000040?ooo`<000001P3oool3000000@0oooo0`0000020?ooo`<000001@3o
ool300000080oooo100000040?ooo`@000000`3oool3000000D0oooo0`0000040?ooo`@00000503o
ool3000000P0oooo0`0000050?ooo`<00000103oool3000000H0oooo0`0000040?ooo`<000000P3o
ool3000000D0oooo0`0000020?ooo`@00000103oool4000000<0oooo0`0000050?ooo`<000003`3o
ool300000140oooo0`0000080?ooo`<000001@3oool3000000@0oooo0`0000060?ooo`<00000103o
ool300000080oooo0`0000050?ooo`<000000P3oool4000000@0oooo1000000X0?ooo``000002P3o
ool4000001@0oooo0`00000/0?ooo`030000003oool0oooo04d0oooo002P0?ooo`030000003oool0
oooo00L0oooo3000000>0?ooo`<00000203oool3000000D0oooo0`0000040?ooo`<000001@3oool3
000000X0oooo0`0000050?ooo`<000000P3oool3000000H0oooo0`00000:0?ooo`@000001@3oool4
000001<0oooo0`0000080?ooo`<000001@3oool3000000@0oooo0`0000050?ooo`<000002P3oool3
000000D0oooo0`0000020?ooo`<000001P3oool3000000X0oooo1000000?0?ooo`<000004@3oool3
000000P0oooo0`0000050?ooo`<00000103oool3000000D0oooo0`00000:0?ooo`<000001@3oool3
00000080oooo0`0000060?ooo`<00000:03oool<000000/0oooo1000000C0?ooo`<00000;03oool0
0`000000oooo0?ooo`1=0?ooo`00X03oool00`000000oooo0?ooo`070?ooo``000003P3oool30000
00P0oooo0`0000050?ooo`<00000103oool3000000D0oooo0`00000;0?ooo`D000000P3oool30000
0080oooo0`0000060?ooo`<000001`3oool7000000H0oooo1000000B0?ooo`<00000203oool30000
00D0oooo0`0000040?ooo`<000001@3oool3000000/0oooo1@0000020?ooo`<000000P3oool30000
00H0oooo0`0000070?ooo`L000003`3oool300000140oooo0`0000080?ooo`<000001@3oool30000
00@0oooo0`0000050?ooo`<000002`3oool500000080oooo0`0000020?ooo`<000001P3oool30000
02P0oooo3000000<0?ooo`@000004P3oool3000002`0oooo00<000000?ooo`3oool0C@3oool00:00
oooo00<000000?ooo`3oool01`3oool3000000@0oooo0`00000@0?ooo`<00000203oool3000000D0
oooo0`0000040?ooo`<000001@3oool;000000@0oooo2@0000020?ooo`<000001P3oool3000000@0
oooo2@0000080?ooo`@000004@3oool3000000P0oooo0`0000050?ooo`<00000103oool3000000D0
oooo2`0000040?ooo`T000000P3oool3000000H0oooo0`0000040?ooo`T00000403oool300000140
oooo0`0000080?ooo`<000001@3oool3000000@0oooo0`0000050?ooo`/00000103oool900000080
oooo0`0000060?ooo`<00000:03oool3000000@0oooo0`00000?0?ooo`@000004@3oool3000002`0
oooo00<000000?ooo`3oool0C@3oool00:00oooo00<000000?ooo`3oool0203oool3000000<0oooo
0`00000@0?ooo`<00000203oool3000000D0oooo0`0000040?ooo`<000001@3oool;000000L0oooo
1P0000020?ooo`<000001P3oool3000000<0oooo2000000;0?ooo`@00000403oool3000000P0oooo
0`0000050?ooo`<00000103oool3000000D0oooo2`0000070?ooo`H000000P3oool3000000H0oooo
0`0000030?ooo`P000004P3oool300000140oooo0`0000080?ooo`<000001@3oool3000000@0oooo
0`0000050?ooo`/000001`3oool600000080oooo0`0000060?ooo`<00000:@3oool3000000<0oooo
0`00000@0?ooo`@00000403oool3000002`0oooo00<000000?ooo`3oool0C@3oool00:00oooo00<0
00000?ooo`3oool02@3oool300000080oooo0`00000@0?ooo`<00000203oool3000000D0oooo0`00
00040?ooo`<000001@3oool3000000D0oooo0`00000:0?ooo`<000000P3oool3000000H0oooo0`00
00030?ooo`D000003`3oool4000000l0oooo0`0000080?ooo`<000001@3oool3000000@0oooo0`00
00050?ooo`<000001@3oool3000000X0oooo0`0000020?ooo`<000001P3oool3000000<0oooo1@00
000E0?ooo`<000004@3oool3000000P0oooo0`0000050?ooo`<00000103oool3000000D0oooo0`00
00050?ooo`<000002P3oool300000080oooo0`0000060?ooo`<00000:P3oool300000080oooo0`00
000A0?ooo`@000003`3oool3000002`0oooo00<000000?ooo`3oool0C@3oool00:00oooo00<00000
0?ooo`3oool02@3oool300000080oooo0`00000@0?ooo`<00000203oool4000000<0oooo10000004
0?ooo`@00000103oool4000000<0oooo0`0000030?ooo`<000001@3oool3000000<0oooo0`000004
0?ooo`@000000`3oool3000000D0oooo0`00000:0?ooo`<000003`3oool3000000P0oooo10000003
0?ooo`@00000103oool4000000@0oooo100000030?ooo`<000000`3oool3000000D0oooo0`000003
0?ooo`<00000103oool4000000<0oooo0`0000050?ooo`<000003`3oool300000140oooo0`000008
0?ooo`@000000`3oool4000000@0oooo100000040?ooo`@000000`3oool3000000<0oooo0`000005
0?ooo`<000000`3oool3000000@0oooo1000000Z0?ooo`<000000P3oool300000180oooo0`00000?
0?ooo`<00000;03oool00`000000oooo0?ooo`1=0?ooo`00X03oool00`000000oooo0?ooo`0:0?oo
o`<0000000@0oooo0000000000000000403oool3000000P0oooo2`0000040?ooo`L000000P3oool9
000000@0oooo0`0000030?ooo`@000000`3oool;000000<0oooo0`0000040?ooo`<00000303oool3
000000h0oooo0`0000080?ooo`/00000103oool700000080oooo2@0000040?ooo`<000000`3oool4
000000<0oooo2`0000030?ooo`<00000103oool3000000`0oooo00@000000?ooo`3oool0oooo0`00
000A0?ooo`<00000203oool;000000@0oooo1`0000020?ooo`T00000103oool3000000<0oooo1000
00030?ooo`/00000:`3oool3000000040?ooo`0000000000000001<0oooo0`00000:0?ooo`040000
003oool0oooo0?ooo`<00000;03oool00`000000oooo0?ooo`1=0?ooo`00X03oool00`000000oooo
0?ooo`0;0?ooo`H00000403oool3000000P0oooo2P0000050?ooo`L000000`3oool7000000D0oooo
2@0000050?ooo`X00000103oool9000000@0oooo0`0000050?ooo`<000003P3oool3000000P0oooo
2P0000050?ooo`L000000`3oool7000000D0oooo2@0000050?ooo`X00000103oool9000000`0oooo
0`000000103oool0000000000000000A0?ooo`<00000203oool:000000D0oooo1`0000030?ooo`L0
00001@3oool9000000D0oooo2P00000/0?ooo`H000002`3oool3000000D0oooo0`00000:0?ooo`<0
000000@0oooo0000000000000000;03oool00`000000oooo0?ooo`1=0?ooo`00X03oool00`000000
oooo0?ooo`0;0?ooo`H00000403oool3000000P0oooo0`0000020?ooo`@000001P3oool300000004
0?ooo`0000000000000000@0oooo1@0000080?ooo`H000001`3oool400000080oooo0`0000050?oo
o`H000001P3oool3000000D0oooo0`00000>0?ooo`<00000203oool300000080oooo100000060?oo
o`<0000000@0oooo0000000000000000103oool5000000P0oooo1P0000070?ooo`@000000P3oool3
000000D0oooo1P00000>0?ooo`L000004@3oool3000000P0oooo0`0000020?ooo`@000001P3oool3
000000040?ooo`0000000000000000@0oooo1@0000080?ooo`H000001`3oool400000080oooo0`00
000/0?ooo`H000002`3oool3000000D0oooo0`00000:0?ooo`L00000;03oool00`000000oooo0?oo
o`1=0?ooo`00X03ooooo00000=h00000C`3oool00?l0ooooo`3oooo>0?ooo`00o`3ooooo0?ooolh0
oooo003o0?ooool0oooocP3oool00?l0ooooo`3oooo>0?ooo`00o`3ooooo0?ooolh0oooo003o0?oo
ool0oooocP3oool00?l0ooooo`3oooo>0?ooo`00o`3ooooo0?ooolh0oooo003o0?ooool0oooocP3o
ool00?l0ooooo`3oooo>0?ooo`00o`3ooooo0?ooolh0oooo003o0?ooool0oooocP3oool00?l0oooo
o`3oooo>0?ooo`00o`3ooooo0?ooolh0oooo003o0?ooool0oooocP3oool00?l0ooooo`3oooo>0?oo
o`00o`3ooooo0?ooolh0oooo003o0?ooool0oooocP3oool00?l0ooooo`3oooo>0?ooo`00o`3ooooo
0?ooolh0oooo003o0?ooool0oooocP3oool00?l0ooooo`3oooo>0?ooo`00o`3ooooo0?ooolh0oooo
003o0?ooool0oooocP3oool00?l0ooooo`3oooo>0?ooo`00o`3ooooo0?ooolh0oooo003o0?ooool0
oooocP3oool00?l0ooooo`3oooo>0?ooo`00o`3ooooo0?ooolh0oooo003o0?ooool0oooocP3oool0
0?l0ooooo`3oooo>0?ooo`00o`3ooooo0?ooolh0oooo003o0?ooool0oooocP3oool00?l0ooooo`3o
ooo>0?ooo`00o`3ooooo0?ooolh0oooo003o0?ooool0oooocP3oool00?l0ooooo`3oooo>0?ooo`00
o`3ooooo0?ooolh0oooo003o0?ooool0oooocP3oool00?l0ooooo`3oooo>0?ooo`00o`3ooooo0?oo
olh0oooo003o0?ooool0oooocP3oool00?l0ooooo`3oooo>0?ooo`00o`3ooooo0?ooolh0oooo003o
0?ooool0oooocP3oool00?l0ooooo`3oooo>0?ooo`00o`3ooooo0?ooolh0oooo003o0?ooolh0oooo
1`00003h0?ooo`00o`3oooo=0?ooo`X00000mP3oool00?l0ooooc03oool;00000?H0oooo003o0?oo
ol`0oooo0`0000050?ooo`@00000m@3oool00?l0ooooe@3oool300000?D0oooo003o0?ooodT0oooo
0`00000:0?ooo`D00000203oool5000000L0oooo100000060?ooo`D000001`3oool3000001X0oooo
300000050?ooo`D000001P3oool3000000D0oooo0`0000060?ooo`@000000P3oool3000000D0oooo
100000030?ooo`<000001@3oool300000=h0oooo003o0?ooodP0oooo1@0000080?ooo`P000001@3o
ool8000000@0oooo1@0000040?ooo`T000001@3oool3000001X0oooo300000040?ooo`P00000103o
ool3000000D0oooo0`0000050?ooo`X00000103oool5000000<0oooo0`0000050?ooo`<00000gP3o
ool00?l0ooooB03oool5000000L0oooo2@0000040?ooo`T00000103oool5000000<0oooo2`000004
0?ooo`<000006P3oool<000000<0oooo2@0000040?ooo`<000001@3oool3000000@0oooo2`000004
0?ooo`D000000`3oool3000000D0oooo0`00003N0?ooo`00o`3ooom80?ooo`D000001`3oool30000
00@0oooo0`0000020?ooo`@000000`3oool4000000<0oooo0`0000050?ooo`@000000`3oool40000
00@0oooo0`00000J0?ooo`<00000303oool3000000@0oooo0`0000030?ooo`<000001@3oool30000
00@0oooo0`0000040?ooo`@00000103oool3000000D0oooo0`0000050?ooo`<00000gP3oool00?l0
ooooA`3oool3000000040?ooo`0000000000000000D0oooo0`00000:0?ooo`<000001@3oool20000
00@0oooo0`0000040?ooo`@000001@3oool4000000<0oooo0`00000J0?ooo`<000002`3oool30000
00/0oooo0`0000050?ooo`<000000`3oool3000000H0oooo0`0000040?ooo`<000001@3oool30000
00D0oooo0`00003N0?ooo`00o`3ooom70?ooo`<0000000@0oooo00000000000000001@3oool30000
00X0oooo0`00000;0?ooo`<00000103oool3000000L0oooo0`0000030?ooo`<000006P3oool30000
00/0oooo0`00000;0?ooo`<000001@3oool3000000<0oooo0`0000060?ooo`<00000103oool30000
00D0oooo0`0000050?ooo`<00000gP3oool00?l0ooooAP3oool3000000<0oooo0`0000040?ooo`/0
00000P3oool3000000/0oooo0`0000040?ooo`<000001`3oool3000000<0oooo0`00000J0?ooo`<0
00002`3oool;000000<0oooo0`0000050?ooo`<000000`3oool3000000H0oooo0`0000040?ooo`<0
00001@3oool3000000D0oooo0`00003N0?ooo`00o`3ooom60?ooo`<000000`3oool3000000@0oooo
2`0000020?ooo`<000002`3oool3000000@0oooo0`0000070?ooo`<000000`3oool3000001X0oooo
0`00000;0?ooo`/000000`3oool3000000D0oooo0`0000030?ooo`<000001P3oool3000000@0oooo
0`0000050?ooo`<000001@3oool300000=h0oooo003o0?ooodD0oooo100000030?ooo`@000000`3o
ool3000000D0oooo0`0000020?ooo`<000001@3oool2000000@0oooo0`0000040?ooo`@000001@3o
ool4000000<0oooo0`00000J0?ooo`<000002`3oool3000000D0oooo0`0000030?ooo`<000001@3o
ool3000000<0oooo0`0000060?ooo`<00000103oool3000000D0oooo0`0000050?ooo`<00000gP3o
ool00?l0ooooA@3oool3000000D0oooo0`0000030?ooo`@000000`3oool3000000<0oooo10000003
0?ooo`@000000`3oool3000000D0oooo100000030?ooo`@00000103oool4000001T0oooo0`00000;
0?ooo`@000000`3oool3000000@0oooo100000030?ooo`@00000103oool3000000@0oooo10000004
0?ooo`<000001@3oool4000000<0oooo1000003N0?ooo`00o`3ooom50?ooo`<000001@3oool30000
00@0oooo2@0000040?ooo`T000000P3oool7000000<0oooo2`0000040?ooo`L000005P3oool30000
00`0oooo2@0000040?ooo`/00000103oool;00000080oooo1`0000030?ooo`/00000gP3oool00?l0
ooooA03oool3000000L0oooo0`0000040?ooo`L000001P3oool800000080oooo1`0000040?ooo`T0
00001@3oool7000001H0oooo0`00000=0?ooo`L000001@3oool:000000H0oooo2P0000020?ooo`L0
00000`3oool:00000=l0oooo003o0?oood@0oooo0`0000070?ooo`<000001@3oool5000000P0oooo
1@0000040?ooo`L000001P3oool5000000L0oooo0`000000103oool0000000000000000F0?ooo`<0
00003P3oool5000000H0oooo0`0000020?ooo`@00000203oool400000080oooo0`0000020?ooo`L0
00000`3oool300000080oooo1000003P0?ooo`00o`3ooom30?ooo`@000001`3oool4000001`0oooo
0`00000a0?ooo`<00000=`3oool3000000D0oooo0`00003V0?ooo`00o`3ooom30?ooo`<000002@3o
ool3000001`0oooo0`00000a0?ooo`<00000=`3oool3000000D0oooo0`00003V0?ooo`00o`3ooom3
0?ooo`<000002@3oool3000001d0oooo0P00000a0?ooo`<00000>03oool2000000D0oooo0`00003V
0?ooo`00o`3ooom20?ooo`<000002`3oool3000001d0oooo00<000000?ooo`3oool0;`3oool30000
03T0oooo00<000000?ooo`3oool00`3oool300000>H0oooo003o0?ooool0oooocP3oool00?l0oooo
o`3oooo>0?ooo`00o`3ooooo0?ooolh0oooo003o0?ooool0oooocP3oool00?l0ooooo`3oooo>0?oo
o`00o`3ooooo0?ooolh0oooo003o0?ooool0oooocP3oool00?l0ooooo`3oooo>0?ooo`00o`3ooooo
0?ooolh0oooo003o0?ooool0oooocP3oool00<D0oooo1@00001C0?ooo`D00000203oool5000000P0
oooo1@0000120?ooo`<00000203oool5000000P0oooo1@0000080?ooo`D00000>`3oool3000000P0
oooo1@0000080?ooo`D00000203oool5000003H0oooo2`0000050?ooo`D00000203oool5000000P0
oooo1@00001J0?ooo`00a03oool700000540oooo1`0000060?ooo`L000001P3oool700000440oooo
0`0000070?ooo`L000001P3oool7000000H0oooo1`00000j0?ooo`<000001`3oool7000000H0oooo
1`0000060?ooo`L00000=@3oool;000000@0oooo1`0000060?ooo`L000001P3oool7000005T0oooo
00330?ooo`T00000C`3oool9000000@0oooo2@0000040?ooo`T00000@03oool3000000H0oooo2@00
00040?ooo`T00000103oool9000003T0oooo0`0000060?ooo`T00000103oool9000000@0oooo2@00
000e0?ooo`X000000`3oool9000000@0oooo2@0000040?ooo`T00000F03oool00<<0oooo0`000003
0?ooo`<00000CP3oool4000000<0oooo0`0000040?ooo`<000000`3oool3000000@0oooo0`000003
0?ooo`<00000@03oool3000000H0oooo0`0000030?ooo`<00000103oool3000000<0oooo0`000004
0?ooo`<000000`3oool3000003T0oooo0`0000050?ooo`@000000`3oool3000000@0oooo0`000003
0?ooo`<00000103oool3000000<0oooo0`00000e0?ooo`@000002@3oool3000000<0oooo0`000004
0?ooo`<000000`3oool3000000@0oooo0`0000030?ooo`<00000F03oool00<80oooo0`0000050?oo
o`<00000C@3oool3000000D0oooo0`0000020?ooo`<000001@3oool300000080oooo0`0000050?oo
o`<00000?`3oool3000000D0oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`0000020?oo
o`<000001@3oool3000003P0oooo0`0000050?ooo`<000001@3oool300000080oooo0`0000050?oo
o`<000000P3oool3000000D0oooo0`00000e0?ooo`@000001`3oool3000000D0oooo0`0000020?oo
o`<000001@3oool300000080oooo0`0000050?ooo`<00000E`3oool00<80oooo0`0000050?ooo`<0
0000C@3oool3000000D0oooo0`0000020?ooo`<000001@3oool300000080oooo0`0000050?ooo`<0
0000?`3oool3000000D0oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`0000020?ooo`<0
00001@3oool3000003P0oooo0`0000050?ooo`<000001@3oool300000080oooo0`0000050?ooo`<0
00000P3oool3000000D0oooo0`00000f0?ooo`@000001P3oool3000000D0oooo0`0000020?ooo`<0
00001@3oool300000080oooo0`0000050?ooo`<00000E`3oool00<80oooo0`0000050?ooo`<00000
E@3oool300000080oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`00000o0?ooo`<00000
1@3oool3000000D0oooo0`0000020?ooo`<000001@3oool300000080oooo0`0000050?ooo`<00000
>03oool3000000d0oooo0`0000020?ooo`<000001@3oool300000080oooo0`0000050?ooo`<00000
=`3oool4000000D0oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`0000020?ooo`<00000
1@3oool3000005L0oooo00320?ooo`<000001@3oool3000005D0oooo0`0000020?ooo`<000001@3o
ool300000080oooo0`0000050?ooo`<00000?`3oool3000000D0oooo0`0000050?ooo`<000000P3o
ool3000000D0oooo0`0000020?ooo`<000001@3oool3000003P0oooo0`00000=0?ooo`<000000P3o
ool3000000D0oooo0`0000020?ooo`<000001@3oool3000003P0oooo100000040?ooo`<000001@3o
ool300000080oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`00001G0?ooo`00`P3oool3
000000D0oooo0`00001=0?ooo`<00000103oool400000080oooo0`0000050?ooo`<000000P3oool3
000000D0oooo0`00000o0?ooo`<000001@3oool3000000D0oooo0`0000020?ooo`<000001@3oool3
00000080oooo0`0000050?ooo`<00000>03oool3000000D0oooo0`0000040?ooo`@000000P3oool3
000000D0oooo0`0000020?ooo`<000001@3oool3000003T0oooo100000030?ooo`<000001@3oool3
00000080oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`00001G0?ooo`00`P3oool30000
00D0oooo0`00001=0?ooo`X000000`3oool3000000D0oooo0`0000020?ooo`<000001@3oool30000
03l0oooo0`0000050?ooo`<000001@3oool300000080oooo0`0000050?ooo`<000000P3oool30000
00D0oooo0`00000h0?ooo`<000001@3oool:000000<0oooo0`0000050?ooo`<000000P3oool30000
00D0oooo0`00000j0?ooo`<000000`3oool3000000D0oooo0`0000020?ooo`<000001@3oool30000
0080oooo0`0000050?ooo`<00000E`3oool00<80oooo0`0000050?ooo`<00000CP3oool8000000@0
oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`00000k0?ooo`040000003oool0oooo0?oo
o`<000001@3oool3000000D0oooo0`0000020?ooo`<000001@3oool300000080oooo0`0000050?oo
o`<00000=03oool010000000oooo0?ooo`3oool3000000H0oooo200000040?ooo`<000001@3oool3
00000080oooo0`0000050?ooo`<00000>`3oool300000080oooo0`0000050?ooo`<000000P3oool3
000000D0oooo0`0000020?ooo`<000001@3oool3000005L0oooo00320?ooo`<000001@3oool30000
04h0oooo1`0000050?ooo`<000001@3oool300000080oooo0`0000050?ooo`<00000>`3oool30000
00040?ooo`0000000000000000D0oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`000002
0?ooo`<000001@3oool3000003@0oooo0`000000103oool000000000000000060?ooo`L000001@3o
ool3000000D0oooo0`0000020?ooo`<000001@3oool3000003<0oooo0`0000050?ooo`<000000P3o
ool3000000D0oooo0`0000020?ooo`<000001@3oool300000080oooo0`0000050?ooo`<00000E`3o
ool00<80oooo0`0000050?ooo`<00000CP3oool3000000T0oooo0`0000050?ooo`<000000P3oool3
000000D0oooo0`00000k0?ooo`L000001@3oool3000000D0oooo0`0000020?ooo`<000001@3oool3
00000080oooo0`0000050?ooo`<00000=03oool7000000H0oooo0`0000090?ooo`<000001@3oool3
00000080oooo0`0000050?ooo`<00000<`3oool3000000D0oooo0`0000020?ooo`<000001@3oool3
00000080oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`00001G0?ooo`00``3oool30000
00<0oooo0`00001?0?ooo`<000002P3oool3000000<0oooo0`0000040?ooo`<000000`3oool30000
03d0oooo1P0000060?ooo`<000000`3oool3000000@0oooo0`0000030?ooo`<00000103oool30000
00<0oooo0`00000f0?ooo`H000001P3oool3000000X0oooo0`0000030?ooo`<00000103oool30000
00<0oooo0`00000d0?ooo`@000000`3oool4000000<0oooo0`0000030?ooo`<00000103oool30000
00<0oooo0`0000040?ooo`<000000`3oool3000005P0oooo00330?ooo`T00000C`3oool9000000@0
oooo2@0000040?ooo`T00000?`3oool4000000H0oooo2@0000040?ooo`T00000103oool9000003P0
oooo100000060?ooo`T00000103oool9000000@0oooo2@00000e0?ooo`T00000103oool9000000@0
oooo2@0000040?ooo`T00000F03oool00<@0oooo1`00001A0?ooo`P000001@3oool7000000H0oooo
1`0000110?ooo`<000001`3oool7000000H0oooo1`0000060?ooo`L00000>P3oool3000000L0oooo
200000050?ooo`L000001P3oool7000003L0oooo200000050?ooo`L000001P3oool7000000H0oooo
1`00001I0?ooo`00a@3oool500000580oooo200000060?ooo`D00000203oool500000480oooo0`00
00080?ooo`D00000203oool5000000P0oooo1@00000k0?ooo`<000001`3oool8000000H0oooo1@00
00080?ooo`D00000>@3oool5000000P0oooo1@0000080?ooo`D00000203oool5000005X0oooo003o
0?ooool0oooocP3oool00?l0ooooo`3oooo>0?ooo`00o`3ooooo0?ooolh0oooo003o0?ooool0oooo
cP3oool00?l0ooooo`3oooo>0?ooo`00o`3ooooo0?ooolh0oooo003o0?ooool0oooocP3oool00;d0
ooooo`00002g000005T0oooo002m0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0
4@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0
4@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0
4P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0
4@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0
4@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0
4@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0
4@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo0P00001I0?ooo`00_@3o
ool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0680oooo00<000000?ooo`3oool0HP3o
ool00`000000oooo0?ooo`1S0?ooo`030000003oool0oooo0680oooo00<000000?ooo`3oool04P3o
ool00`000000oooo0?ooo`1G0?ooo`00_@3oool00`000000oooo0?ooo`080?ooo`0307moO`3oool0
oooo0680oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3oool0
oooo0680oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`1G0?ooo`00_@3oool00`00
0000oooo0?ooo`080?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0HP3oool00`1o
Ogl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool04P3oool00`00
0000oooo0?ooo`1G0?ooo`00O@3oool5000000D0oooo0`0000070?ooo`D00000203oool5000000X0
oooo0`00000=0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0
oooo0?ooo`1R0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0
oooo0?ooo`0B0?ooo`030000003oool0oooo00X0oooo1@0000050?ooo`<000001`3oool5000000P0
oooo1@00000:0?ooo`<000006P3oool007`0oooo1`0000040?ooo`<000001P3oool7000000H0oooo
1`0000090?ooo`<000003@3oool00`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo01L0oooo
00<0ofIV0?ooo`3oool0B03oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo06<0oooo
00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo00T0oooo
1`0000040?ooo`<000001P3oool7000000H0oooo1`0000090?ooo`<000006P3oool007/0oooo2@00
00030?ooo`<000001@3oool9000000@0oooo2@0000080?ooo`<000003@3oool00`000000oooo0?oo
o`080?ooo`0307moO`3oool0oooo01L0oooo00<0ofIV0?ooo`3oool0B03oool00`1oOgl0oooo0?oo
o`1R0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?oo
o`0B0?ooo`030000003oool0oooo00P0oooo2@0000030?ooo`<000001@3oool9000000@0oooo2@00
00080?ooo`<000006P3oool007/0oooo0`0000030?ooo`<000002`3oool3000000<0oooo0`000004
0?ooo`<000000`3oool3000000P0oooo0`00000=0?ooo`030000003oool0oooo00P0oooo00<0Ogmo
0?ooo`3oool05`3oool00`3oIVH0oooo0?ooo`180?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo
0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<00000
0?ooo`3oool0203oool3000000<0oooo0`00000;0?ooo`<000000`3oool3000000@0oooo0`000003
0?ooo`<00000203oool3000001X0oooo001j0?ooo`<000001@3oool3000000T0oooo0`0000050?oo
o`<000000P3oool3000000D0oooo0`0000070?ooo`<000003@3oool00`000000oooo0?ooo`080?oo
o`0307moO`3oool0oooo01L0oooo0P3oIVI90?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?oo
o`3oool0H`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<000000?oo
o`3oool01`3oool3000000D0oooo0`0000090?ooo`<000001@3oool300000080oooo0`0000050?oo
o`<000001`3oool3000001X0oooo001j0?ooo`<000001@3oool3000000T0oooo0`0000050?ooo`<0
00000P3oool3000000D0oooo0`0000070?ooo`<000003@3oool400000200Ogmo00<0ofIV07moO`3o
IVH0o`1oOgn=07moO`<000002@3oool3000000D0oooo0`0000090?ooo`<000001@3oool300000080
oooo0`0000050?ooo`<000001`3oool3000001X0oooo001j0?ooo`<000001@3oool3000000T0oooo
0`0000050?ooo`<000000P3oool3000000D0oooo0`0000070?ooo`<000003@3oool00`000000oooo
0?ooo`080?ooo`0307moO`3oool0oooo01H0oooo00<0ofIV0?ooo`3oIVH0B@3oool00`1oOgl0oooo
0?ooo`1R0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo
0?ooo`0B0?ooo`030000003oool0oooo00L0oooo0`0000050?ooo`<000002@3oool3000000D0oooo
0`0000020?ooo`<000001@3oool3000000L0oooo0`00000J0?ooo`00NP3oool3000000D0oooo0`00
00090?ooo`<000001@3oool300000080oooo0`0000050?ooo`<000001`3oool3000000d0oooo00<0
00000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`0F0?ooo`040?mVIP1VI_l0oooo0?mVITP0oooo
00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo0680oooo
00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`070?ooo`<000001@3oool3000000T0oooo
0`0000050?ooo`<000000P3oool3000000D0oooo0`0000070?ooo`<000006P3oool007X0oooo0`00
00050?ooo`<000002@3oool3000000D0oooo0`0000020?ooo`<000001@3oool3000000L0oooo0`00
000=0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool05P3oool0103oIVH0IVKo0?oo
o`3oIVI80?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo
0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool01`3oool3000000D0oooo
0`0000090?ooo`<000001@3oool300000080oooo0`0000050?ooo`<000001`3oool3000001X0oooo
001j0?ooo`<000001@3oool3000000T0oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`00
00070?ooo`<000003@3oool00`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo01L0oooo00<0
IVKo0?ooo`3oIVH0B03oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo06<0oooo00<0
Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo00L0oooo0`00
00050?ooo`<000002@3oool3000000D0oooo0`0000020?ooo`<000001@3oool3000000L0oooo0`00
000J0?ooo`00NP3oool3000000D0oooo0`0000090?ooo`<000001@3oool300000080oooo0`000005
0?ooo`<000000`3oool010000000oooo0?ooo`3oool3000000d0oooo00<000000?ooo`3oool0203o
ool00`1oOgl0oooo0?ooo`0G0?ooo`80IVKo00<0ofIV0?ooo`3oool0AP3oool00`1oOgl0oooo0?oo
o`1R0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?oo
o`0B0?ooo`030000003oool0oooo00L0oooo0`0000050?ooo`<000002@3oool3000000D0oooo0`00
00020?ooo`<000001@3oool3000000<0oooo00@000000?ooo`3oool0oooo0`00000J0?ooo`00NP3o
ool3000000D0oooo0`0000090?ooo`<000001@3oool300000080oooo0`0000050?ooo`<000000`3o
ool3000000040?ooo`0000000000000000d0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo
0?ooo`0F0?ooo`0506IVo`3oool0IVKo0?ooo`3oIVH0A`3oool00`1oOgl0oooo0?ooo`1R0?ooo`03
07moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`03
0000003oool0oooo00L0oooo0`0000050?ooo`<000002@3oool3000000D0oooo0`0000020?ooo`<0
00001@3oool3000000<0oooo0`000000103oool0000000000000000J0?ooo`00NP3oool3000000D0
oooo0`0000090?ooo`<000001@3oool300000080oooo0`0000050?ooo`<000000`3oool7000000d0
oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`0F0?ooo`0506IVo`3oool0IVKo0?oo
o`3oIVH0A`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?oo
o`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo00L0oooo0`0000050?oo
o`<000002@3oool3000000D0oooo0`0000020?ooo`<000001@3oool3000000<0oooo1`00000J0?oo
o`00N`3oool3000000<0oooo0`00000;0?ooo`<000000`3oool3000000@0oooo0`0000030?ooo`<0
00001@3oool6000000d0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`0F0?ooo`05
06IVo`3oool0IVKo0?ooo`3oIVH0A`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo
06<0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo
00P0oooo0`0000030?ooo`<000002`3oool3000000<0oooo0`0000040?ooo`<000000`3oool30000
00D0oooo1P00000J0?ooo`00N`3oool9000000/0oooo2@0000040?ooo`T000001`3oool4000000d0
oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`0F0?ooo`0606IVo`3oool0oooo06IV
o`3oool0ofIVAP3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo
0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo00P0oooo2@00000;
0?ooo`T00000103oool9000000L0oooo1000000J0?ooo`00O03oool7000000d0oooo1`0000060?oo
o`L000002@3oool3000000d0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`0I0?oo
o`0306IVo`3oool0ofIV04H0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`1S0?oo
o`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`090?oo
o`L000003@3oool7000000H0oooo1`0000090?ooo`<000006P3oool007d0oooo1@00000?0?ooo`D0
0000203oool5000000X0oooo0`00000=0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3o
ool05`3oool01P1Vc6H0oooo0?ooo`1VI_l0oooo0?mVITD0oooo00<0Ogmo0?ooo`3oool0HP3oool0
0`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool04P3oool0
0`000000oooo0?ooo`0:0?ooo`D000003`3oool5000000P0oooo1@00000:0?ooo`<000006P3oool0
0;d0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`0F0?ooo`80I/aV0P3oool0101V
I_l0oooo0?ooo`3oIVI40?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0H`3oool0
0`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool0E`3oool0
0;d0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`0G0?ooo`0706K<IP3oool0oooo
0?ooo`1VI_l0oooo0?mVIP140?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0H`3o
ool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool0E`3o
ool00;d0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`0G0?ooo`0306K<IP3oool0
oooo0080oooo00<0IVKo0?ooo`3oIVH0@`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0
oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0
oooo05L0oooo002m0?ooo`<00000203oool00`1oOgl0oooo0?ooo`0H0?ooo`0506K<IP3oool0oooo
0?ooo`1VI_l00P3oool00`3oIVH0oooo0?ooo`100?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo
0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0140oooo0P00001I
0?ooo`00_@3oool00`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo01P0oooo00<0I/aV0?oo
o`3oool00P3oool0101VI_l0oooo0?ooo`3oIVI10?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo
0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<00000
0?ooo`3oool0E`3oool00;d0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`0H0?oo
o`0306K<IP3oool0oooo00<0oooo00@0IVKo0?ooo`3oool0ofIV@03oool00`1oOgl0oooo0?ooo`1R
0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B
0?ooo`030000003oool0oooo05L0oooo002m0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?oo
o`3oool0603oool00`1Vc6H0oooo0?ooo`030?ooo`0506IVo`3oool0oooo0?ooo`3oIVH0?`3oool0
0`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3oool0
0`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo05L0oooo002m0?ooo`030000003oool0oooo
00P0oooo00<0Ogmo0?ooo`3oool06@3oool00`1Vc6H0oooo0?ooo`030?ooo`0406IVo`3oool0oooo
0?mVISl0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3oool0
oooo0680oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`1G0?ooo`00_@3oool00`00
0000oooo0?ooo`080?ooo`0307moO`3oool0oooo01T0oooo00<0I/aV0?ooo`3oool0103oool0101V
I_l0oooo0?ooo`3oIVHn0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0H`3oool0
0`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool0E`3oool0
0;d0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`0J0?ooo`0306K<IP3oool0oooo
00<0oooo00D0IVKo0?ooo`3oool0oooo0?mVIP0m0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo
0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<00000
0?ooo`3oool0E`3oool00;d0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`0J0?oo
o`0306K<IP3oool0oooo00@0oooo00D0IVKo0?ooo`3oool0oooo0?mVIP0l0?ooo`0307moO`3oool0
oooo0680oooo00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0
oooo0180oooo00<000000?ooo`3oool0E`3oool00;d0oooo00<000000?ooo`3oool0203oool00`1o
Ogl0oooo0?ooo`0K0?ooo`0306K<IP3oool0oooo00@0oooo00D0IVKo0?ooo`3oool0oooo0?mVIP0k
0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`1R
0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool0E`3oool00;d0oooo0`0000080?oo
o`0307moO`3oool0oooo01`0oooo00<0I/aV0?ooo`3oool00`3oool00`1VI_l0oooo0?ooo`020?oo
o`030?mVIP3oool0oooo03P0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`1S0?oo
o`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool04@3oool2000005T0oooo002m0?ooo`03
0000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0703oool00`1Vc6H0oooo0?ooo`040?ooo`05
06IVo`3oool0oooo0?ooo`3oIVH0>P3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo
06<0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo
05L0oooo002m0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool07@3oool00`1Vc6H0
oooo0?ooo`040?ooo`0506IVo`3oool0oooo0?ooo`3oIVH0>@3oool00`1oOgl0oooo0?ooo`1R0?oo
o`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?oo
o`030000003oool0oooo05L0oooo002m0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3o
ool07@3oool00`1Vc6H0oooo0?ooo`040?ooo`0306IVo`3oool0oooo0080oooo00<0ofIV0?ooo`3o
ool0=P3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3o
ool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo05L0oooo002m0?ooo`030000
003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool07P3oool00`1Vc6H0oooo0?ooo`040?ooo`0306IV
o`3oool0oooo0080oooo00<0ofIV0?ooo`3oool0=@3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307mo
O`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000
003oool0oooo05L0oooo002m0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool07`3o
ool00`1Vc6H0oooo0?ooo`040?ooo`0306IVo`3oool0oooo0080oooo00<0ofIV0?ooo`3oool0=03o
ool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3o
ool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo05L0oooo002m0?ooo`<00000203oool0
0`1oOgl0oooo0?ooo`0O0?ooo`0306K<IP3oool0oooo00@0oooo00<0IVKo0?ooo`3oool00`3oool0
0`3oIVH0oooo0?ooo`0c0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0H`3oool0
0`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0140oooo0P00001I0?ooo`00_@3oool00`00
0000oooo0?ooo`080?ooo`0307moO`3oool0oooo0200oooo00<0I/aV0?ooo`3oool0103oool00`1V
I_l0oooo0?ooo`020?ooo`030?mVIP3oool0oooo03<0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1o
Ogl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool04P3oool00`00
0000oooo0?ooo`1G0?ooo`00_@3oool00`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo0240
oooo00<0I/aV0?ooo`3oool0103oool00`1VI_l0oooo0?ooo`020?ooo`030?mVIP3oool0oooo0380
oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo0680
oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`1G0?ooo`00_@3oool00`000000oooo
0?ooo`080?ooo`0307moO`3oool0oooo0240oooo00<0I/aV0?ooo`3oool0103oool00`1VI_l0oooo
0?ooo`030?ooo`040?mVIP3oool0oooo0?mVIS00oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0
oooo0?ooo`1S0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool04P3oool00`000000
oooo0?ooo`1G0?ooo`00_@3oool00`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo0280oooo
00<0I/aV0?ooo`3oool0103oool00`1VI_l0oooo0?ooo`030?ooo`<0ofIV<03oool00`1oOgl0oooo
0?ooo`1R0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo
0?ooo`0B0?ooo`030000003oool0oooo05L0oooo002m0?ooo`<00000203oool00`1oOgl0oooo0?oo
o`0R0?ooo`0306K<IP3oool0oooo00D0oooo00<0IVKo0?ooo`3oool00`3oool20?mVIS00oooo00<0
Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo0680oooo00<0
Ogmo0?ooo`3oool04@3oool2000005T0oooo002m0?ooo`030000003oool0oooo00P0oooo00<0Ogmo
0?ooo`3oool08`3oool00`1Vc6H0oooo0?ooo`040?ooo`0306IVo`3oool0oooo00@0oooo00<0ofIV
0?ooo`3oool0;P3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo
0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo05L0oooo002m0?oo
o`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0903oool00`1Vc6H0oooo0?ooo`040?oo
o`0306IVo`3oool0oooo00@0oooo00<0ofIV0?ooo`3oool0;@3oool00`1oOgl0oooo0?ooo`1R0?oo
o`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?oo
o`030000003oool0oooo05L0oooo002m0?ooo`<00000203oool00`1oOgl0oooo0?ooo`0T0?ooo`03
06K<IP3oool0oooo00D0oooo00<0IVKo0?ooo`3oool00`3oool00`3oIVH0oooo0?ooo`0]0?ooo`03
07moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`1R0?ooo`03
07moO`3oool0oooo0140oooo0P00001I0?ooo`00_@3oool00`000000oooo0?ooo`080?ooo`0307mo
O`3oool0oooo02D0oooo00<0I/aV0?ooo`3oool0103oool00`1VI_l0oooo0?ooo`030?ooo`030?mV
IP3oool0oooo02d0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`1S0?ooo`0307mo
O`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`1G0?ooo`00_@3o
ool00`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo02H0oooo00<0I/aV0?ooo`3oool0103o
ool00`1VI_l0oooo0?ooo`020?ooo`030?mVIP3oool0oooo02d0oooo00<0Ogmo0?ooo`3oool0HP3o
ool00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool04P3o
ool00`000000oooo0?ooo`1G0?ooo`00_@3oool00`000000oooo0?ooo`080?ooo`0307moO`3oool0
oooo02H0oooo00<0I/aV0?ooo`3oool01@3oool00`1VI_l0oooo0?ooo`020?ooo`80ofIV;@3oool0
0`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3oool0
0`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo05L0oooo002m0?ooo`<00000203oool00`1o
Ogl0oooo0?ooo`0W0?ooo`0306K<IP3oool0oooo00@0oooo00@0IVKo0?ooo`3oool0IVKo0`3oool0
0`3oIVH0oooo0?ooo`0Z0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0H`3oool0
0`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0140oooo0P00001I0?ooo`00_@3oool00`00
0000oooo0?ooo`080?ooo`0307moO`3oool0oooo02L0oooo00<0I/aV0?ooo`3oool01@3oool306IV
o`@0oooo0P3oIVHZ0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0H`3oool00`1o
Ogl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool0E`3oool008X0
oooo1@0000050?ooo`<000001`3oool5000000X0oooo0`00000=0?ooo`030000003oool0oooo00P0
oooo00<0Ogmo0?ooo`3oool0:03oool00`1Vc6H0oooo0?ooo`050?ooo`80IVKo1P3oool00`3oIVH0
oooo0?ooo`0W0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0
oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool02P3oool5000000D0
oooo0`0000070?ooo`D000002P3oool3000002L0oooo00290?ooo`L00000103oool3000000H0oooo
1`0000090?ooo`<000003@3oool3000000P0oooo00<0Ogmo0?ooo`3oool0:@3oool00`1Vc6H0oooo
0?ooo`060?ooo`0306IVo`3oool0oooo00@0oooo0P3oIVHW0?ooo`0307moO`3oool0oooo0680oooo
00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0140oooo
0P00000;0?ooo`L00000103oool3000000H0oooo1`0000090?ooo`<000009`3oool008P0oooo2@00
00030?ooo`<000001@3oool9000000P0oooo0`00000=0?ooo`030000003oool0oooo00P0oooo00<0
Ogmo0?ooo`3oool0:@3oool00`1Vc6H0oooo0?ooo`060?ooo`0306IVo`3oool0oooo00H0oooo00<0
ofIV0?ooo`3oool0903oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo06<0oooo00<0
Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo00P0oooo2@00
00030?ooo`<000001@3oool9000000P0oooo0`00000W0?ooo`00R03oool3000000<0oooo0`00000;
0?ooo`<000000`3oool3000000P0oooo0`00000=0?ooo`<00000203oool00`1oOgl0oooo0?ooo`0Z
0?ooo`0306K<IP3oool0oooo00H0oooo00<0IVKo0?ooo`3oool01P3oool20?mVIR@0oooo00<0Ogmo
0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo
0?ooo`3oool04@3oool2000000X0oooo0`0000030?ooo`<000002`3oool3000000<0oooo0`000008
0?ooo`<000009`3oool008L0oooo0`0000050?ooo`<000002@3oool3000000D0oooo0`0000070?oo
o`<000003@3oool00`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo02/0oooo00<0I/aV0?oo
o`3oool01P3oool00`1VI_l0oooo0?ooo`070?ooo`030?mVIP3oool0oooo0240oooo00<0Ogmo0?oo
o`3oool0HP3oool00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?oo
o`3oool04P3oool00`000000oooo0?ooo`070?ooo`<000001@3oool3000000T0oooo0`0000050?oo
o`<000001`3oool3000002L0oooo00270?ooo`<000001@3oool3000000T0oooo0`0000050?ooo`<0
00001`3oool3000000d0oooo1000000e07moO`0306K<IP1oOgl0Ogmo00L0Ogmo00<0IVKo07moO`1o
Ogl01`1oOgl20?mVI_l0OgmoI@1oOgl3000000T0oooo0`0000050?ooo`<000002@3oool3000000D0
oooo0`0000070?ooo`<000009`3oool008L0oooo0`0000050?ooo`<000002@3oool3000000D0oooo
0`0000070?ooo`<000003@3oool00`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo02`0oooo
00<0I/aV0?ooo`3oool01`3oool00`1VI_l0oooo0?ooo`080?ooo`030?mVIP3oool0oooo01h0oooo
00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo0680oooo
00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`070?ooo`<000001@3oool3000000T0oooo
0`0000050?ooo`<000001`3oool3000002L0oooo00270?ooo`<000001@3oool3000000T0oooo0`00
00050?ooo`<000001`3oool3000000d0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?oo
o`0/0?ooo`0306K<IP3oool0oooo00P0oooo00<0IVKo0?ooo`3oool0203oool20?mVIQh0oooo00<0
Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo0680oooo00<0
Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`070?ooo`<000001@3oool3000000T0oooo0`00
00050?ooo`<000001`3oool3000002L0oooo00270?ooo`<000001@3oool3000000T0oooo0`000005
0?ooo`<000001`3oool3000000d0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`0]
0?ooo`0306K<IP3oool0oooo00P0oooo00<0IVKo0?ooo`3oool02@3oool00`3oIVH0oooo0?ooo`0K
0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`1R
0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool01`3oool3000000D0oooo0`000009
0?ooo`<000001@3oool3000000L0oooo0`00000W0?ooo`00Q`3oool3000000D0oooo0`0000090?oo
o`<000001@3oool3000000L0oooo0`00000=0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?oo
o`3oool0;P3oool0101Vc6H0oooo0?ooo`1Vc6H70?ooo`0306IVo`3oool0oooo00T0oooo00<0ofIV
0?ooo`3oool06P3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo
0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo00L0oooo0`000005
0?ooo`<000002@3oool3000000D0oooo0`0000070?ooo`<000009`3oool008L0oooo0`0000050?oo
o`<000002@3oool3000000D0oooo0`0000030?ooo`040000003oool0oooo0?ooo`<000003@3oool0
0`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo02h0oooo00@0I/aV0?ooo`1Vc6H0I/aV203o
ool00`1VI_l0oooo0?ooo`090?ooo`030?mVIP3oool0oooo01T0oooo00<0Ogmo0?ooo`3oool0HP3o
ool00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool04P3o
ool00`000000oooo0?ooo`070?ooo`<000001@3oool3000000T0oooo0`0000050?ooo`<000000`3o
ool010000000oooo0?ooo`3oool3000002L0oooo00270?ooo`<000001@3oool3000000T0oooo0`00
00050?ooo`<000000`3oool3000000040?ooo`0000000000000000d0oooo00<000000?ooo`3oool0
203oool00`1oOgl0oooo0?ooo`0_0?ooo`<0I/aV2@3oool206IVo`X0oooo0P3oIVHI0?ooo`0307mo
O`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307mo
O`3oool0oooo0180oooo00<000000?ooo`3oool01`3oool3000000D0oooo0`0000090?ooo`<00000
1@3oool3000000<0oooo0`000000103oool0000000000000000W0?ooo`00Q`3oool3000000D0oooo
0`0000090?ooo`<000001@3oool3000000<0oooo1`00000=0?ooo`030000003oool0oooo00P0oooo
00<0Ogmo0?ooo`3oool0<P3oool00`1Vc6H0oooo0?ooo`080?ooo`0306IVo`3oool0oooo00T0oooo
00<0ofIV0?ooo`3oool05P3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo06<0oooo
00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo00L0oooo
0`0000050?ooo`<000002@3oool3000000D0oooo0`0000030?ooo`L000009`3oool008P0oooo0`00
00030?ooo`<000002`3oool3000000<0oooo0`0000050?ooo`H000003@3oool00`000000oooo0?oo
o`080?ooo`0307moO`3oool0oooo0380oooo00<0I/aV0?ooo`3oool02@3oool00`1VI_l0oooo0?oo
o`090?ooo`030?mVIP3oool0oooo01D0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?oo
o`1S0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?oo
o`080?ooo`<000000`3oool3000000/0oooo0`0000030?ooo`<000001@3oool6000002L0oooo0028
0?ooo`T000002`3oool9000000L0oooo1000000=0?ooo`030000003oool0oooo00P0oooo00<0Ogmo
0?ooo`3oool0<`3oool00`1Vc6H0oooo0?ooo`090?ooo`0306IVo`3oool0oooo00T0oooo00<0ofIV
0?ooo`3oool0503oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo
0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo00P0oooo2@00000;
0?ooo`T000001`3oool4000002L0oooo00290?ooo`L000003@3oool7000000T0oooo0`00000=0?oo
o`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0=03oool00`1Vc6H0oooo0?ooo`090?oo
o`0306IVo`3oool0oooo00T0oooo00<0ofIV0?ooo`3oool04`3oool00`1oOgl0oooo0?ooo`1R0?oo
o`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?oo
o`030000003oool0oooo00T0oooo1`00000=0?ooo`L000002@3oool3000002L0oooo002:0?ooo`D0
00003`3oool5000000X0oooo0`00000=0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3o
ool0=@3oool00`1Vc6H0oooo0?ooo`090?ooo`0306IVo`3oool0oooo00T0oooo00<0ofIV0?ooo`3o
ool04P3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3o
ool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo00X0oooo1@00000?0?ooo`D0
00002P3oool3000002L0oooo002m0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0
5P3oool00`3<I/`0oooo0?ooo`0M0?ooo`0306K<IP3oool0oooo00T0oooo00<0IVKo0?ooo`3oool0
2@3oool20?mVIQ80oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`1S0?ooo`0307mo
O`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`1G0?ooo`00_@3o
ool00`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo01H0oooo00<0c6K<0?ooo`3oool07P3o
ool00`1Vc6H0oooo0?ooo`090?ooo`0306IVo`3oool0oooo00X0oooo00<0ofIV0?ooo`3oool03`3o
ool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3o
ool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo05L0oooo002m0?ooo`030000003oool0
oooo00P0oooo00<0Ogmo0?ooo`3oool05`3oool00`3<I/`0oooo0?ooo`0N0?ooo`0306K<IP3oool0
oooo00T0oooo00<0IVKo0?ooo`3oool02P3oool00`3oIVH0oooo0?ooo`0>0?ooo`0307moO`3oool0
oooo0680oooo00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0
oooo0180oooo00<000000?ooo`3oool0E`3oool00;d0oooo0`0000080?ooo`0307moO`3oool0oooo
01L0oooo00<0c6K<0?ooo`3oool07`3oool00`1Vc6H0oooo0?ooo`090?ooo`80IVKo2`3oool00`3o
IVH0oooo0?ooo`0=0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0H`3oool00`1o
Ogl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0140oooo0P00001I0?ooo`00_@3oool00`000000
oooo0?ooo`080?ooo`0307moO`3oool0oooo01P0oooo00<0c6K<0?ooo`3oool07`3oool00`1Vc6H0
oooo0?ooo`0:0?ooo`0306IVo`3oool0oooo00T0oooo00<0ofIV0?ooo`3oool0303oool00`1oOgl0
oooo0?ooo`1R0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0
oooo0?ooo`0B0?ooo`030000003oool0oooo05L0oooo002m0?ooo`030000003oool0oooo00P0oooo
00<0Ogmo0?ooo`3oool0603oool00`3<I/`0oooo0?ooo`0P0?ooo`0306K<IP3oool0oooo00X0oooo
00<0IVKo0?ooo`3oool02@3oool20?mVIP`0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo
0?ooo`1S0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo
0?ooo`1G0?ooo`00_@3oool00`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo01P0oooo00<0
c6K<0?ooo`3oool08@3oool00`1Vc6H0oooo0?ooo`0:0?ooo`80IVKo2`3oool00`3oIVH0oooo0?oo
o`090?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?oo
o`1R0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool0E`3oool00;d0oooo00<00000
0?ooo`3oool0203oool00`1oOgl0oooo0?ooo`0I0?ooo`030<aVc03oool0oooo0240oooo00<0I/aV
0?ooo`3oool02`3oool00`1VI_l0oooo0?ooo`090?ooo`030?mVIP3oool0oooo00P0oooo00<0Ogmo
0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo
0?ooo`3oool04P3oool00`000000oooo0?ooo`1G0?ooo`00_@3oool00`000000oooo0?ooo`080?oo
o`0307moO`3oool0oooo01T0oooo00<0c6K<0?ooo`3oool08P3oool00`1Vc6H0oooo0?ooo`0;0?oo
o`80IVKo2P3oool00`3oIVH0oooo0?ooo`070?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?oo
o`3oool0H`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<000000?oo
o`3oool0E`3oool00;d0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`0J0?ooo`03
0<aVc03oool0oooo0280oooo00<0I/aV0?ooo`3oool0303oool00`1VI_l0oooo0?ooo`080?ooo`80
ofIV1`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3o
ool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo05L0oooo002m0?ooo`030000
003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool06P3oool00`3<I/`0oooo0?ooo`0S0?ooo`0306K<
IP3oool0oooo00`0oooo0P1VI_l:0?ooo`80ofIV1@3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307mo
O`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000
003oool0oooo05L0oooo002m0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool06`3o
ool00`3<I/`0oooo0?ooo`0S0?ooo`0306K<IP3oool0oooo00d0oooo00<0IVKo0?ooo`3oool02@3o
ool20?mVIP<0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3o
ool0oooo0680oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`1G0?ooo`00_@3oool3
000000P0oooo00<0Ogmo0?ooo`3oool06`3oool00`3<I/`0oooo0?ooo`0T0?ooo`0306K<IP3oool0
oooo00d0oooo00<0IVKo0?ooo`3oool02P3oool20?mVIP030?ooo`1oOgl0oooo06<0oooo00<0Ogmo
0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0140oooo0P00001I
0?ooo`00_@3oool00`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo01`0oooo00<0c6K<0?oo
o`3oool0903oool00`1Vc6H0oooo0?ooo`0=0?ooo`80IVKo303oool20?mVIP80oooo00<0ofIV0?oo
o`3oool0G`3oool00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?oo
o`3oool04P3oool00`000000oooo0?ooo`1G0?ooo`00_@3oool00`000000oooo0?ooo`080?ooo`03
07moO`3oool0oooo01`0oooo00<0c6K<0?ooo`3oool09@3oool00`1Vc6H0oooo0?ooo`0>0?ooo`03
06IVo`3oool0oooo00X0oooo00D0Ogmo0?mVIP3oIVH0oooo0?mVIP1P0?ooo`0307moO`3oool0oooo
06<0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo
05L0oooo002m0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool07@3oool00`3<I/`0
oooo0?ooo`0U0?ooo`0306K<IP3oool0oooo00h0oooo0P1VI_l:0?ooo`0307moO`3oool0oooo0080
oooo0P3oIVIN0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0
oooo0?ooo`0B0?ooo`030000003oool0oooo05L0oooo002m0?ooo`030000003oool0oooo00P0oooo
00<0Ogmo0?ooo`3oool07@3oool00`3<I/`0oooo0?ooo`0V0?ooo`80I/aV403oool00`1VI_l0oooo
0?ooo`070?ooo`0307moO`3oool0oooo00@0oooo00<0ofIV0?ooo`3oool0F`3oool00`1oOgl0oooo
0?ooo`1S0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo
0?ooo`1G0?ooo`00_@3oool00`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo01h0oooo00<0
c6K<0?ooo`3oool09`3oool00`1Vc6H0oooo0?ooo`0>0?ooo`80IVKo1`3oool00`1oOgl0oooo0?oo
o`050?ooo`80ofIVF`3oool00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo0680oooo00<0
Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`1G0?ooo`00_@3oool3000000P0oooo00<0Ogmo
0?ooo`3oool07`3oool00`3<I/`0oooo0?ooo`0W0?ooo`0306K<IP3oool0oooo00l0oooo00<0IVKo
0?ooo`3oool0103oool00`1oOgl0oooo0?ooo`070?ooo`80ofIVF@3oool00`1oOgl0oooo0?ooo`1S
0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool04@3oool2000005T0oooo002m0?oo
o`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool07`3oool00`3<I/`0oooo0?ooo`0X0?oo
o`80I/aV403oool206IVo`@0oooo00<0Ogmo0?ooo`3oool02@3oool20?mVIUL0oooo00<0Ogmo0?oo
o`3oool0H`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<000000?oo
o`3oool0E`3oool00;d0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`0P0?ooo`03
0<aVc03oool0oooo02T0oooo00<0I/aV0?ooo`3oool03`3oool01@1VI_l0oooo0?ooo`3oool0Ogmo
00d0oooo0P3oIVIE0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1o
Ogl0oooo0?ooo`0B0?ooo`030000003oool0oooo05L0oooo002m0?ooo`030000003oool0oooo00P0
oooo00<0Ogmo0?ooo`3oool0803oool00`3<I/`0oooo0?ooo`0Z0?ooo`0306K<IP3oool0oooo00l0
oooo0P1VI_l00`3oool0Ogmo0?ooo`0>0?ooo`80ofIVD`3oool00`1oOgl0oooo0?ooo`1S0?ooo`03
07moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`1G0?ooo`00
_@3oool00`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo0240oooo00<0c6K<0?ooo`3oool0
:P3oool206K<IQ40oooo00<0IVKo07moO`3oool0403oool20?mVIU40oooo00<0Ogmo0?ooo`3oool0
H`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool0
E`3oool00;d0oooo0`0000080?ooo`0307moO`3oool0oooo0280oooo00<0c6K<0?ooo`3oool0:`3o
ool00`1Vc6H0oooo0?ooo`0?0?ooo`80IVKo00<0oooo06IVo`3oool03`3oool20?mVITl0oooo00<0
Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0140oooo0P00
001I0?ooo`00_@3oool00`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo0280oooo00<0c6K<
0?ooo`3oool0;03oool00`1Vc6H0oooo0?ooo`0>0?ooo`0407moO`3oool0IVKo06IVoa80oooo0P3o
IVI=0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?oo
o`0B0?ooo`030000003oool0oooo05L0oooo002m0?ooo`030000003oool0oooo00P0oooo00<0Ogmo
0?ooo`3oool08`3oool00`3<I/`0oooo0?ooo`0/0?ooo`80I/aV3P3oool01@1oOgl0oooo0?ooo`3o
ool0IVKo01<0oooo0P3oIVI;0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3o
ool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo05L0oooo002m0?ooo`<00000203oool0
0`1oOgl0oooo0?ooo`0S0?ooo`030<aVc03oool0oooo02h0oooo00<0I/aV0?ooo`3oool02`3oool0
1@1oOgl0oooo0?ooo`3oool0IVKo01D0oooo0P3oIVI90?ooo`0307moO`3oool0oooo06<0oooo00<0
Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0A0?ooo`800000F@3oool00;d0oooo00<00000
0?ooo`3oool0203oool00`1oOgl0oooo0?ooo`0T0?ooo`030<aVc03oool0oooo02h0oooo00<0I/aV
0?ooo`3oool02P3oool00`1oOgl0oooo0?ooo`020?ooo`80IVKo5@3oool20?mVITL0oooo00<0Ogmo
0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<00000
0?ooo`3oool0E`3oool00;d0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`0U0?oo
o`030<aVc03oool0oooo02h0oooo0P1Vc6H:0?ooo`0307moO`3oool0oooo00@0oooo00<0IVKo0?oo
o`3oool0503oool20?mVITD0oooo00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`1R0?oo
o`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool0E`3oool00;d0oooo00<000000?ooo`3o
ool0203oool00`1oOgl0oooo0?ooo`0U0?ooo`030<aVc03oool0oooo0300oooo00<0I/aV0?ooo`3o
ool01`3oool00`1oOgl0oooo0?ooo`050?ooo`80IVKo5P3oool20?mVIT<0oooo00<0Ogmo0?ooo`3o
ool0H`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3o
ool0E`3oool00;d0oooo0`0000080?ooo`0307moO`3oool0oooo02H0oooo00<0c6K<0?ooo`3oool0
<03oool206K<IPL0oooo00<0Ogmo0?ooo`3oool01`3oool206IVoaH0oooo0P3oIVI10?ooo`0307mo
O`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0A0?ooo`800000
F@3oool00;d0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`0W0?ooo`030<aVc03o
ool0oooo0340oooo00<0I/aV0?ooo`3oool0103oool00`1oOgl0oooo0?ooo`090?ooo`80IVKo5P3o
ool20?mVISl0oooo00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3o
ool0oooo0180oooo00<000000?ooo`3oool0E`3oool009P0oooo1@0000050?ooo`<000002@3oool3
000000`0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`0W0?ooo`030<aVc03oool0
oooo0380oooo0P1Vc6H40?ooo`0307moO`3oool0oooo00/0oooo0P1VI_lF0?ooo`80ofIV?@3oool0
0`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool04P3oool0
0`000000oooo0?ooo`0:0?ooo`D000001@3oool3000000T0oooo0`00000d0?ooo`00U`3oool70000
00@0oooo0`0000090?ooo`<00000303oool3000000P0oooo00<0Ogmo0?ooo`3oool0:03oool00`3<
I/`0oooo0?ooo`0c0?ooo`0506K<IP3oool0oooo0?ooo`1oOgl03`3oool306IVoaD0oooo00<0ofIV
0?ooo`3oool0>P3oool00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo
0?ooo`3oool04@3oool2000000/0oooo1`0000040?ooo`<000002@3oool3000003@0oooo002F0?oo
o`T000000`3oool3000000T0oooo0`00000<0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?oo
o`3oool0:03oool00`3<I/`0oooo0?ooo`0d0?ooo`80I/aV00<0oooo07moO`3oool04@3oool206IV
oa@0oooo0P3oIVHj0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1o
Ogl0oooo0?ooo`0B0?ooo`030000003oool0oooo00P0oooo2@0000030?ooo`<000002@3oool30000
03@0oooo002F0?ooo`<000000`3oool3000000l0oooo0`00000<0?ooo`<00000203oool00`1oOgl0
oooo0?ooo`0Y0?ooo`030<aVc03oool0oooo03D0oooo00<0I/aV07moO`3oool04`3oool206IVoa@0
oooo0P3oIVHh0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0
oooo0?ooo`0A0?ooo`8000002P3oool3000000<0oooo0`00000?0?ooo`<00000=03oool009D0oooo
0`0000050?ooo`<000003P3oool3000000`0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo
0?ooo`0Z0?ooo`030<aVc03oool0oooo03D0oooo0P1Vc6H00`3oool0I/aV0?ooo`0B0?ooo`<0IVKo
4`3oool20?mVISH0oooo00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307mo
O`3oool0oooo0180oooo00<000000?ooo`3oool01`3oool3000000D0oooo0`00000>0?ooo`<00000
=03oool009D0oooo0`0000050?ooo`<000003P3oool3000000`0oooo1000000d07moO`030<aVc01o
Ogl0Ogmo03L0Ogmo0P1Vc6HF07moO`80IVKo4`1oOgl20?mVI_l0Ogmo4`1oOgl3000000T0oooo0`00
00050?ooo`<000003P3oool3000003@0oooo002E0?ooo`<000001@3oool3000000h0oooo0`00000<
0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0:`3oool00`3<I/`0oooo0?ooo`0d
0?ooo`0507moO`3oool0oooo0?ooo`1Vc6H05`3oool206IVoa<0oooo0P3oIVHb0?ooo`0307moO`3o
ool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003o
ool0oooo00L0oooo0`0000050?ooo`<000003P3oool3000003@0oooo002E0?ooo`<000001@3oool3
000000h0oooo0`00000<0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0:`3oool0
0`3<I/`0oooo0?ooo`0d0?ooo`0507moO`3oool0oooo0?ooo`1Vc6H06@3oool306IVoa80oooo0`3o
IVH_0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?oo
o`0B0?ooo`030000003oool0oooo00L0oooo0`0000050?ooo`<000003P3oool3000003@0oooo002E
0?ooo`<000001@3oool3000000h0oooo0`00000<0?ooo`030000003oool0oooo00P0oooo00<0Ogmo
0?ooo`3oool0;03oool00`3<I/`0oooo0?ooo`0c0?ooo`0307moO`3oool0oooo0080oooo0P1Vc6HJ
0?ooo`80IVKo4`3oool50?mVIRX0oooo00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`1R
0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool01`3oool3000000D0oooo0`00000>
0?ooo`<00000=03oool009D0oooo0`0000050?ooo`<000003P3oool3000000`0oooo00<000000?oo
o`3oool0203oool00`1oOgl0oooo0?ooo`0]0?ooo`030<aVc03oool0oooo0380oooo00<0Ogmo0?oo
o`3oool0103oool00`1Vc6H0oooo0?ooo`0I0?ooo`80IVKo5P3oool50?mVIRD0oooo00<0Ogmo0?oo
o`3oool0H`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<000000?oo
o`3oool01`3oool3000000D0oooo0`00000>0?ooo`<00000=03oool009D0oooo0`0000050?ooo`<0
00002P3oool010000000oooo0?ooo`3oool3000000`0oooo00<000000?ooo`3oool0203oool00`1o
Ogl0oooo0?ooo`0]0?ooo`030<aVc03oool0oooo0380oooo00<0Ogmo0?ooo`3oool01@3oool206K<
IQ/0oooo0P1VI_lI0?ooo`D0ofIV803oool00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo
0680oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`070?ooo`<000001@3oool30000
00X0oooo00@000000?ooo`3oool0oooo0`00000d0?ooo`00U@3oool3000000D0oooo0`00000:0?oo
o`<0000000@0oooo0000000000000000303oool00`000000oooo0?ooo`080?ooo`0307moO`3oool0
oooo02h0oooo00<0c6K<0?ooo`3oool0<@3oool00`1oOgl0oooo0?ooo`070?ooo`80I/aV6`3oool2
06IVoa`0oooo103oIVHL0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3oool0
0`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo00L0oooo0`0000050?ooo`<000002P3oool3
000000040?ooo`0000000000000003@0oooo002E0?ooo`<000001@3oool3000000X0oooo1`00000<
0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0;P3oool00`3<I/`0oooo0?ooo`0a
0?ooo`0307moO`3oool0oooo00T0oooo0P1Vc6HK0?ooo`<0IVKo7@3oool30?mVIQT0oooo00<0Ogmo
0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<00000
0?ooo`3oool01`3oool3000000D0oooo0`00000:0?ooo`L00000=03oool009H0oooo0`0000030?oo
o`<00000303oool6000000`0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`0_0?oo
o`030<aVc03oool0oooo0300oooo00<0Ogmo0?ooo`3oool02`3oool206K<IQ`0oooo0P1VI_lN0?oo
o`<0ofIV5P3oool00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?oo
o`3oool04P3oool00`000000oooo0?ooo`080?ooo`<000000`3oool3000000`0oooo1P00000d0?oo
o`00UP3oool9000000h0oooo1000000<0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3o
ool0<03oool00`3<I/`0oooo0?ooo`0_0?ooo`0307moO`3oool0oooo00d0oooo0P1Vc6HL0?ooo`80
IVKo7`3oool20?mVIQ@0oooo00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`1R0?ooo`03
07moO`3oool0oooo0180oooo00<000000?ooo`3oool0203oool9000000h0oooo1000000d0?ooo`00
U`3oool700000100oooo0`00000<0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0
<03oool00`3<I/`0oooo0?ooo`0_0?ooo`0307moO`3oool0oooo00l0oooo0P1Vc6HL0?ooo`80IVKo
7`3oool30?mVIQ40oooo00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307mo
O`3oool0oooo0180oooo00<000000?ooo`3oool02@3oool700000100oooo0`00000d0?ooo`00V03o
ool500000140oooo0`00000<0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0<@3o
ool20<aVc2l0oooo00<0Ogmo0?ooo`3oool04@3oool206K<IQ`0oooo0`1VI_lO0?ooo`<0ofIV3P3o
ool00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool04P3o
ool00`000000oooo0?ooo`0:0?ooo`D000004@3oool3000003@0oooo002m0?ooo`030000003oool0
oooo00P0oooo00<0Ogmo0?ooo`3oool0<`3oool00`3<I/`0oooo0?ooo`0/0?ooo`0307moO`3oool0
oooo01<0oooo0P1Vc6HM0?ooo`<0IVKo7`3oool30?mVIP/0oooo00<0Ogmo0?ooo`3oool0H`3oool0
0`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool0E`3oool0
0;d0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`0d0?ooo`030<aVc03oool0oooo
02/0oooo00<0Ogmo0?ooo`3oool05@3oool206K<IQh0oooo0`1VI_lO0?ooo`<0ofIV203oool00`1o
Ogl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool04P3oool00`00
0000oooo0?ooo`1G0?ooo`00_@3oool00`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo03@0
oooo00<0c6K<0?ooo`3oool0:`3oool00`1oOgl0oooo0?ooo`0G0?ooo`80I/aV7`3oool206IVob00
oooo0`3oIVH50?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0
oooo0?ooo`0B0?ooo`030000003oool0oooo05L0oooo002m0?ooo`<00000203oool00`1oOgl0oooo
0?ooo`0e0?ooo`030<aVc03oool0oooo02X0oooo00<0Ogmo0?ooo`3oool06@3oool206K<IQl0oooo
0`1VI_lP0?ooo`80ofIV0`3oool00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo0680oooo
00<0Ogmo0?ooo`3oool04@3oool2000005T0oooo002m0?ooo`030000003oool0oooo00P0oooo00<0
Ogmo0?ooo`3oool0=P3oool00`3<I/`0oooo0?ooo`0Y0?ooo`0307moO`3oool0oooo01/0oooo0`1V
c6HO0?ooo`<0IVKo7`3oool30?mVIP0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3o
ool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo05L0oooo002m0?ooo`030000003oool0
oooo00P0oooo00<0Ogmo0?ooo`3oool0=`3oool00`3<I/`0oooo0?ooo`0X0?ooo`0307moO`3oool0
oooo01h0oooo0P1Vc6HP0?ooo`<0IVKo7`3oool30?mVIP80oooo00<0ofIV0?ooo`3oool0GP3oool0
0`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool0E`3oool0
05P0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`1A0?ooo`030000003oool0oooo
00P0oooo00<0Ogmo0?ooo`3oool0>03oool00`3<I/`0oooo0?ooo`0W0?ooo`0307moO`3oool0oooo
0200oooo0`1Vc6HP0?ooo`<0IVKo703oool00`1oOgl0oooo0?ooo`030?mVIV00oooo00<0Ogmo0?oo
o`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo05L0oooo001G0?ooo`03
0000003oool0oooo0100oooo00<000000?ooo`3oool0D03oool00`000000oooo0?ooo`080?ooo`03
07moO`3oool0oooo03T0oooo00<0c6K<0?ooo`3oool09P3oool00`1oOgl0oooo0?ooo`0S0?ooo`80
I/aV8@3oool306IVoaT0oooo00<0Ogmo0?ooo`3oool00`3oool20?mVIUh0oooo00<0Ogmo0?ooo`3o
ool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo05L0oooo001F0?ooo`030000
003oool0oooo0180oooo00<000000?ooo`3oool0C`3oool00`000000oooo0?ooo`080?ooo`0307mo
O`3oool0oooo03T0oooo00<0c6K<0?ooo`3oool09P3oool00`1oOgl0oooo0?ooo`0U0?ooo`<0I/aV
8@3oool306IVoaH0oooo00<0Ogmo0?ooo`3oool01@3oool20?mVIU`0oooo00<0Ogmo0?ooo`3oool0
HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo05L0oooo001E0?ooo`800000503o
ool200000500oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`0j0?ooo`030<aVc03o
ool0oooo02D0oooo00<0Ogmo0?ooo`3oool0:03oool206K<IR80oooo0P1VI_lD0?ooo`0307moO`3o
ool0oooo00L0oooo1@3oIVIG0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool04P3o
ool00`000000oooo0?ooo`1G0?ooo`006P3oool3000000P0oooo0`0000040?ooo`<00000103oool3
000000@0oooo0`0000060?ooo`D00000303oool3000000P0oooo1@0000070?ooo`<00000C`3oool0
0`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo03/0oooo00<0c6K<0?ooo`3oool0903oool0
0`1oOgl0oooo0?ooo`0Z0?ooo`<0I/aV8@3oool306IVoa40oooo00<0Ogmo0?ooo`3oool0303oool6
0?mVIU40oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0
oooo05L0oooo000J0?ooo`<00000203oool3000000@0oooo0`0000040?ooo`<00000103oool30000
00D0oooo2000000:0?ooo`8000001`3oool9000000H0oooo0P00001?0?ooo`030000003oool0oooo
00P0oooo00<0Ogmo0?ooo`3oool0?03oool00`3<I/`0oooo0?ooo`0S0?ooo`0307moO`3oool0oooo
02d0oooo0P1Vc6HR0?ooo`<0IVKo3P3oool00`1oOgl0oooo0?ooo`0B0?ooo`L0ofIVBP3oool00`1o
Ogl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool0E`3oool001X0
oooo0`0000080?ooo`<00000103oool3000000@0oooo0`0000040?ooo`<00000103oool9000000T0
oooo0`0000070?ooo`<00000103oool3000000D0oooo0`00001>0?ooo`<00000203oool00`1oOgl0
oooo0?ooo`0m0?ooo`030<aVc03oool0oooo0280oooo00<0Ogmo0?ooo`3oool0;`3oool306K<IR80
oooo0`1VI_l;0?ooo`0307moO`3oool0oooo01T0oooo1@3oIVI50?ooo`0307moO`3oool0oooo0680
oooo00<0Ogmo0?ooo`3oool04@3oool2000005T0oooo000J0?ooo`<00000203oool3000000@0oooo
0`0000040?ooo`<00000103oool3000000@0oooo0`0000040?ooo`<00000203oool3000000H0oooo
0`0000050?ooo`<000001@3oool3000004h0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo
0?ooo`0m0?ooo`030<aVc03oool0oooo0280oooo00<0Ogmo0?ooo`3oool0<P3oool206K<IR<0oooo
0`1VI_l80?ooo`0307moO`3oool0oooo01h0oooo0`3oIVI20?ooo`0307moO`3oool0oooo0680oooo
00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`1G0?ooo`006P3oool3000000P0oooo0`00
00040?ooo`<00000103oool3000000@0oooo0`0000030?ooo`<00000403oool3000000d0oooo1000
00050?ooo`<00000CP3oool00`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo03h0oooo00<0
c6K<0?ooo`3oool08@3oool00`1oOgl0oooo0?ooo`0d0?ooo`<0I/aV8`3oool306IVo`D0oooo00<0
Ogmo0?ooo`3oool08@3oool40?mVISh0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?oo
o`0B0?ooo`030000003oool0oooo05L0oooo000J0?ooo`<00000203oool3000000@0oooo0`000004
0?ooo`<00000103oool3000000<0oooo0`00000@0?ooo`<000002P3oool7000000D0oooo0`00001>
0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0?`3oool00`3<I/`0oooo0?ooo`0P
0?ooo`0307moO`3oool0oooo03L0oooo0P1Vc6HT0?ooo`80IVKo0`3oool00`1oOgl0oooo0?ooo`0U
0?ooo`<0ofIV>`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<00000
0?ooo`3oool0E`3oool001X0oooo0`0000080?ooo`<00000103oool3000000@0oooo0`0000040?oo
o`<000000`3oool;000000P0oooo0`0000070?ooo`T000001P3oool3000004h0oooo00<000000?oo
o`3oool0203oool00`1oOgl0oooo0?ooo`100?ooo`030<aVc03oool0oooo01l0oooo00<0Ogmo0?oo
o`3oool0>@3oool306K<IR<0oooo0`1VI_l00`1oOgl0oooo0?ooo`0X0?ooo`<0ofIV>03oool00`1o
Ogl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool0E`3oool001X0
oooo0`0000080?ooo`<00000103oool3000000@0oooo0`0000040?ooo`<000000`3oool;000000P0
oooo0`0000060?ooo`P00000203oool3000004h0oooo00<000000?ooo`3oool0203oool00`1oOgl0
oooo0?ooo`110?ooo`030<aVc03oool0oooo01h0oooo00<0Ogmo0?ooo`3oool0?03oool206K<IR@0
oooo0`1VI_l20?ooo`0306IVo`3oool0oooo02H0oooo103oIVHd0?ooo`0307moO`3oool0oooo0680
oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`1G0?ooo`006P3oool3000000P0oooo
0`0000040?ooo`<00000103oool3000000@0oooo0`0000030?ooo`<000001@3oool3000000P0oooo
0`0000060?ooo`D000002`3oool3000004h0oooo0`0000080?ooo`0307moO`3oool0oooo0480oooo
00<0c6K<0?ooo`3oool07@3oool00`1oOgl0oooo0?ooo`0n0?ooo`<0I/aV8@3oool00`1oOgl0oooo
0?ooo`0306IVob`0oooo0`3oIVHa0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0
4@3oool2000005T0oooo000J0?ooo`<00000203oool3000000@0oooo100000030?ooo`@000000`3o
ool3000000<0oooo100000030?ooo`<000002P3oool2000000H0oooo0`0000050?ooo`<000001@3o
ool3000004h0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`120?ooo`030<aVc03o
ool0oooo01d0oooo00<0Ogmo0?ooo`3oool0@@3oool206K<IQl0oooo00<0Ogmo0?ooo`3oool00`3o
ool00`1VI_l0oooo0?ooo`0/0?ooo`@0ofIV;@3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3o
ool0oooo0180oooo00<000000?ooo`3oool0E`3oool001X0oooo0`0000080?ooo`<00000103ooolA
000000@0oooo2@00000:0?ooo`8000001P3oool3000000@0oooo0`0000060?ooo`800000C`3oool0
0`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo04<0oooo00<0c6K<0?ooo`3oool0703oool0
0`1oOgl0oooo0?ooo`130?ooo`<0I/aV703oool00`1oOgl0oooo0?ooo`030?ooo`L0IVKo;03oool4
0?mVIRT0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0
oooo05L0oooo000J0?ooo`<00000203oool3000000@0oooo2@0000000`3oool00000000000040000
00H0oooo1`00000;0?ooo`<000001P3oool9000000D0oooo0`00001?0?ooo`030000003oool0oooo
00P0oooo00<0Ogmo0?ooo`3oool0A03oool00`3<I/`0oooo0?ooo`0K0?ooo`0307moO`3oool0oooo
04H0oooo0`1Vc6HI0?ooo`0307moO`3oool0oooo00X0oooo101VI_l/0?ooo`@0ofIV9@3oool00`1o
Ogl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool0E`3oool001X0
oooo0`0000080?ooo`<00000103oool3000000030?ooo`0000000000008000000`3oool4000000P0
oooo1@00000=0?ooo`8000001`3oool6000000L0oooo0P00001@0?ooo`030000003oool0oooo00P0
oooo00<0Ogmo0?ooo`3oool0A@3oool00`3<I/`0oooo0?ooo`0J0?ooo`0307moO`3oool0oooo04T0
oooo0`1Vc6HF0?ooo`0307moO`3oool0oooo00h0oooo101VI_l/0?ooo`@0ofIV8@3oool00`1oOgl0
oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool0E`3oool001X0oooo
0`00000i0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0C`3oool3000000P0oooo
00<0Ogmo0?ooo`3oool0AP3oool00`3<I/`0oooo0?ooo`0I0?ooo`0307moO`3oool0oooo04`0oooo
0P1Vc6HD0?ooo`0307moO`3oool0oooo0180oooo101VI_l/0?ooo`D0ofIV703oool00`1oOgl0oooo
0?ooo`1R0?ooo`0307moO`3oool0oooo0140oooo0P00001I0?ooo`005@3oool=000000<0oooo0`00
000_0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0D03oool00`000000oooo0?oo
o`080?ooo`0307moO`3oool0oooo04L0oooo00<0c6K<0?ooo`3oool0603oool00`1oOgl0oooo0?oo
o`1>0?ooo`<0I/aV4@3oool00`1oOgl0oooo0?ooo`0F0?ooo`@0IVKo;@3oool50?mVIQL0oooo00<0
Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo05L0oooo000E
0?ooo`d000000`3oool300000300oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`1A
0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0B03oool00`3<I/`0oooo0?ooo`0G
0?ooo`0307moO`3oool0oooo0540oooo0`1Vc6H>0?ooo`0307moO`3oool0oooo01X0oooo101VI_l^
0?ooo`H0ofIV4@3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<00000
0?ooo`3oool0E`3oool001D0oooo3@0000030?ooo`<00000U@3oool00`000000oooo0?ooo`080?oo
o`0307moO`3oool0oooo04T0oooo00<0c6K<0?ooo`3oool05P3oool00`1oOgl0oooo0?ooo`1D0?oo
o`<0I/aV2`3oool00`1oOgl0oooo0?ooo`0N0?ooo`<0IVKo<@3oool50?mVIP`0oooo00<0Ogmo0?oo
o`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo05L0oooo002m0?ooo`<0
0000203oool00`1oOgl0oooo0?ooo`190?ooo`030<aVc03oool0oooo01H0oooo00<0Ogmo0?ooo`3o
ool0E`3oool306K<IPP0oooo00<0Ogmo0?ooo`3oool08@3oool406IVoc80oooo103oIVH80?ooo`03
07moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool04@3oool2000005T0oooo002m0?ooo`030000
003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0BP3oool00`3<I/`0oooo0?ooo`0E0?ooo`0307mo
O`3oool0oooo05X0oooo0`1Vc6H50?ooo`0307moO`3oool0oooo02D0oooo0`1VI_lc0?ooo`@0ofIV
103oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool0
E`3oool00;d0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`1;0?ooo`030<aVc03o
ool0oooo01@0oooo00<0Ogmo0?ooo`3oool0G@3oool206K<IP<0oooo00<0Ogmo0?ooo`3oool0:03o
ool306IVoc@0oooo103oIVH00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<0
00000?ooo`3oool0E`3oool00;d0oooo0`0000080?ooo`0307moO`3oool0oooo04`0oooo00<0c6K<
0?ooo`3oool04`3oool00`1oOgl0oooo0?ooo`1O0?ooo`<0I/aV00<0Ogmo0?ooo`3oool0:`3oool4
06IVoc@0oooo103oIVIQ0?ooo`0307moO`3oool0oooo0140oooo0P00001I0?ooo`00_@3oool00`00
0000oooo0?ooo`080?ooo`0307moO`3oool0oooo04d0oooo00<0c6K<0?ooo`3oool04P3oool00`1o
Ogl0oooo0?ooo`1R0?ooo`<0I/aV0P3oool00`1Vc6H0oooo0?ooo`0Z0?ooo`@0IVKo<03oool0101o
Ogl0oooo0?ooo`3oool40?mVIUd0oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`1G
0?ooo`00[P3oool3000000`0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`1>0?oo
o`030<aVc03oool0oooo0140oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0306K<
IS00oooo1@1VI_l[0?ooo`0307moO`3oool0oooo00D0oooo1@3oIVIH0?ooo`0307moO`3oool0oooo
0180oooo00<000000?ooo`3oool0303oool3000004P0oooo002^0?ooo`<00000303oool3000000P0
oooo00<0Ogmo0?ooo`3oool0C`3oool00`3<I/`0oooo0?ooo`0@0?ooo`0307moO`3oool0oooo0680
oooo00<0Ogmo0?ooo`3oool00`3oool306K<IS80oooo1@1VI_lV0?ooo`0307moO`3oool0oooo00X0
oooo1@3oIVIC0?ooo`0307moO`3oool0oooo0140oooo0P00000>0?ooo`<00000B03oool00:h0oooo
0`00000<0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0D03oool00`3<I/`0oooo
0?ooo`0?0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool01P3oool406K<IS<0oooo
1@1VI_lQ0?ooo`0307moO`3oool0oooo00l0oooo1@3oIVI>0?ooo`0307moO`3oool0oooo0180oooo
00<000000?ooo`3oool0303oool3000004P0oooo002^0?ooo`<00000303oool3000000P0oooo00<0
Ogmo0?ooo`3oool0D@3oool00`3<I/`0oooo0?ooo`0>0?ooo`0307moO`3oool0oooo0680oooo00<0
Ogmo0?ooo`3oool02P3oool406K<IS@0oooo1@1VI_lL0?ooo`0307moO`3oool0oooo01@0oooo1@3o
IVI90?ooo`0307moO`3oool0oooo0140oooo0P00000>0?ooo`<00000B03oool00:h0oooo0`00000<
0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0DP3oool00`3<I/`0oooo0?ooo`0=
0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool03P3oool406K<ISD0oooo101VI_lH
0?ooo`0307moO`3oool0oooo01T0oooo1P3oIVI30?ooo`0307moO`3oool0oooo0180oooo00<00000
0?ooo`3oool0303oool3000004P0oooo002^0?ooo`<00000303ooomQ000000030<aVc00000000000
08H00000101Vc6He000000@0IVKo=P0000060?mVIU<000003P3oool3000004P0oooo002^0?ooo`<0
0000303oool00`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo05<0oooo00<0c6K<0?ooo`3o
ool0303oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo01H0oooo101Vc6He0?ooo`@0
IVKo403oool00`1oOgl0oooo0?ooo`0U0?ooo`L0ofIV=P3oool00`1oOgl0oooo0?ooo`0B0?ooo`03
0000003oool0oooo00`0oooo0`0000180?ooo`00[P3oool3000000`0oooo00<000000?ooo`3oool0
203oool00`1oOgl0oooo0?ooo`1D0?ooo`030<aVc03oool0oooo00/0oooo00<0Ogmo0?ooo`3oool0
HP3oool00`1oOgl0oooo0?ooo`0J0?ooo`@0I/aV=@3oool406IVo``0oooo00<0Ogmo0?ooo`3oool0
;03oool60?mVIS00oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0<0?ooo`<00000
B03oool00:h0oooo0`00000<0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0E@3o
ool00`3<I/`0oooo0?ooo`0:0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool07P3o
ool406K<ISD0oooo1@1VI_l70?ooo`0307moO`3oool0oooo0380oooo1@3oIVH[0?ooo`0307moO`3o
ool0oooo0180oooo00<000000?ooo`3oool0303oool3000004P0oooo002^0?ooo`<00000303oool0
0`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo05H0oooo00<0c6K<0?ooo`3oool02@3oool0
0`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0280oooo101Vc6Hf0?ooo`D0IVKo0P3oool0
0`1oOgl0oooo0?ooo`0g0?ooo`H0ofIV9@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0
oooo00`0oooo0`0000180?ooo`00ZP3oool010000000oooo0?ooo`3oool3000000`0oooo00<00000
0?ooo`3oool0203oool00`1oOgl0oooo0?ooo`1G0?ooo`030<aVc03oool0oooo00P0oooo00<0Ogmo
0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0V0?ooo`@0I/aV=`3oool506IVocd0oooo1@3oIVHP
0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool0203oool010000000oooo0?ooo`3o
ool3000004P0oooo002Z0?ooo`<0000000@0oooo0000000000000000303oool00`000000oooo0?oo
o`080?ooo`0307moO`3oool0oooo05P0oooo00<0c6K<0?ooo`3oool01`3oool00`1oOgl0oooo0?oo
o`1R0?ooo`0307moO`3oool0oooo02X0oooo101Vc6He0?ooo`0307moO`3oool0oooo00D0IVKo?@3o
ool60?mVIQX0oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`080?ooo`<0000000@0
oooo0000000000000000B03oool00:X0oooo1`00000<0?ooo`030000003oool0oooo00P0oooo00<0
Ogmo0?ooo`3oool0F@3oool00`3<I/`0oooo0?ooo`060?ooo`0307moO`3oool0oooo0680oooo00<0
Ogmo0?ooo`3oool0;P3oool406K<IS40oooo00<0Ogmo0?ooo`3oool01@3oool506IVoch0oooo1P3o
IVHD0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool0203oool7000004P0oooo002[
0?ooo`H00000303oool00`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo05X0oooo00<0c6K<
0?ooo`3oool01@3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0380oooo101Vc6H]
0?ooo`0307moO`3oool0oooo00X0oooo1@1VI_lo0?ooo`L0ofIV3@3oool00`1oOgl0oooo0?ooo`0B
0?ooo`030000003oool0oooo00T0oooo1P0000180?ooo`00[@3oool4000000`0oooo00<000000?oo
o`3oool0203oool00`1oOgl0oooo0?ooo`1K0?ooo`80c6K<1@3oool00`1oOgl0oooo0?ooo`1R0?oo
o`0307moO`3oool0oooo03H0oooo101Vc6HY0?ooo`0307moO`3oool0oooo00l0oooo1@1VI_m10?oo
o`D0ofIV203oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo00/0oooo100000180?oo
o`00[P3oool3000000`0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`1M0?ooo`03
0<aVc03oool0oooo0080oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0j0?ooo`@0
I/aV9@3oool00`1oOgl0oooo0?ooo`0D0?ooo`D0IVKo@@3oool40?mVIP@0oooo00<0Ogmo0?ooo`3o
ool04P3oool00`000000oooo0?ooo`0<0?ooo`<00000B03oool00:h0oooo0`00000<0?ooo`030000
003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0GP3oool01@3<I/`0oooo0?ooo`3oool0Ogmo06@0
oooo00<0Ogmo0?ooo`3oool0?P3oool406K<IR40oooo00<0Ogmo0?ooo`3oool06@3oool606IVocl0
oooo103oIVH00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo00`0oooo0`0000180?ooo`00
_@3oool00`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo05l0oooo00@0c6K<0?ooo`3oool0
OgmoI03oool00`1oOgl0oooo0?ooo`120?ooo`D0I/aV703oool00`1oOgl0oooo0?ooo`0O0?ooo`H0
IVKo?@3oool40?mVIPH0oooo00<0ofIV0?ooo`3oool0203oool00`000000oooo0?ooo`1G0?ooo`00
_@3oool00`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo0600oooo00<0c6K<0?ooo`1oOgl0
I03oool00`1oOgl0oooo0?ooo`170?ooo`@0I/aV603oool00`1oOgl0oooo0?ooo`0U0?ooo`L0IVKo
=P3oool0101oOgl0oooo0?ooo`3oool40?mVIP030?ooo`3oIVH0ofIV00X0oooo00<000000?ooo`3o
ool0E`3oool00;d0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`1Q0?ooo`80c6K<
I03oool00`1oOgl0oooo0?ooo`1;0?ooo`@0I/aV503oool00`1oOgl0oooo0?ooo`0/0?ooo`H0IVKo
<03oool00`1oOgl0oooo0?ooo`050?ooo`80ofIV00<0oooo0?mVIP3oool0203oool00`000000oooo
0?ooo`1G0?ooo`00_@3oool3000000P0oooo00<0Ogmo0?ooo`3oool0HP3oool0101oOgl0c6K<0?oo
o`3<I/aQ0?ooo`0307moO`3oool0oooo04l0oooo101Vc6H@0?ooo`0307moO`3oool0oooo0380oooo
1@1VI_l[0?ooo`0307moO`3oool0oooo00P0oooo00<0ofIV0?ooo`3oool01P3oool2000005T0oooo
002m0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0HP3oool0101oOgl0oooo0<aV
c03<I/aQ0?ooo`0307moO`3oool0oooo05<0oooo101Vc6H<0?ooo`0307moO`3oool0oooo03L0oooo
1P1VI_lU0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool0E`3oool00;d0oooo00<0
00000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`1R0?ooo`0507moO`3oool0oooo0?ooo`3<I/`0
H03oool00`1oOgl0oooo0?ooo`1G0?ooo`D0I/aV1`3oool00`1oOgl0oooo0?ooo`0m0?ooo`D0IVKo
803oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo05L0oooo002m0?ooo`030000003o
ool0oooo00P0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`020?ooo`030<aVc03o
ool0oooo05d0oooo00<0Ogmo0?ooo`3oool0G03oool506K<IP80oooo00<0Ogmo0?ooo`3oool0@P3o
ool606IVoaX0oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`1G0?ooo`00_@3oool0
0`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool00`3oool0
0`3<I/`0oooo0?ooo`1L0?ooo`0307moO`3oool0oooo0640oooo1@1Vc6I80?ooo`H0IVKo503oool0
0`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo05L0oooo002m0?ooo`030000003oool0oooo
00P0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`040?ooo`80c6K<G03oool00`1o
Ogl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo00D0I/aVB@3oool706IVo`d0oooo00<0Ogmo0?oo
o`3oool04P3oool00`000000oooo0?ooo`1G0?ooo`00_@3oool00`000000oooo0?ooo`080?ooo`03
07moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool01P3oool00`3<I/`0oooo0?ooo`1I0?ooo`03
07moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool01@3oool506K<IT/0oooo1P1VI_l70?ooo`03
07moO`3oool0oooo0180oooo00<000000?ooo`3oool0E`3oool00;d0oooo00<000000?ooo`3oool0
203oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo00L0oooo00<0c6K<0?ooo`3oool0
F03oool00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo00X0oooo1@1Vc6I<0?ooo`D0IVKo
0P3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo05L0oooo002m0?ooo`030000003o
ool0oooo00P0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`080?ooo`80c6K<F03o
ool00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo00l0oooo1@1Vc6I<0?ooo`@0IVKo4`3o
ool00`000000oooo0?ooo`1G0?ooo`00_@3oool3000000P0oooo00<0Ogmo0?ooo`3oool0HP3oool0
0`1oOgl0oooo0?ooo`0:0?ooo`80c6K<EP3oool00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3oool0
oooo01@0oooo1@1Vc6I90?ooo`0307moO`3oool0IVKo00@0IVKo0`3oool00`1VI_l0oooo0?ooo`07
0?ooo`800000F@3oool00;d0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`1R0?oo
o`0307moO`3oool0oooo00`0oooo0P3<I/aD0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?oo
o`3oool06@3oool506K<IT@0oooo00<0Ogmo0?ooo`3oool0103oool406IVo`X0oooo00<000000?oo
o`3oool0E`3oool00;d0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`1R0?ooo`03
07moO`3oool0oooo00h0oooo0P3<I/aB0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3o
ool07P3oool506K<ISl0oooo00<0Ogmo0?ooo`3oool0203oool00`1VI_l0oooo0?ooo`070?ooo`03
0000003oool0oooo05L0oooo002m0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0
HP3oool00`1oOgl0oooo0?ooo`0@0?ooo`80c6K<D03oool00`1oOgl0oooo0?ooo`1S0?ooo`0307mo
O`3oool0oooo02<0oooo1@1Vc6Hj0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool0
E`3oool00;d0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3o
ool0oooo0180oooo0P3<I/a>0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0:03o
ool506K<ISD0oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`1G0?ooo`00_@3oool0
0`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0503oool2
0<aVc4`0oooo00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`0]0?ooo`H0I/aV;`3oool0
0`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo05L0oooo002m0?ooo`<00000203oool00`1o
Ogl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo01H0oooo0P3<I/a:0?ooo`0307moO`3oool0oooo
06<0oooo00<0Ogmo0?ooo`3oool0<`3oool706K<IRP0oooo00<0Ogmo0?ooo`3oool04@3oool20000
05T0oooo002m0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0
oooo0?ooo`0H0?ooo`80c6K<B03oool00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo03X0
oooo1`1Vc6HQ0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool0E`3oool00;d0oooo
00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo01X0oooo
0P3<I/a60?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0@@3oool706K<IQX0oooo
00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`1G0?ooo`00_@3oool00`000000oooo0?oo
o`080?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0703oool20<aVc4@0oooo00<0
Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`180?ooo`H0I/aV503oool00`1oOgl0oooo0?oo
o`0B0?ooo`030000003oool0oooo05L0oooo002m0?ooo`030000003oool0oooo00P0oooo00<0Ogmo
0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0N0?ooo`80c6K<@P3oool00`1oOgl0oooo0?ooo`1S
0?ooo`0307moO`3oool0oooo04h0oooo1`1Vc6H=0?ooo`0307moO`3oool0oooo0180oooo00<00000
0?ooo`3oool0E`3oool00;d0oooo0`0000080?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?oo
o`3oool0803oool20<aVc400oooo00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`1E0?oo
o`H0I/aV1`3oool00`1oOgl0oooo0?ooo`0A0?ooo`800000F@3oool00;d0oooo00<000000?ooo`3o
ool0203oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0280oooo0P3<I/`n0?ooo`03
07moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0F`3oool506K<IP80oooo00<0Ogmo0?ooo`3o
ool04P3oool00`000000oooo0?ooo`1G0?ooo`00_@3oool00`000000oooo0?ooo`080?ooo`0307mo
O`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0903oool20<aVc3`0oooo00<0Ogmo0?ooo`3oool0
H`3oool00`1oOgl0oooo0?ooo`1P0?ooo`@0I/aV4`3oool00`000000oooo0?ooo`1G0?ooo`00_@3o
ool00`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool09P3o
ool30<aVc3T0oooo00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3o
ool0I/aV00@0I/aV0`3oool00`1Vc6H0oooo0?ooo`080?ooo`030000003oool0oooo05L0oooo002m
0?ooo`<00000203oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo02T0oooo0P3<I/`g
0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`04
0?ooo`@0I/aV2@3oool2000005T0oooo002m0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?oo
o`3oool0HP3oool00`1oOgl0oooo0?ooo`0[0?ooo`80c6K<=@3oool00`1oOgl0oooo0?ooo`1S0?oo
o`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0203oool00`1Vc6H0oooo0?ooo`070?oo
o`030000003oool0oooo05L0oooo002m0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3o
ool0HP3oool00`1oOgl0oooo0?ooo`0]0?ooo`80c6K<<`3oool00`1oOgl0oooo0?ooo`1S0?ooo`03
07moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`1G0?ooo`00
_@3oool3000000P0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0_0?ooo`<0c6K<
<03oool00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0
4@3oool2000005T0oooo002m0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0HP3o
ool00`1oOgl0oooo0?ooo`0b0?ooo`80c6K<;P3oool00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3o
ool0oooo0680oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`1G0?ooo`00X03oool3
000000P0oooo1@00000=0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0HP3oool0
0`1oOgl0oooo0?ooo`0d0?ooo`<0c6K<:`3oool00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3oool0
oooo0680oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0<0?ooo`<00000203oool5
000003/0oooo002P0?ooo`<000001`3oool7000000`0oooo0`0000080?ooo`0307moO`3oool0oooo
0680oooo00<0Ogmo0?ooo`3oool0=`3oool20<aVc2T0oooo00<0Ogmo0?ooo`3oool0H`3oool00`1o
Ogl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0140oooo0P00000>0?ooo`<000001`3oool70000
03X0oooo002P0?ooo`<000001P3oool9000000/0oooo00<000000?ooo`3oool0203oool00`1oOgl0
oooo0?ooo`1R0?ooo`0307moO`3oool0oooo03T0oooo0`3<I/`V0?ooo`0307moO`3oool0oooo06<0
oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo00`0
oooo0`0000060?ooo`T00000>@3oool00:00oooo0`0000060?ooo`<000000`3oool3000000/0oooo
0`0000080?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0?03oool20<aVc2@0oooo
00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0140oooo
0P00000>0?ooo`<000001P3oool3000000<0oooo0`00000i0?ooo`00X03oool3000000D0oooo0`00
00050?ooo`<000002P3oool00`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo0680oooo00<0
Ogmo0?ooo`3oool0?P3oool30<aVc240oooo00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?oo
o`1R0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool0303oool3000000D0oooo0`00
00050?ooo`<00000>03oool00:00oooo0`0000050?ooo`<000001@3oool3000000X0oooo1000002`
07moO`80c6K<o@1oOgl3000000h0oooo0`0000050?ooo`<000001@3oool3000003P0oooo002P0?oo
o`<000001@3oool3000000D0oooo0`00000:0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?oo
o`3oool0HP3oool00`1oOgl0oooo0?ooo`130?ooo`<0c6K<703oool00`1oOgl0oooo0?ooo`1S0?oo
o`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0<0?oo
o`<000001@3oool3000000D0oooo0`00000h0?ooo`00X03oool3000000D0oooo0`0000050?ooo`<0
00002P3oool00`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3o
ool0AP3oool30<aVc1T0oooo00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`1R0?ooo`03
07moO`3oool0oooo0180oooo00<000000?ooo`3oool0303oool3000000D0oooo0`0000050?ooo`<0
0000>03oool00:00oooo0`0000050?ooo`<000001@3oool3000000X0oooo00<000000?ooo`3oool0
203oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo04T0oooo0`3<I/`F0?ooo`0307mo
O`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000
003oool0oooo00`0oooo0`0000050?ooo`<000001@3oool3000003P0oooo002P0?ooo`<000001@3o
ool3000000D0oooo0`00000:0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0HP3o
ool00`1oOgl0oooo0?ooo`1<0?ooo`80c6K<503oool00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3o
ool0oooo0680oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0<0?ooo`<000001@3o
ool3000000D0oooo0`00000h0?ooo`00W03oool010000000oooo0?ooo`3oool3000000D0oooo0`00
00050?ooo`<000002P3oool00`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo0680oooo00<0
Ogmo0?ooo`3oool0CP3oool30<aVc140oooo00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?oo
o`1R0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool0203oool010000000oooo0?oo
o`3oool3000000D0oooo0`0000050?ooo`<00000>03oool009`0oooo0`000000103oool000000000
000000050?ooo`<000001@3oool3000000X0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo
0?ooo`1R0?ooo`0307moO`3oool0oooo0540oooo0`3<I/`>0?ooo`0307moO`3oool0oooo06<0oooo
00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo00P0oooo
0`000000103oool000000000000000050?ooo`<000001@3oool3000003P0oooo002L0?ooo`L00000
1@3oool3000000D0oooo0`00000:0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0
HP3oool00`1oOgl0oooo0?ooo`1D0?ooo`<0c6K<2`3oool00`1oOgl0oooo0?ooo`1S0?ooo`0307mo
O`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`080?ooo`L00000
1@3oool3000000D0oooo0`00000h0?ooo`00W@3oool6000000H0oooo0`0000030?ooo`<000002`3o
ool00`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0E`3o
ool30<aVc0P0oooo00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3o
ool0oooo0180oooo00<000000?ooo`3oool02@3oool6000000H0oooo0`0000030?ooo`<00000>@3o
ool009l0oooo100000060?ooo`T000002`3oool00`000000oooo0?ooo`080?ooo`0307moO`3oool0
oooo0680oooo00<0Ogmo0?ooo`3oool0FP3oool30<aVc0D0oooo00<0Ogmo0?ooo`3oool0H`3oool0
0`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool02`3oool4
000000H0oooo2@00000i0?ooo`00X03oool3000000L0oooo1`00000<0?ooo`030000003oool0oooo
00P0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`1M0?ooo`80c6K<0`3oool00`1o
Ogl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool04P3oool00`00
0000oooo0?ooo`0<0?ooo`<000001`3oool7000003X0oooo002P0?ooo`<00000203oool5000000d0
oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo05l0
oooo0`3<I/`00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3o
ool04P3oool00`000000oooo0?ooo`0<0?ooo`<00000203oool5000003/0oooo002m0?ooo`030000
003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`1R0?ooo`<0c6K<
0P3oool00`3<I/`0oooo0?ooo`1N0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0
4P3oool00`000000oooo0?ooo`1G0?ooo`00_@3oool00`000000oooo0?ooo`080?ooo`0307moO`3o
ool0oooo0680oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`020<aVc0040?ooo`3<
I/`0c6K<0<aVc5d0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000
003oool0oooo05L0oooo002m0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0HP3o
ool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo00H0oooo103<I/aI0?ooo`0307moO`3o
ool0oooo0680oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`1G0?ooo`00_@3oool3
000000P0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0
oooo00X0oooo0`3<I/aF0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool04@3oool2
000005T0oooo002m0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1o
Ogl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo00d0oooo103<I/aB0?ooo`0307moO`3oool0oooo
0680oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`1G0?ooo`00_@3oool00`000000
oooo0?ooo`080?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0
oooo0?ooo`0A0?ooo`<0c6K<C`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180
oooo00<000000?ooo`3oool0E`3oool00;d0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo
0?ooo`1R0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0503oool30<aVc4`0oooo
00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo05L0oooo
002m0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?oo
o`1R0?ooo`0307moO`3oool0oooo01L0oooo103<I/a80?ooo`0307moO`3oool0oooo0680oooo00<0
Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`1G0?ooo`00_@3oool00`000000oooo0?ooo`08
0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0K
0?ooo`<0c6K<A@3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<00000
0?ooo`3oool0E`3oool00;d0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`1R0?oo
o`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool07P3oool40<aVc440oooo00<0Ogmo0?oo
o`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo05L0oooo002m0?ooo`03
0000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`1R0?ooo`03
07moO`3oool0oooo0280oooo103<I/`m0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3o
ool04P3oool00`000000oooo0?ooo`1G0?ooo`00_@3oool00`000000oooo0?ooo`080?ooo`0307mo
O`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0V0?ooo`@0c6K<
>@3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool0
E`3oool00;d0oooo0`0000080?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0HP3o
ool00`1oOgl0oooo0?ooo`0Z0?ooo`@0c6K<=@3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3o
ool0oooo0140oooo0P00001I0?ooo`00_@3oool00`000000oooo0?ooo`080?ooo`0307moO`3oool0
oooo0680oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0^0?ooo`@0c6K<<@3oool0
0`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool0E`3oool0
0;d0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo
0680oooo00<0Ogmo0?ooo`3oool0<P3oool40<aVc2d0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1o
Ogl0oooo0?ooo`0B0?ooo`030000003oool0oooo05L0oooo002m0?ooo`030000003oool0oooo00P0
oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo03H0
oooo103<I/`Y0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool04P3oool00`000000
oooo0?ooo`1G0?ooo`00_@3oool00`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo0680oooo
00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0j0?ooo`@0c6K<9@3oool00`1oOgl0oooo
0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool0E`3oool00;d0oooo00<0
00000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0680oooo00<0
Ogmo0?ooo`3oool0?P3oool40<aVc240oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?oo
o`0B0?ooo`030000003oool0oooo05L0oooo002m0?ooo`<00000203oool00`1oOgl0oooo0?ooo`1R
0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0@P3oool50<aVc1`0oooo00<0Ogmo
0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0A0?ooo`800000F@3oool00;d0oooo00<000000?oo
o`3oool0203oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?oo
o`3oool0A`3oool40<aVc1P0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?oo
o`030000003oool0oooo05L0oooo002m0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3o
ool0HP3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo04/0oooo103<I/`D0?ooo`03
07moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`1G0?ooo`00
_@3oool00`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0
HP3oool00`1oOgl0oooo0?ooo`1?0?ooo`@0c6K<403oool00`1oOgl0oooo0?ooo`1R0?ooo`0307mo
O`3oool0oooo0180oooo00<000000?ooo`3oool0E`3oool00;d0oooo00<000000?ooo`3oool0203o
ool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0D`3o
ool40<aVc0`0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003o
ool0oooo05L0oooo002m0?ooo`<00000203oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0
oooo0680oooo00<0Ogmo0?ooo`3oool0E`3oool50<aVc0L0oooo00<0Ogmo0?ooo`3oool0HP3oool0
0`1oOgl0oooo0?ooo`0A0?ooo`800000F@3oool00;d0oooo00<000000?ooo`3oool0203oool00`1o
Ogl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0G03oool50<aV
c080oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo
05L0oooo002m0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0
oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0640oooo1@3<I/aR0?ooo`0307moO`3oool0oooo0180
oooo00<000000?ooo`3oool0E`3oool00;d0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo
0?ooo`1R0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo
0?ooo`050<aVc5d0oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`1G0?ooo`00_@3o
ool3000000P0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3o
ool0oooo06<0oooo00<0Ogmo0?ooo`3oool01@3oool50<aVc5P0oooo00<0Ogmo0?ooo`3oool04@3o
ool2000005T0oooo002m0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0HP3oool0
0`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool02P3oool5
0<aVc5<0oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`1G0?ooo`00_@3oool00`00
0000oooo0?ooo`080?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0HP3oool00`1o
Ogl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo00l0oooo1@3<I/a>0?ooo`0307moO`3oool0oooo
0180oooo00<000000?ooo`3oool0E`3oool00;d0oooo0`0000080?ooo`0307moO`3oool0oooo0680
oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo01@0
oooo1@3<I/a90?ooo`0307moO`3oool0oooo0140oooo0P00001I0?ooo`00_@3oool00`000000oooo
0?ooo`080?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo
0?ooo`1S0?ooo`0307moO`3oool0oooo01T0oooo1@3<I/a40?ooo`0307moO`3oool0oooo0180oooo
00<000000?ooo`3oool0E`3oool00;d0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?oo
o`1R0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?oo
o`0N0?ooo`D0c6K<?`3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo05L0oooo002C
0?ooo`<00000203oool5000000P0oooo1@00000=0?ooo`<00000203oool00`1oOgl0oooo0?ooo`1R
0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`0S
0?ooo`D0c6K<>P3oool00`1oOgl0oooo0?ooo`0A0?ooo`8000003P3oool3000000P0oooo1@000008
0?ooo`D00000;P3oool009<0oooo0`0000070?ooo`L000001P3oool7000000`0oooo00<000000?oo
o`3oool0203oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?oo
o`3oool0H`3oool00`1oOgl0oooo0?ooo`0X0?ooo`D0c6K<=@3oool00`1oOgl0oooo0?ooo`0B0?oo
o`030000003oool0oooo00`0oooo0`0000070?ooo`L000001P3oool7000002d0oooo002C0?ooo`<0
00001P3oool9000000@0oooo2@00000;0?ooo`<00000203oool00`1oOgl0oooo0?ooo`1R0?ooo`03
07moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`0]0?ooo`H0
c6K<;`3oool00`1oOgl0oooo0?ooo`0A0?ooo`8000003P3oool3000000H0oooo2@0000040?ooo`T0
0000;03oool009<0oooo0`0000060?ooo`<000000`3oool3000000@0oooo0`0000030?ooo`<00000
2`3oool00`000000oooo0?ooo`080?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0
HP3oool00`1oOgl0oooo0?ooo`1S0?ooo`0307moO`3oool0oooo03<0oooo1`3<I/`X0?ooo`0307mo
O`3oool0oooo0180oooo00<000000?ooo`3oool0303oool3000000H0oooo0`0000030?ooo`<00000
103oool3000000<0oooo0`00000/0?ooo`00T`3oool3000000D0oooo0`0000050?ooo`<000000P3o
ool3000000D0oooo0`00000:0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0HP3o
ool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0>P3o
ool70<aVc240oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0<0?ooo`<000001@3o
ool3000000D0oooo0`0000020?ooo`<000001@3oool3000002/0oooo002C0?ooo`<000001@3oool3
000000D0oooo0`0000020?ooo`<000001@3oool3000000X0oooo1000003o07moOg`0Ogmo1P3<I/`^
07moO`<000003P3oool3000000D0oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`00000[
0?ooo`00T`3oool3000000D0oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`00000:0?oo
o`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`1R0?oo
o`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0A`3oool50<aVc1H0oooo00<0Ogmo0?oo
o`3oool04P3oool00`000000oooo0?ooo`0<0?ooo`<000001@3oool3000000D0oooo0`0000020?oo
o`<000001@3oool3000002/0oooo002C0?ooo`<000001@3oool3000000D0oooo0`0000020?ooo`<0
00001@3oool3000000X0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`1R0?ooo`03
07moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`1<0?ooo`D0
c6K<4@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo00`0oooo0`0000050?ooo`<0
00001@3oool300000080oooo0`0000050?ooo`<00000:`3oool009<0oooo0`0000050?ooo`<00000
1@3oool300000080oooo0`0000050?ooo`<000002P3oool00`000000oooo0?ooo`080?ooo`0307mo
O`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`1S0?ooo`0307mo
O`3oool0oooo0540oooo1@3<I/`<0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool0
303oool3000000D0oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`00000[0?ooo`00T`3o
ool3000000D0oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`00000:0?ooo`030000003o
ool0oooo00P0oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3o
ool0oooo06<0oooo00<0Ogmo0?ooo`3oool0EP3oool60<aVc0H0oooo00<0Ogmo0?ooo`3oool04P3o
ool00`000000oooo0?ooo`0<0?ooo`<000001@3oool3000000D0oooo0`0000020?ooo`<000001@3o
ool3000002/0oooo002?0?ooo`040000003oool0oooo0?ooo`<000001@3oool3000000D0oooo0`00
00020?ooo`<000001@3oool3000000X0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?oo
o`1R0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?oo
o`1L0?ooo`H0c6K<00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`080?ooo`040000003o
ool0oooo0?ooo`<000001@3oool3000000D0oooo0`0000020?ooo`<000001@3oool3000002/0oooo
002?0?ooo`<0000000@0oooo00000000000000001@3oool3000000D0oooo0`0000020?ooo`<00000
1@3oool3000000X0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`1R0?ooo`0307mo
O`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`1R0?ooo`H0c6K<
103oool00`3<I/`0oooo0?ooo`080?ooo`030000003oool0oooo00P0oooo0`000000103oool00000
0000000000050?ooo`<000001@3oool300000080oooo0`0000050?ooo`<00000:`3oool008l0oooo
1`0000050?ooo`<000001@3oool300000080oooo0`0000050?ooo`<000002P3oool00`000000oooo
0?ooo`080?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0HP3oool00`1oOgl0oooo
0?ooo`1S0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool00`3oool40<aVc0030?oo
o`3<I/`0oooo00P0oooo00<000000?ooo`3oool0203oool7000000D0oooo0`0000050?ooo`<00000
0P3oool3000000D0oooo0`00000[0?ooo`00T03oool6000000H0oooo0`0000030?ooo`<00000103o
ool3000000<0oooo0`00000;0?ooo`030000003oool0oooo00P0oooo00<0Ogmo0?ooo`3oool0HP3o
ool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo06<0oooo00<0Ogmo0?ooo`3oool0HP3o
ool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo00T0oooo1P0000060?ooo`<000000`3o
ool3000000@0oooo0`0000030?ooo`<00000;03oool00980oooo100000060?ooo`T00000103oool9
000000/0oooo00<000000?ooo`3oool0203oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0
oooo0680oooo00<0Ogmo0?ooo`3oool0H`3oool00`1oOgl0oooo0?ooo`1R0?ooo`0307moO`3oool0
oooo0180oooo00<000000?ooo`3oool02`3oool4000000H0oooo2@0000040?ooo`T00000;03oool0
09<0oooo0`0000070?ooo`L000001P3oool7000000`0oooo00<000000?ooo`3oool0203oool00`1o
Ogl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0680oooo00<0Ogmo0?ooo`3oool0H`3oool00`1o
Ogl0oooo0?ooo`1R0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool0303oool30000
00L0oooo1`0000060?ooo`L00000;@3oool009<0oooo0`0000080?ooo`D00000203oool5000000d0
oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`1R0?ooo`030000003oool0oooo0680
oooo00<000000?ooo`3oool0H`3oool00`000000oooo0?ooo`1R0?ooo`030000003oool0oooo0180
oooo00<000000?ooo`3oool0303oool3000000P0oooo1@0000080?ooo`D00000;P3oool00;d0oooo
00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo
00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo
00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0140oooo
00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0180oooo
00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0180oooo
00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo
00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo
00<000000?ooo`3oool04@3oool2000005T0oooo002m0?ooo`030000003oool0oooo00P0oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0180oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo00<0
00000?ooo`3oool04P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0140oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0140oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0180oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo0P00
001I0?ooo`00_@3ooooo00000;L00000F@3oool00?l0ooooo`3oooo>0?ooo`00o`3ooooo0?ooolh0
oooo003o0?ooool0oooocP3oool00?l0ooooo`3oooo>0?ooo`00o`3ooooo0?ooolh0oooo003o0?oo
ool0oooocP3oool00?l0ooooo`3oooo>0?ooo`00o`3ooooo0?ooobH0oooo1000002T0?ooo`00o`3o
oooo0?ooobH0oooo1P00002R0?ooo`00o`3ooooo0?ooobH0oooo1P00002R0?ooo`00o`3ooooo0?oo
obT0oooo1000002Q0?ooo`00o`3ooooo0?ooobX0oooo0`00002Q0?ooo`00R`3oool=000000H0oooo
0`0000080?ooo`D000000P3oool300000080oooo0`0000060?ooo`@000000P3oool3000000D0oooo
1@0000020?ooo`<000000`3oool4000000<0oooo0`0000070?ooo`D000001`3oool3000000D0oooo
0`00000>0?ooo`<00000203oool3000000@0oooo0`0000040?ooo`<00000103oool3000000H0oooo
1@00000?0?ooo`D000001`3oool3000000D0oooo0`00000;0?ooo`D000000P3oool3000000/0oooo
100000090?ooo`D00000203oool5000000h0oooo0`0000040?ooo`<00000103oool3000000@0oooo
0`0000070?ooo`<000000`3oool:000000d0oooo0`00000H0?ooo`<000001@3oool3000000L0oooo
1@0000020?ooo`<000001@3oool3000001D0oooo0`0000050?ooo`@00000103oool500000080oooo
0`0000020?ooo`<000001@3oool3000000@0oooo0`0000060?ooo`@000000P3oool3000000@0oooo
0`0000040?ooo`<00000103oool3000000P0oooo2`00000W0?ooo`00R`3oool=000000H0oooo0`00
00070?ooo`X000000`3oool3000000D0oooo2P0000040?ooo`X000000`3oool5000000<0oooo0`00
00050?ooo`T000001@3oool3000000D0oooo0`00000>0?ooo`<00000203oool3000000@0oooo0`00
00040?ooo`<00000103oool3000000D0oooo2000000;0?ooo`T000001@3oool3000000D0oooo0`00
000:0?ooo`X000002P3oool8000000H0oooo1`0000060?ooo`L000003@3oool3000000@0oooo0`00
00040?ooo`<00000103oool3000000L0oooo0`0000030?ooo`X000003@3oool3000001P0oooo0`00
00050?ooo`<000001P3oool:000000D0oooo1@00000D0?ooo`<00000103oool5000000<0oooo2P00
00030?ooo`<000001@3oool3000000@0oooo0`0000050?ooo`X00000103oool3000000@0oooo0`00
00040?ooo`<00000203oool;000002L0oooo002;0?ooo`d000001@3oool5000000D0oooo10000003
0?ooo`@000000`3oool3000000@0oooo2`0000030?ooo`@000000`3oool4000000<0oooo1@000003
0?ooo`<00000103oool;000000@0oooo0`0000050?ooo`<000003P3oool3000000P0oooo0`000004
0?ooo`<00000103oool3000000@0oooo0`0000040?ooo`T000002P3oool;000000@0oooo0`000005
0?ooo`<000002@3oool4000000<0oooo1000000:0?ooo`T00000103oool9000000@0oooo2@00000<
0?ooo`<000000`3oool5000000<0oooo0`0000040?ooo`<000001`3oool3000000<0oooo2P00000=
0?ooo`<000005`3oool5000000<0oooo1@0000040?ooo`@000000`3oool4000000D0oooo1@00000D
0?ooo`<00000103oool500000080oooo100000030?ooo`@000000`3oool3000000D0oooo0`000004
0?ooo`<00000103oool;000000@0oooo0`0000040?ooo`<00000103oool3000000T0oooo2P00000W
0?ooo`00R`3oool3000000l0oooo1@0000050?ooo`<000001@3oool3000000<0oooo0`0000040?oo
o`@000000`3oool4000000<0oooo0`0000050?ooo`<000000`3oool3000000D0oooo0`0000040?oo
o`@000000`3oool4000000@0oooo0`0000050?ooo`<000003P3oool3000000P0oooo0`0000040?oo
o`<00000103oool3000000@0oooo0`0000040?ooo`<00000103oool3000000T0oooo100000030?oo
o`@00000103oool3000000D0oooo0`0000090?ooo`<000001@3oool3000000T0oooo100000030?oo
o`<00000103oool3000000<0oooo0`0000040?ooo`<000000`3oool3000000`0oooo0`0000030?oo
o`D000000`3oool3000000@0oooo0`0000070?ooo`<00000103oool3000000`0oooo3000000E0?oo
o`D000000`3oool5000000@0oooo0`0000050?ooo`<00000103oool7000001<0oooo0`0000040?oo
o`<00000103oool3000000D0oooo0`0000030?ooo`<000001@3oool3000000@0oooo0`0000040?oo
o`@000000`3oool4000000@0oooo0`0000040?ooo`<00000103oool3000000T0oooo1000000]0?oo
o`00R`3oool3000000l0oooo0P0000000`3oool00000000000050?ooo`<000001@3oool3000000<0
oooo0`0000040?ooo`<000001@3oool3000000<0oooo0`0000050?ooo`<000000`3oool3000000D0
oooo0`0000030?ooo`@000001@3oool4000000<0oooo0`0000050?ooo`<000003P3oool3000000P0
oooo0`0000040?ooo`<00000103oool3000000@0oooo0`0000030?ooo`<00000403oool4000000D0
oooo100000030?ooo`<000001@3oool3000000T0oooo0`0000050?ooo`<000002@3oool3000000D0
oooo0`0000020?ooo`<000001@3oool300000080oooo0`0000050?ooo`<000002`3oool3000000<0
oooo1@0000030?ooo`<00000103oool3000000L0oooo0`0000050?ooo`<000002`3oool<000001D0
oooo0P0000000`3oool00000000000030?ooo`80000000<0oooo000000000000103oool3000000D0
oooo0`0000040?ooo`<0000000@0oooo00000000000000004`3oool3000000@0oooo0`0000040?oo
o`<000001@3oool3000000<0oooo0`0000050?ooo`<00000103oool3000000@0oooo0`0000050?oo
o`<00000103oool3000000@0oooo0`0000040?ooo`<000002P3oool4000002`0oooo002;0?ooo`<0
00003P3oool3000000040?ooo`0000000000000000D0oooo1@0000020?ooo`<000000`3oool30000
00@0oooo0`0000050?ooo`<00000103oool500000080oooo0`0000030?ooo`<000001@3oool30000
00<0oooo0`0000070?ooo`<000000`3oool3000000D0oooo0`00000>0?ooo`<00000203oool30000
00@0oooo0`0000040?ooo`<00000103oool3000000<0oooo0`00000@0?ooo`<000001`3oool30000
00<0oooo0`0000050?ooo`<000002P3oool500000080oooo0`00000A0?ooo`<000000P3oool30000
00D0oooo0`0000020?ooo`<000001@3oool3000000/0oooo0`0000030?ooo`80000000<0oooo0000
000000000`3oool3000000@0oooo0`0000070?ooo`<000001P3oool3000000X0oooo300000030?oo
o`h000000`3oool3000000050?ooo`00000000000000003oool00`000000103oool0000000000000
00040?ooo`D000000P3oool3000000@0oooo0`000000103oool0000000000000000C0?ooo`<00000
103oool3000000D0oooo1@0000020?ooo`<000000`3oool3000000D0oooo0`0000040?ooo`<00000
103oool3000000D0oooo0`0000040?ooo`<00000103oool3000000@0oooo0`00000;0?ooo`@00000
:`3oool008/0oooo0`00000>0?ooo`<0000000@0oooo00000000000000001P3oool9000000<0oooo
0`0000040?ooo`<000001@3oool3000000D0oooo2@0000030?ooo`<000001@3oool3000000<0oooo
0`0000070?ooo`<000000`3oool3000000D0oooo0`00000>0?ooo`<00000203oool3000000@0oooo
0`0000040?ooo`<00000103oool3000000<0oooo2`0000080?ooo`<000001`3oool3000000<0oooo
0`0000050?ooo`<000002`3oool9000000`0oooo10000000103oool000000000000000020?ooo`<0
00001@3oool300000080oooo0`0000050?ooo`<000002`3oool300000080oooo0`000000103oool0
00000000000000020?ooo`<00000103oool3000000L0oooo0`0000060?ooo`@000002@3oool30000
00@0oooo0`0000050?ooo`h000000`3oool3000000050?ooo`00000000000000003oool00`000000
103oool000000000000000050?ooo`T000000`3oool3000000<0oooo0`00000B0?ooo`<00000103o
ool3000000H0oooo2@0000030?ooo`<000001@3oool3000000@0oooo0`0000040?ooo`<000001@3o
ool3000000@0oooo0`0000040?ooo`<00000103oool3000000`0oooo1000000Z0?ooo`00R`3oool<
000000@0oooo0`0000030?ooo`<00000203oool6000000<0oooo0`0000040?ooo`<000001@3oool3
000000P0oooo1P0000030?ooo`<000001@3oool3000000<0oooo0`0000070?ooo`<000000`3oool3
000000D0oooo0`00000>0?ooo`<00000203oool3000000@0oooo0`0000040?ooo`<00000103oool3
000000<0oooo2`0000080?ooo`<000001`3oool3000000<0oooo0`0000050?ooo`<000003P3oool6
000000X0oooo2P0000020?ooo`<000001@3oool300000080oooo0`0000050?ooo`<000002`3oool3
00000080oooo0`000000103oool000000000000000020?ooo`<00000103oool=000000L0oooo0`00
000:0?ooo`<000000`3oool3000001H0oooo0`0000001@3oool00000000000000000oooo00<00000
00@0oooo0000000000000000203oool6000000<0oooo0`0000030?ooo`<000004P3oool3000000@0
oooo0`0000090?ooo`H000000`3oool3000000D0oooo0`0000040?ooo`<00000103oool3000000D0
oooo0`0000040?ooo`<00000103oool3000000@0oooo0`00000=0?ooo`@00000:@3oool008/0oooo
300000040?ooo`<000000`3oool3000000/0oooo0`0000030?ooo`<00000103oool3000000D0oooo
0`00000;0?ooo`<000000`3oool3000000D0oooo0`0000030?ooo`@000001@3oool4000000<0oooo
0`0000050?ooo`<000003P3oool3000000P0oooo0`0000040?ooo`<00000103oool3000000@0oooo
0`0000030?ooo`<000001@3oool3000000P0oooo100000050?ooo`@000000`3oool3000000D0oooo
0`00000A0?ooo`<000002P3oool:00000080oooo0`0000050?ooo`<000000P3oool3000000D0oooo
0`00000;0?ooo`<000000P3oool3000000040?ooo`000000000000000080oooo0`0000040?ooo`d0
0000203oool3000000X0oooo0`0000020?ooo`<000005@3oool3000000<0oooo1@0000030?ooo`<0
00002P3oool300000080oooo100000030?ooo`<000004P3oool3000000@0oooo0`00000<0?ooo`<0
00000`3oool3000000D0oooo0`0000040?ooo`<00000103oool3000000D0oooo0`0000040?ooo`<0
0000103oool3000000@0oooo0`00000>0?ooo`@00000:03oool008/0oooo300000040?ooo`<00000
0`3oool3000000<0oooo0`0000050?ooo`<000000`3oool3000000@0oooo0`0000050?ooo`<00000
0`3oool3000000D0oooo0`0000030?ooo`<000001@3oool3000000@0oooo100000030?ooo`@00000
103oool4000000<0oooo1000000>0?ooo`<00000203oool3000000@0oooo100000030?ooo`@00000
0`3oool3000000<0oooo100000030?ooo`<000002P3oool4000000<0oooo100000040?ooo`@00000
0`3oool4000000T0oooo0`0000050?ooo`<000002@3oool4000000<0oooo100000020?ooo`<00000
1@3oool300000080oooo0`0000050?ooo`<000002`3oool300000080oooo0P0000030?ooo`800000
0P3oool3000000@0oooo3@0000090?ooo`<000002@3oool300000080oooo0`00000E0?ooo`<00000
0`3oool5000000<0oooo0`0000020?ooo`<000001@3oool300000080oooo0`0000050?ooo`<00000
4@3oool3000000@0oooo0`0000040?ooo`<000001@3oool3000000<0oooo100000030?ooo`@00000
103oool3000000@0oooo0`0000050?ooo`<00000103oool4000000<0oooo100000030?ooo`<00000
3`3oool3000002P0oooo002;0?ooo`<00000303oool3000000D0oooo0`0000030?ooo`<000000`3o
ool4000000<0oooo0`0000040?ooo`<000001@3oool3000000@0oooo0`0000030?ooo`@0000000<0
oooo0000000000001@0000030?ooo`<00000103oool;000000@0oooo2`00000>0?ooo`<00000203o
ool3000000@0oooo4@0000040?ooo`T000002P3oool;000000@0oooo2`00000:0?ooo`<000000`3o
ool4000000T0oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`0000020?ooo`<000001@3o
ool3000000/0oooo0`000000103oool000000000000000030?ooo`<0000000@0oooo000000000000
0000103oool3000000L0oooo0`0000030?ooo`X000002@3oool3000000040?ooo`00000000000000
01D0oooo0`0000030?ooo`D000000`3oool3000000<0oooo0`0000030?ooo`@000000P3oool30000
00D0oooo0`00000A0?ooo`<000000P3oool7000000<0oooo0`0000030?ooo`@000000`3oool;0000
00@0oooo0`0000040?ooo`<000001@3oool3000000@0oooo4@00000@0?ooo`<000009`3oool008/0
oooo0`00000<0?ooo`<000001@3oool3000000<0oooo2@0000040?ooo`<00000103oool3000000D0
oooo0`0000040?ooo`T000000P3oool7000000<0oooo0`0000050?ooo`T000001@3oool:000000l0
oooo0`0000080?ooo`<00000103oool9000000030?ooo`000000000000@000001P3oool7000000`0
oooo2@0000050?ooo`X000002`3oool9000000X0oooo0`0000050?ooo`<000000P3oool3000000D0
oooo0`0000020?ooo`<000001@3oool3000000/0oooo0`000000103oool000000000000000030?oo
o`<0000000@0oooo0000000000000000103oool3000000L0oooo0`0000030?ooo`X000002P3oool6
000001@0oooo100000040?ooo`<000001@3oool300000080oooo2@0000020?ooo`<000001`3oool3
00000100oooo0`0000020?ooo`L000000`3oool9000000@0oooo2P0000050?ooo`<00000103oool3
000000D0oooo0`0000040?ooo`T0000000<0oooo000000000000100000090?ooo`<000001@3oool3
000002L0oooo002;0?ooo`<00000303oool3000000D0oooo0`0000050?ooo`H000001@3oool30000
00@0oooo0`0000050?ooo`<000001P3oool6000000<0oooo1`0000030?ooo`<000001`3oool50000
00L0oooo0`0000020?ooo`@00000403oool3000000P0oooo0`0000040?ooo`<0000000<0oooo0000
000000000P0000030?ooo`@00000203oool5000000l0oooo1@0000070?ooo`<000000P3oool40000
00h0oooo1P00000;0?ooo`<000001@3oool300000080oooo0`0000050?ooo`<000000P3oool30000
00D0oooo0`00000;0?ooo`<0000000<0oooo0000000000001@3oool2000000040?ooo`0000000000
000000@0oooo0`0000070?ooo`<000000`3oool:000000X0oooo1P00000D0?ooo`<000001@3oool3
000000D0oooo0`0000040?ooo`H000000`3oool3000000L0oooo0`00000@0?ooo`<000000P3oool7
000000D0oooo1P0000050?ooo`<000000P3oool4000000H0oooo0`0000040?ooo`<000001@3oool3
000000@0oooo0`0000000`3oool0000000000002000000<0oooo1000000:0?ooo`<000001@3oool3
000002L0oooo002;0?ooo`<000009`3oool300000200oooo0`00000d0?ooo`<00000KP3oool40000
00<0oooo0`0000040?ooo`<000000`3oool3000000@0oooo0`0000030?ooo`<00000303oool60000
00D0oooo1P0000040?ooo`<000001`3oool3000001P0oooo1@00001A0?ooo`<00000103oool30000
0500oooo100000030?ooo`@000009`3oool008/0oooo3@00000M0?ooo`<00000803oool3000000D0
oooo0`00000W0?ooo`d000000`3oool3000006@0oooo2@0000040?ooo`T00000103oool9000000`0
oooo1P0000050?ooo`H00000103oool3000000L0oooo0`00000I0?ooo`@00000D@3oool3000000@0
oooo0`00000Q0?ooo`<00000;@3oool9000002P0oooo002;0?ooo`d000007@3oool300000240oooo
0P0000050?ooo`<000009`3oool=000000<0oooo0`00001U0?ooo`L000001P3oool7000000H0oooo
1`00000=0?ooo`D000001`3oool5000000@0oooo0`0000070?ooo`<000006@3oool400000540oooo
0`0000050?ooo`8000008@3oool3000002h0oooo2000000X0?ooo`00R`3oool=000001d0oooo0`00
000R0?ooo`030000003oool0oooo00<0oooo0`00000W0?ooo`d000000`3oool3000006H0oooo1@00
00080?ooo`D00000203oool5000000h0oooo1@0000070?ooo`D00000103oool3000000L0oooo0`00
000J0?ooo`<00000D@3oool3000000H0oooo00<000000?ooo`3oool07`3oool3000002l0oooo1@00
000Z0?ooo`00o`3ooooo0?ooolh0oooo003o0?ooool0oooocP3oool00?l0ooooo`3oooo>0?ooo`00
o`3ooooo0?ooolh0oooo003o0?ooool0oooocP3oool00?l0ooooo`3oooo>0?ooo`00o`3ooooo0?oo
olh0oooo003o0?ooool0oooocP3oool00?l0ooooo`3oooo>0?ooo`00o`3ooooo0?ooolh0oooo003o
0?ooool0oooocP3oool00?l0ooooo`3oooo>0?ooo`00o`3ooooo0?ooolh0oooo003o0?ooool0oooo
cP3oool00?l0ooooo`3oooo>0?ooo`00o`3ooooo0?ooolh0oooo003o0?ooool0oooocP3oool00?l0
ooooo`3oooo>0?ooo`00o`3ooooo0?ooolh0oooo003o0?ooool0oooocP3oool00?l0ooooo`3oooo>
0?ooo`00o`3ooooo0?ooolh0oooo003o0?ooool0oooocP3oool00?l0ooooo`3oooo>0?ooo`00o`3o
oooo0?ooolh0oooo003o0?ooool0oooocP3oool00?l0ooooo`3oooo>0?ooo`00o`3ooooo0?ooolh0
oooo003o0?ooool0oooocP3oool00?l0ooooo`3oooo>0?ooo`00o`3ooooo0?ooolh0oooo003o0?oo
ool0oooocP3oool00?l0ooooo`3oooo>0?ooo`00o`3ooooo0?ooolh0oooo003o0?ooool0oooocP3o
ool00?l0ooooo`3oooo>0?ooo`00o`3ooooo0?ooolh0oooo003o0?ooool0oooocP3oool00?l0oooo
o`3oooo>0?ooo`00o`3ooooo0?ooolh0oooo003o0?ooool0oooocP3oool00?l0ooooo`3oooo>0?oo
o`00o`3ooooo0?ooolh0oooo003o0?ooool0oooocP3oool00001\
\>"],
  ImageRangeCache->{{{0, 715.438}, {506.938, 0}} -> {-1.05068, -0.83546, \
0.00440575, 0.00440575}, {{17.25, 698.125}, {432.875, 12.0625}} -> {-988.247, \
-6.53924, 7.40139, 0.030572}, {{160.25, 636.875}, {494.813, 447.188}} -> \
{-1.73052, -0.700857, 0.0152442, 0.0743314}, {{178.125, 276.5}, {491.125, \
471}} -> {-22.5903, -2.57145, 0.14907, 0.14907}, {{292.25, 390.688}, \
{491.125, 471}} -> {-33.9351, -2.57145, 0.14907, 0.14907}, {{406.375, \
504.813}, {491.125, 471}} -> {-45.2769, -2.57145, 0.14907, 0.14907}, \
{{520.563, 618.938}, {491.125, 471}} -> {-56.6217, -2.57145, 0.14907, \
0.14907}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[GridBox[{
        {
          ButtonBox[
            StyleBox["\[FirstPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageFirst"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"First Slide"], 
          ButtonBox[
            StyleBox["\[LeftPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPagePrevious"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Previous Slide"], 
          ButtonBox[
            StyleBox["\[RightPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageNext"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Next Slide"], 
          ButtonBox[
            StyleBox["\[LastPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageLast"]}],
            ButtonStyle->"SlideHyperlink"], " ", 
          ButtonBox[
            StyleBox[
              RowBox[{
                CounterBox["SlideShowNavigationBar"], \(\(\ \)\(of\)\(\ \)\), 
                
                CounterBox["SlideShowNavigationBar", "LastSlide"]}],
              "SR"],
            ButtonFrame->"None"]}
        }]]]], "SlideShowNavigationBar"],

Cell[TextData[{
  "One way of making the performance measure independent of problem size is \
to determine the number of operations you are doing per unit time.  One such \
measure is \"flops\", or floating operations per second.  This is computed by \
using the number of operations required to do the operation by the running \
time.  This shows a plot of the megaflops achieved as a function of matrix \
size.  This shows that ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " is getting very good performance which scales well with the number of \
processors.  Notice that the curve flattens out beyond about size 1000 where \
peak performance is used.  "
}], "Text"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .70902 
%%ImageSize: 850 602.664 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.47619 0.397834 0.47619 [
[ 0 0 0 0 ]
[ 1 .70902 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
% Start of sub-graphic
p
0.0238095 0.103532 0.97619 0.692136 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.000463897 0.0147151 5.93423e-05 [
[.02381 -0.0125 -6.4375 -18 ]
[.02381 -0.0125 6.4375 0 ]
[.25576 -0.0125 -15.3125 -18 ]
[.25576 -0.0125 15.3125 0 ]
[.48771 -0.0125 -19.75 -18 ]
[.48771 -0.0125 19.75 0 ]
[.71966 -0.0125 -19.75 -18 ]
[.71966 -0.0125 19.75 0 ]
[.9516 -0.0125 -19.75 -18 ]
[.9516 -0.0125 19.75 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -60.0625 -36 ]
[.5 -0.0125 60.0625 -18 ]
[-0.0125 .01472 -12.875 -9 ]
[-0.0125 .01472 0 9 ]
[-0.0125 .1334 -39.5 -9 ]
[-0.0125 .1334 0 9 ]
[-0.0125 .25208 -39.5 -9 ]
[-0.0125 .25208 0 9 ]
[-0.0125 .37077 -39.5 -9 ]
[-0.0125 .37077 0 9 ]
[-0.0125 .48945 -39.5 -9 ]
[-0.0125 .48945 0 9 ]
[-0.0125 .60814 -48.375 -9 ]
[-0.0125 .60814 0 9 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -118.875 -9 ]
[-0.01875 .30902 -48.375 9 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[.5 .63053 -156.875 0 ]
[.5 .63053 156.875 18 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
.5 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .61803 L
s
.25576 0 m
.25576 .61803 L
s
.48771 0 m
.48771 .61803 L
s
.71966 0 m
.71966 .61803 L
s
.9516 0 m
.9516 .61803 L
s
0 .01472 m
1 .01472 L
s
0 .1334 m
1 .1334 L
s
0 .25208 m
1 .25208 L
s
0 .37077 m
1 .37077 L
s
0 .48945 m
1 .48945 L
s
0 .60814 m
1 .60814 L
s
0 g
.02381 0 m
.02381 .00625 L
s
gsave
.02381 -0.0125 -67.4375 -22 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
1.000 setlinewidth
grestore
.25576 0 m
.25576 .00625 L
s
gsave
.25576 -0.0125 -76.3125 -22 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(500) show
1.000 setlinewidth
grestore
.48771 0 m
.48771 .00625 L
s
gsave
.48771 -0.0125 -80.75 -22 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1000) show
1.000 setlinewidth
grestore
.71966 0 m
.71966 .00625 L
s
gsave
.71966 -0.0125 -80.75 -22 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1500) show
1.000 setlinewidth
grestore
.9516 0 m
.9516 .00625 L
s
gsave
.9516 -0.0125 -80.75 -22 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2000) show
1.000 setlinewidth
grestore
.125 Mabswid
.0702 0 m
.0702 .00375 L
s
.11659 0 m
.11659 .00375 L
s
.16298 0 m
.16298 .00375 L
s
.20937 0 m
.20937 .00375 L
s
.30215 0 m
.30215 .00375 L
s
.34854 0 m
.34854 .00375 L
s
.39493 0 m
.39493 .00375 L
s
.44132 0 m
.44132 .00375 L
s
.5341 0 m
.5341 .00375 L
s
.58049 0 m
.58049 .00375 L
s
.62688 0 m
.62688 .00375 L
s
.67327 0 m
.67327 .00375 L
s
.76605 0 m
.76605 .00375 L
s
.81243 0 m
.81243 .00375 L
s
.85882 0 m
.85882 .00375 L
s
.90521 0 m
.90521 .00375 L
s
.99799 0 m
.99799 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -121.062 -40 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Vector) show
125.938 17.000 moveto
(Length) show
179.125 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .01472 m
.00625 .01472 L
s
gsave
-0.0125 .01472 -73.875 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
1.000 setlinewidth
grestore
0 .1334 m
.00625 .1334 L
s
gsave
-0.0125 .1334 -100.5 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2000) show
1.000 setlinewidth
grestore
0 .25208 m
.00625 .25208 L
s
gsave
-0.0125 .25208 -100.5 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4000) show
1.000 setlinewidth
grestore
0 .37077 m
.00625 .37077 L
s
gsave
-0.0125 .37077 -100.5 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6000) show
1.000 setlinewidth
grestore
0 .48945 m
.00625 .48945 L
s
gsave
-0.0125 .48945 -100.5 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(8000) show
1.000 setlinewidth
grestore
0 .60814 m
.00625 .60814 L
s
gsave
-0.0125 .60814 -109.375 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10000) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .04439 m
.00375 .04439 L
s
0 .07406 m
.00375 .07406 L
s
0 .10373 m
.00375 .10373 L
s
0 .16307 m
.00375 .16307 L
s
0 .19274 m
.00375 .19274 L
s
0 .22241 m
.00375 .22241 L
s
0 .28176 m
.00375 .28176 L
s
0 .31143 m
.00375 .31143 L
s
0 .3411 m
.00375 .3411 L
s
0 .40044 m
.00375 .40044 L
s
0 .43011 m
.00375 .43011 L
s
0 .45978 m
.00375 .45978 L
s
0 .51912 m
.00375 .51912 L
s
0 .5488 m
.00375 .5488 L
s
0 .57847 m
.00375 .57847 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -179.875 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(MFLOPS) show
129.500 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.02381 .61178 m
.02381 .61803 L
s
.25576 .61178 m
.25576 .61803 L
s
.48771 .61178 m
.48771 .61803 L
s
.71966 .61178 m
.71966 .61803 L
s
.9516 .61178 m
.9516 .61803 L
s
.125 Mabswid
.0702 .61428 m
.0702 .61803 L
s
.11659 .61428 m
.11659 .61803 L
s
.16298 .61428 m
.16298 .61803 L
s
.20937 .61428 m
.20937 .61803 L
s
.30215 .61428 m
.30215 .61803 L
s
.34854 .61428 m
.34854 .61803 L
s
.39493 .61428 m
.39493 .61803 L
s
.44132 .61428 m
.44132 .61803 L
s
.5341 .61428 m
.5341 .61803 L
s
.58049 .61428 m
.58049 .61803 L
s
.62688 .61428 m
.62688 .61803 L
s
.67327 .61428 m
.67327 .61803 L
s
.76605 .61428 m
.76605 .61803 L
s
.81243 .61428 m
.81243 .61803 L
s
.85882 .61428 m
.85882 .61803 L
s
.90521 .61428 m
.90521 .61803 L
s
.99799 .61428 m
.99799 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .01472 m
1 .01472 L
s
.99375 .1334 m
1 .1334 L
s
.99375 .25208 m
1 .25208 L
s
.99375 .37077 m
1 .37077 L
s
.99375 .48945 m
1 .48945 L
s
.99375 .60814 m
1 .60814 L
s
.125 Mabswid
.99625 .04439 m
1 .04439 L
s
.99625 .07406 m
1 .07406 L
s
.99625 .10373 m
1 .10373 L
s
.99625 .16307 m
1 .16307 L
s
.99625 .19274 m
1 .19274 L
s
.99625 .22241 m
1 .22241 L
s
.99625 .28176 m
1 .28176 L
s
.99625 .31143 m
1 .31143 L
s
.99625 .3411 m
1 .3411 L
s
.99625 .40044 m
1 .40044 L
s
.99625 .43011 m
1 .43011 L
s
.99625 .45978 m
1 .45978 L
s
.99625 .51912 m
1 .51912 L
s
.99625 .5488 m
1 .5488 L
s
.99625 .57847 m
1 .57847 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
gsave
.5 .63053 -217.875 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(MFLOPS) show
138.375 17.000 moveto
(on) show
162.312 17.000 moveto
(a) show
175.625 17.000 moveto
(900) show
206.688 17.000 moveto
(MHz) show
243.938 17.000 moveto
(4) show
252.812 17.000 moveto
%%IncludeResource: font Mathematica1-Bold
%%IncludeFont: Mathematica1-Bold
/Mathematica1-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
263.625 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(way) show
302.688 17.000 moveto
(Itanium) show
363.875 17.000 moveto
(2) show
372.750 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 .4 .4 r
.004 w
.08087 .2115 m
.08319 .31861 L
.08551 .29571 L
.09339 .26415 L
.13978 .39451 L
.14025 .38078 L
.14257 .44397 L
.14489 .33884 L
.18617 .453 L
.23256 .43741 L
.25901 .46724 L
.26132 .50901 L
.26364 .49457 L
.27895 .49615 L
.32534 .52041 L
.37173 .4851 L
.41812 .56351 L
.46451 .57193 L
.49652 .55502 L
.49884 .58959 L
.50116 .56189 L
.5109 .54362 L
.55729 .59964 L
.60368 .58946 L
.65007 .58755 L
.69646 .59364 L
.74285 .58185 L
.78924 .59612 L
.83563 .59835 L
.88202 .60212 L
.92841 .60332 L
.97155 .55057 L
.97387 .59663 L
.97619 .53301 L
s
.4 .4 1 r
.08087 .16327 m
.08319 .2471 L
.08551 .21592 L
.09339 .23186 L
.13978 .26791 L
.14025 .26071 L
.14257 .29355 L
.14489 .26404 L
.18617 .28579 L
.23256 .30312 L
.25901 .29755 L
.26132 .32841 L
.26364 .29937 L
.27895 .2995 L
.32534 .31272 L
.37173 .32545 L
.41812 .327 L
.46451 .32763 L
.49652 .32235 L
.49884 .34342 L
.50116 .30897 L
.5109 .31924 L
.55729 .33285 L
.60368 .33333 L
.65007 .33589 L
.69646 .3369 L
.74285 .33798 L
.78924 .34065 L
.83563 .34417 L
.88202 .34244 L
.92841 .34712 L
.97155 .32733 L
.97387 .35224 L
.97619 .31962 L
s
.4 .8 .4 r
.08087 .13994 m
.08319 .16521 L
.08551 .14304 L
.09339 .15246 L
.13978 .1594 L
.14025 .15707 L
.14257 .17865 L
.14489 .15877 L
.18617 .16551 L
.23256 .17158 L
.25901 .16849 L
.26132 .18288 L
.26364 .16334 L
.27895 .16907 L
.32534 .17365 L
.37173 .17748 L
.41812 .17875 L
.46451 .18064 L
.49652 .17566 L
.49884 .18848 L
.50116 .17217 L
.5109 .17699 L
.55729 .18184 L
.60368 .18366 L
.65007 .18337 L
.69646 .18452 L
.74285 .18209 L
.78924 .18458 L
.83563 .18478 L
.88202 .18432 L
.92841 .18617 L
.97155 .17779 L
.97387 .1907 L
.97619 .17527 L
s
.8 .4 .8 r
.08087 .0275 m
.08319 .02797 L
.08551 .02761 L
.09339 .02764 L
.13978 .02742 L
.14025 .02742 L
.14257 .02698 L
.14489 .02744 L
.18617 .02546 L
.23256 .02299 L
.25901 .0221 L
.26132 .02266 L
.26364 .02212 L
.27895 .02186 L
.32534 .02184 L
.37173 .02181 L
.41812 .02186 L
.46451 .02185 L
.49652 .02185 L
.49884 .02202 L
.50116 .02185 L
.5109 .02187 L
.55729 .02187 L
.60368 .02188 L
.65007 .02189 L
.69646 .0219 L
.74285 .0219 L
.78924 .02191 L
.83563 .02191 L
.88202 .02191 L
.92841 .02192 L
.97155 .02191 L
.97387 .02208 L
.97619 .02191 L
s
MathSubEnd
P
% End of sub-graphic
0 g
.22381 .01688 m
.22381 .08355 L
.89048 .08355 L
.89048 .01688 L
F
1 g
.22381 .01688 m
.22381 .08355 L
.89048 .08355 L
.89048 .01688 L
F
0 g
.001 w
[ ] 0 setdash
.22381 .01688 m
.89048 .01688 L
.89048 .08355 L
.22381 .08355 L
.22381 .01688 L
s
% Start of sub-graphic
p
0.22381 0.0168814 0.890476 0.083548 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0206612 0.206612 0.00423729 0.0423729 [
[ 0 0 0 0 ]
[ 1 .1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .1 L
0 .1 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0371901 0.00762712 0.243802 0.05 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.5 0.5 0.5 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
1 .4 .4 r
.21371 w
[ ] 0 setdash
0 .5 m
1 .5 L
s
MathSubEnd
P
% End of sub-graphic
0 g
gsave
.1405 .05339 -100.688 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
76.312 17.000 moveto
(Threads) show
138.375 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
% Start of sub-graphic
p
0.27686 0.00762712 0.483471 0.05 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.5 0.5 0.5 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
.4 .4 1 r
.21371 w
[ ] 0 setdash
0 .5 m
1 .5 L
s
MathSubEnd
P
% End of sub-graphic
0 g
gsave
.38017 .05339 -100.688 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
76.312 17.000 moveto
(Threads) show
138.375 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
% Start of sub-graphic
p
0.516529 0.00762712 0.72314 0.05 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.5 0.5 0.5 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
.4 .8 .4 r
.21371 w
[ ] 0 setdash
0 .5 m
1 .5 L
s
MathSubEnd
P
% End of sub-graphic
0 g
gsave
.61983 .05339 -96.25 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
76.312 17.000 moveto
(Thread) show
129.500 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
% Start of sub-graphic
p
0.756198 0.00762712 0.96281 0.05 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.5 0.5 0.5 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
.8 .4 .8 r
.21371 w
[ ] 0 setdash
0 .5 m
1 .5 L
s
MathSubEnd
P
% End of sub-graphic
0 g
gsave
.8595 .05339 -80.75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 26 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4.2) show
85.188 17.000 moveto
(.1) show
98.500 17.000 moveto
%%IncludeResource: font Helvetica-Bold
%%IncludeFont: Helvetica-Bold
/Helvetica-Bold findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
MathSubEnd
P
% End of sub-graphic
0 0 m
1 0 L
1 .70902 L
0 .70902 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[53]:=",
  ImageSize->{446.625, 316.625},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000:M0001fR000`400?l00000o`00003oo`3ooooo
0?oooil0oooo003o0?ooool0ooooW`3oool00?l0ooooo`3ooonO0?ooo`00o`3ooooo0?oooil0oooo
003o0?ooool0ooooW`3oool00?l0ooooo`3ooonO0?ooo`00o`3ooooo0?oooil0oooo003o0?ooool0
ooooW`3oool00?l0ooooo`3ooonO0?ooo`00o`3ooooo0?oooil0oooo003o0?ooool0ooooW`3oool0
09D0ooooo`000030000004T0oooo002E0?ooo`030000003oool0oooo0?l0oooo_03oool00`000000
oooo0?ooo`170?ooo`00U@3oool00`000000oooo0?ooo`3o0?oook`0oooo00<000000?ooo`3oool0
A`3oool009D0oooo00<000000?ooo`3oool0o`3ooonl0?ooo`030000003oool0oooo04L0oooo002E
0?ooo`030000003oool0oooo0?l0oooo_03oool00`000000oooo0?ooo`170?ooo`00U@3oool00`00
0000oooo0?ooo`3o0?oook`0oooo00<000000?ooo`3oool0A`3oool009D0oooo00<000000?ooo`3o
ool0o`3ooonl0?ooo`030000003oool0oooo04L0oooo002E0?ooo`030000003oool0oooo0?l0oooo
_03oool00`000000oooo0?ooo`170?ooo`00U@3oool00`000000oooo0?ooo`3o0?oook`0oooo00<0
00000?ooo`3oool0A`3oool009D0oooo00<000000?ooo`3oool0o`3ooonl0?ooo`030000003oool0
oooo04L0oooo002E0?ooo`030000003oool0oooo0?l0oooo_03oool00`000000oooo0?ooo`170?oo
o`00U@3oool00`000000oooo0?ooo`3o0?oook`0oooo00<000000?ooo`3oool0A`3oool009D0oooo
00<000000?ooo`3oool0<P3ooolC0?mVIUP0oooo4`1VI_mH0?oooa<0I/aVF03ooolC0<aVc3D0oooo
00<000000?ooo`3oool0A`3oool009D0oooo00<000000?ooo`3oool0<P3ooolC0?mVIUP0oooo4`1V
I_mH0?oooa<0I/aVF03ooolC0<aVc3D0oooo00<000000?ooo`3oool0A`3oool009D0oooo00<00000
0?ooo`3oool0<P3ooolC0?mVIUP0oooo4`1VI_mH0?oooa<0I/aVF03ooolC0<aVc3D0oooo00<00000
0?ooo`3oool0A`3oool009D0oooo00<000000?ooo`3oool0<P3ooolC0?mVIUP0oooo4`1VI_mH0?oo
oa<0I/aVF03ooolC0<aVc3D0oooo00<000000?ooo`3oool0A`3oool009D0oooo00<000000?ooo`3o
ool0o`3ooonl0?ooo`030000003oool0oooo04L0oooo002E0?ooo`030000003oool0oooo0?l0oooo
_03oool00`000000oooo0?ooo`170?ooo`00U@3oool00`000000oooo0?ooo`3o0?oook`0oooo00<0
00000?ooo`3oool0A`3oool009D0oooo00<000000?ooo`3oool0o`3ooonl0?ooo`030000003oool0
oooo04L0oooo002E0?ooo`030000003oool0oooo0?l0oooo_03oool00`000000oooo0?ooo`170?oo
o`00U@3oool00`000000oooo0?ooo`3o0?oook`0oooo00<000000?ooo`3oool0A`3oool009D0oooo
00<000000?ooo`3oool0o`3ooonl0?ooo`030000003oool0oooo04L0oooo002E0?ooo`030000003o
ool0oooo0?l0oooo_03oool00`000000oooo0?ooo`170?ooo`00U@3oool00`000000oooo0?ooo`3o
0?oook`0oooo00<000000?ooo`3oool0A`3oool009D0oooo00<000000?ooo`3oool0o`3ooonl0?oo
o`030000003oool0oooo04L0oooo002E0?ooo`030000003oool0oooo0?l0oooo_03oool00`000000
oooo0?ooo`170?ooo`00U@3oool00`000000oooo0?ooo`3o0?oook`0oooo00<000000?ooo`3oool0
A`3oool009D0oooo00<000000?ooo`3oool0o`3ooonl0?ooo`030000003oool0oooo04L0oooo002E
0?ooo`030000003oool0oooo0?l0oooo_03oool00`000000oooo0?ooo`170?ooo`00U@3oool00`00
0000oooo0?ooo`3o0?oook`0oooo00<000000?ooo`3oool0A`3oool009D0oooo00<000000?ooo`3o
ool0o`3ooonl0?ooo`030000003oool0oooo04L0oooo002E0?ooo`030000003oool0oooo00X0oooo
0`00000@0?ooo`<00000203oool3000000D0oooo0`0000040?ooo`<00000203oool5000000L0oooo
1@0000020?ooo`<00000103oool400000080oooo0`0000060?ooo`l000003P3oool3000000P0oooo
0`0000050?ooo`<00000103oool3000000P0oooo1@0000070?ooo`D000000P3oool3000000@0oooo
100000020?ooo`<000001P3oool5000000/0oooo0`00000A0?ooo`<00000203oool3000000D0oooo
0`0000040?ooo`<00000203oool5000000L0oooo1@0000020?ooo`<00000103oool400000080oooo
0`00000W0?ooo`<00000103oool3000000@0oooo2`0000020?ooo`<000002@3oool3000002P0oooo
00<000000?ooo`3oool0A`3oool009D0oooo00<000000?ooo`3oool02P3oool300000100oooo0`00
00080?ooo`<000001@3oool3000000@0oooo0`0000070?ooo`P00000103oool:000000@0oooo2P00
00040?oooa4000003P3oool3000000P0oooo0`0000050?ooo`<00000103oool3000000L0oooo2000
00040?ooo`X00000103oool:000000@0oooo2@0000090?ooo`<000004@3oool3000000P0oooo0`00
00050?ooo`<00000103oool3000000L0oooo200000040?ooo`X00000103oool:000002L0oooo0`00
00040?ooo`<00000103oool;00000080oooo0`0000090?ooo`<00000:03oool00`000000oooo0?oo
o`170?ooo`00U@3oool00`000000oooo0?ooo`0:0?ooo`<00000403oool3000000P0oooo0`000005
0?ooo`<00000103oool3000000H0oooo2@0000030?ooo`@000000`3oool4000000<0oooo2`000004
0?ooo`<00000103oool:000000h0oooo0`0000080?ooo`<000001@3oool3000000@0oooo0`000006
0?ooo`T000000`3oool4000000<0oooo100000030?ooo`/00000103oool3000000@0oooo0`000008
0?ooo`<000004@3oool3000000P0oooo0`0000050?ooo`<00000103oool3000000H0oooo2@000003
0?ooo`@000000`3oool4000000<0oooo2`00000W0?ooo`<00000103oool3000000D0oooo2P000002
0?ooo`<000002@3oool3000002P0oooo00<000000?ooo`3oool0A`3oool009D0oooo00<000000?oo
o`3oool00`3oool<000000h0oooo0`0000080?ooo`<000001@3oool3000000@0oooo0`0000060?oo
o`<00000103oool300000080oooo0`0000050?ooo`<000000P3oool4000000@0oooo100000030?oo
o`<000001@3oool4000001@0oooo0`0000080?ooo`<000001@3oool3000000@0oooo0`0000060?oo
o`<00000103oool300000080oooo0`0000050?ooo`<000000P3oool4000000@0oooo100000030?oo
o`<000001@3oool3000000P0oooo0`00000A0?ooo`<00000203oool3000000D0oooo0`0000040?oo
o`<000001P3oool3000000@0oooo0`0000020?ooo`<000001@3oool300000080oooo100000040?oo
o`@00000803oool<000000X0oooo1000000D0?ooo`<00000:03oool00`000000oooo0?ooo`170?oo
o`00U@3oool00`000000oooo0?ooo`030?ooo``000003P3oool3000000P0oooo0`0000050?ooo`<0
0000103oool3000000D0oooo0`00000:0?ooo`<000001@3oool300000080oooo0`0000060?ooo`<0
00002P3oool6000001<0oooo0`0000080?ooo`<000001@3oool3000000@0oooo0`0000050?ooo`<0
00002P3oool3000000D0oooo0`0000020?ooo`<000001P3oool3000000X0oooo100000080?ooo`<0
00004@3oool3000000P0oooo0`0000050?ooo`<00000103oool3000000D0oooo0`00000:0?ooo`<0
00001@3oool300000080oooo0`0000060?ooo`<00000803oool<000000/0oooo1000000C0?ooo`<0
0000:03oool00`000000oooo0?ooo`170?ooo`00U@3oool00`000000oooo0?ooo`030?ooo``00000
3P3oool3000000P0oooo0`0000050?ooo`<00000103oool3000000D0oooo0`00000;0?ooo`D00000
0P3oool300000080oooo0`0000060?ooo`<000001`3oool:00000180oooo0`0000080?ooo`<00000
1@3oool3000000@0oooo0`0000050?ooo`<000002`3oool500000080oooo0`0000020?ooo`<00000
1P3oool3000000L0oooo1`0000080?ooo`<000004@3oool3000000P0oooo0`0000050?ooo`<00000
103oool3000000D0oooo0`00000;0?ooo`D000000P3oool300000080oooo0`0000060?ooo`<00000
803oool<000000`0oooo1000000B0?ooo`<00000:03oool00`000000oooo0?ooo`170?ooo`00U@3o
ool00`000000oooo0?ooo`030?ooo`<00000103oool300000100oooo0`0000080?ooo`<000001@3o
ool3000000@0oooo0`0000050?ooo`/00000103oool900000080oooo0`0000060?ooo`<00000103o
ool9000000030?ooo`0000000000008000004@3oool3000000P0oooo0`0000050?ooo`<00000103o
ool3000000D0oooo2`0000040?ooo`T000000P3oool3000000H0oooo0`0000040?ooo`T000002@3o
ool300000140oooo0`0000080?ooo`<000001@3oool3000000@0oooo0`0000050?ooo`/00000103o
ool900000080oooo0`0000060?ooo`<00000803oool3000000@0oooo0`00000?0?ooo`@000004@3o
ool3000002P0oooo00<000000?ooo`3oool0A`3oool009D0oooo00<000000?ooo`3oool0103oool3
000000<0oooo0`00000@0?ooo`<00000203oool3000000D0oooo0`0000040?ooo`<000001@3oool;
000000L0oooo1P0000020?ooo`<000001P3oool3000000<0oooo200000040?ooo`@00000403oool3
000000P0oooo0`0000050?ooo`<00000103oool3000000D0oooo2`0000070?ooo`H000000P3oool3
000000H0oooo0`0000030?ooo`P000002`3oool300000140oooo0`0000080?ooo`<000001@3oool3
000000@0oooo0`0000050?ooo`/000001`3oool600000080oooo0`0000060?ooo`<000008@3oool3
000000<0oooo0`00000@0?ooo`@00000403oool3000002P0oooo00<000000?ooo`3oool0A`3oool0
09D0oooo00<000000?ooo`3oool01@3oool300000080oooo0`00000@0?ooo`<00000203oool30000
00D0oooo0`0000040?ooo`<000001@3oool3000000D0oooo0`00000:0?ooo`<000000P3oool30000
00H0oooo0`0000030?ooo`D00000203oool4000000l0oooo0`0000080?ooo`<000001@3oool30000
00@0oooo0`0000050?ooo`<000001@3oool3000000X0oooo0`0000020?ooo`<000001P3oool30000
00<0oooo1@00000>0?ooo`<000004@3oool3000000P0oooo0`0000050?ooo`<00000103oool30000
00D0oooo0`0000050?ooo`<000002P3oool300000080oooo0`0000060?ooo`<000008P3oool30000
0080oooo0`00000A0?ooo`@000003`3oool3000002P0oooo00<000000?ooo`3oool0A`3oool009D0
oooo00<000000?ooo`3oool01@3oool300000080oooo0`00000@0?ooo`<00000203oool4000000<0
oooo100000040?ooo`@00000103oool4000000<0oooo0`0000030?ooo`<000001@3oool3000000<0
oooo0`0000040?ooo`@000000`3oool3000000D0oooo0`0000030?ooo`<000003`3oool3000000P0
oooo100000030?ooo`@00000103oool4000000@0oooo100000030?ooo`<000000`3oool3000000D0
oooo0`0000030?ooo`<00000103oool4000000<0oooo0`0000050?ooo`<00000203oool300000140
oooo0`0000080?ooo`@000000`3oool4000000@0oooo100000040?ooo`@000000`3oool3000000<0
oooo0`0000050?ooo`<000000`3oool3000000@0oooo1000000R0?ooo`<000000P3oool300000180
oooo0`00000?0?ooo`<00000:03oool00`000000oooo0?ooo`170?ooo`00U@3oool00`000000oooo
0?ooo`060?ooo`<0000000@0oooo0000000000000000403oool3000000P0oooo2`0000040?ooo`L0
00000P3oool9000000@0oooo0`0000030?ooo`@000000`3oool;000000<0oooo0`0000040?ooo`<0
00001@3oool3000000h0oooo0`0000080?ooo`/00000103oool700000080oooo2@0000040?ooo`<0
00000`3oool4000000<0oooo2`0000030?ooo`<00000103oool3000000D0oooo00@000000?ooo`3o
ool0oooo0`00000A0?ooo`<00000203oool;000000@0oooo1`0000020?ooo`T00000103oool30000
00<0oooo100000030?ooo`/000008`3oool3000000040?ooo`0000000000000001<0oooo0`00000:
0?ooo`040000003oool0oooo0?ooo`<00000:03oool00`000000oooo0?ooo`170?ooo`00U@3oool0
0`000000oooo0?ooo`070?ooo`H00000403oool3000000P0oooo2P0000050?ooo`L000000`3oool7
000000D0oooo2@0000050?ooo`X00000103oool9000000D0oooo0`00000>0?ooo`<00000203oool:
000000D0oooo1`0000030?ooo`L000001@3oool9000000D0oooo2P0000040?ooo`T000001@3oool3
000000040?ooo`000000000000000140oooo0`0000080?ooo`X000001@3oool7000000<0oooo1`00
00050?ooo`T000001@3oool:000002@0oooo1P00000;0?ooo`<000001@3oool3000000X0oooo0`00
0000103oool0000000000000000X0?ooo`030000003oool0oooo04L0oooo002E0?ooool00000`000
00190?ooo`00o`3ooooo0?oooil0oooo003o0?ooool0ooooW`3oool00?l0ooooo`3ooonO0?ooo`00
o`3ooooo0?oooil0oooo003o0?ooool0ooooW`3oool00?l0ooooo`3ooonO0?ooo`00o`3ooooo0?oo
oil0oooo003o0?ooool0ooooW`3oool00?l0ooooo`3ooonO0?ooo`00o`3ooooo0?oooil0oooo003o
0?ooool0ooooW`3oool00?l0ooooo`3ooonO0?ooo`00o`3ooooo0?oooil0oooo003o0?ooool0oooo
W`3oool00?l0ooooo`3ooonO0?ooo`00o`3ooooo0?oooil0oooo003o0?ooool0ooooW`3oool00?l0
ooooo`3ooonO0?ooo`00o`3ooooo0?oooil0oooo003o0?ooool0ooooW`3oool00?l0ooooo`3ooonO
0?ooo`00o`3ooooo0?oooil0oooo003o0?ooool0ooooW`3oool00?l0ooooo`3ooonO0?ooo`00o`3o
oooo0?oooil0oooo003o0?ooool0ooooW`3oool00?l0ooooo`3ooonO0?ooo`00o`3ooooo0?oooil0
oooo003o0?ooool0ooooW`3oool00?l0ooooo`3ooonO0?ooo`00o`3ooooo0?oooil0oooo003o0?oo
ool0ooooW`3oool00?l0ooooo`3ooonO0?ooo`00o`3ooooo0?oooil0oooo003o0?ooomL0oooo1`00
00300?ooo`00o`3ooooF0?ooo`X00000_P3oool00?l0ooooe@3oool;00000;h0oooo003o0?ooomD0
oooo0`0000050?ooo`@00000_@3oool00?l0oooogP3oool300000;d0oooo003o0?oooe80oooo0`00
000:0?ooo`D00000203oool5000000L0oooo100000060?ooo`D000001`3oool3000001X0oooo3000
00050?ooo`D000001P3oool3000000D0oooo0`0000060?ooo`@000000P3oool3000000D0oooo1000
00030?ooo`<000001@3oool300000:H0oooo003o0?oooe40oooo1@0000080?ooo`P000001@3oool8
000000@0oooo1@0000040?ooo`T000001@3oool3000001X0oooo300000040?ooo`P00000103oool3
000000D0oooo0`0000050?ooo`X00000103oool5000000<0oooo0`0000050?ooo`<00000YP3oool0
0?l0ooooD@3oool5000000L0oooo2@0000040?ooo`T00000103oool5000000<0oooo2`0000040?oo
o`<000006P3oool<000000<0oooo2@0000040?ooo`<000001@3oool3000000@0oooo2`0000040?oo
o`D000000`3oool3000000D0oooo0`00002V0?ooo`00o`3ooomA0?ooo`D000001`3oool3000000@0
oooo0`0000020?ooo`@000000`3oool4000000<0oooo0`0000050?ooo`@000000`3oool4000000@0
oooo0`00000J0?ooo`<00000303oool3000000@0oooo0`0000030?ooo`<000001@3oool3000000@0
oooo0`0000040?ooo`@00000103oool3000000D0oooo0`0000050?ooo`<00000YP3oool00?l0oooo
D03oool3000000040?ooo`0000000000000000D0oooo0`00000:0?ooo`<000001@3oool2000000@0
oooo0`0000040?ooo`@000001@3oool4000000<0oooo0`00000J0?ooo`<000002`3oool3000000/0
oooo0`0000050?ooo`<000000`3oool3000000H0oooo0`0000040?ooo`<000001@3oool3000000D0
oooo0`00002V0?ooo`00o`3ooom@0?ooo`<0000000@0oooo00000000000000001@3oool3000000X0
oooo0`00000;0?ooo`<00000103oool3000000L0oooo0`0000030?ooo`<000006P3oool3000000/0
oooo0`00000;0?ooo`<000001@3oool3000000<0oooo0`0000060?ooo`<00000103oool3000000D0
oooo0`0000050?ooo`<00000YP3oool00?l0ooooC`3oool3000000<0oooo0`0000040?ooo`/00000
0P3oool3000000/0oooo0`0000040?ooo`<000001`3oool3000000<0oooo0`00000J0?ooo`<00000
2`3oool;000000<0oooo0`0000050?ooo`<000000`3oool3000000H0oooo0`0000040?ooo`<00000
1@3oool3000000D0oooo0`00002V0?ooo`00o`3ooom?0?ooo`<000000`3oool3000000@0oooo2`00
00020?ooo`<000002`3oool3000000@0oooo0`0000070?ooo`<000000`3oool3000001X0oooo0`00
000;0?ooo`/000000`3oool3000000D0oooo0`0000030?ooo`<000001P3oool3000000@0oooo0`00
00050?ooo`<000001@3oool300000:H0oooo003o0?ooodh0oooo100000030?ooo`@000000`3oool3
000000D0oooo0`0000020?ooo`<000001@3oool2000000@0oooo0`0000040?ooo`@000001@3oool4
000000<0oooo0`00000J0?ooo`<000002`3oool3000000D0oooo0`0000030?ooo`<000001@3oool3
000000<0oooo0`0000060?ooo`<00000103oool3000000D0oooo0`0000050?ooo`<00000YP3oool0
0?l0ooooCP3oool3000000D0oooo0`0000030?ooo`@000000`3oool3000000<0oooo100000030?oo
o`@000000`3oool3000000D0oooo100000030?ooo`@00000103oool4000001T0oooo0`00000;0?oo
o`@000000`3oool3000000@0oooo100000030?ooo`@00000103oool3000000@0oooo100000040?oo
o`<000001@3oool4000000<0oooo1000002V0?ooo`00o`3ooom>0?ooo`<000001@3oool3000000@0
oooo2@0000040?ooo`T000000P3oool7000000<0oooo2`0000040?ooo`L000005P3oool3000000`0
oooo2@0000040?ooo`/00000103oool;00000080oooo1`0000030?ooo`/00000YP3oool00?l0oooo
C@3oool3000000L0oooo0`0000040?ooo`L000001P3oool800000080oooo1`0000040?ooo`T00000
1@3oool7000001H0oooo0`00000=0?ooo`L000001@3oool:000000H0oooo2P0000020?ooo`L00000
0`3oool:00000:L0oooo003o0?ooodd0oooo0`0000070?ooo`<000001@3oool5000000P0oooo1@00
00040?ooo`L000001P3oool5000000L0oooo0`000000103oool0000000000000000F0?ooo`<00000
3P3oool5000000H0oooo0`0000020?ooo`@00000203oool400000080oooo0`0000020?ooo`L00000
0`3oool300000080oooo1000002X0?ooo`00o`3ooom<0?ooo`@000001`3oool4000001`0oooo0`00
000a0?ooo`<00000=`3oool3000000D0oooo0`00002^0?ooo`00o`3ooom<0?ooo`<000002@3oool3
000001`0oooo0`00000a0?ooo`<00000=`3oool3000000D0oooo0`00002^0?ooo`00o`3ooom<0?oo
o`<000002@3oool3000001d0oooo0P00000a0?ooo`<00000>03oool2000000D0oooo0`00002^0?oo
o`00o`3ooom;0?ooo`<000002`3oool3000001d0oooo00<000000?ooo`3oool0;`3oool3000003T0
oooo00<000000?ooo`3oool00`3oool300000:h0oooo003o0?ooool0ooooW`3oool00?l0ooooo`3o
oonO0?ooo`00o`3ooooo0?oooil0oooo003o0?ooool0ooooW`3oool00?l0ooooo`3ooonO0?ooo`00
o`3ooooo0?oooil0oooo003o0?ooool0ooooW`3oool00?l0ooooo`3ooonO0?ooo`00o`3ooooo0?oo
oil0oooo003o0?ooool0ooooW`3oool00=80oooo1@00001A0?ooo`D00000203oool5000000P0oooo
1@0000100?ooo`<00000203oool5000000P0oooo1@0000080?ooo`D00000>P3oool3000000P0oooo
1@0000080?ooo`D00000203oool5000003@0oooo2`0000050?ooo`D00000203oool5000000P0oooo
1@00000U0?ooo`00d@3oool7000004l0oooo1`0000060?ooo`L000001P3oool7000003l0oooo0`00
00070?ooo`L000001P3oool7000000H0oooo1`00000i0?ooo`<000001`3oool7000000H0oooo1`00
00060?ooo`L00000<`3oool;000000@0oooo1`0000060?ooo`L000001P3oool7000002@0oooo003@
0?ooo`T00000C@3oool9000000@0oooo2@0000040?ooo`T00000?P3oool3000000H0oooo2@000004
0?ooo`T00000103oool9000003P0oooo0`0000060?ooo`T00000103oool9000000@0oooo2@00000c
0?ooo`X000000`3oool9000000@0oooo2@0000040?ooo`T000008`3oool00=00oooo0`0000030?oo
o`<00000C03oool4000000<0oooo0`0000040?ooo`<000000`3oool3000000@0oooo0`0000030?oo
o`<00000?P3oool3000000H0oooo0`0000030?ooo`<00000103oool3000000<0oooo0`0000040?oo
o`<000000`3oool3000003P0oooo0`0000050?ooo`@000000`3oool3000000@0oooo0`0000030?oo
o`<00000103oool3000000<0oooo0`00000c0?ooo`@000002@3oool3000000<0oooo0`0000040?oo
o`<000000`3oool3000000@0oooo0`0000030?ooo`<000008`3oool00<l0oooo0`0000050?ooo`<0
0000B`3oool3000000D0oooo0`0000020?ooo`<000001@3oool300000080oooo0`0000050?ooo`<0
0000?@3oool3000000D0oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`0000020?ooo`<0
00001@3oool3000003L0oooo0`0000050?ooo`<000001@3oool300000080oooo0`0000050?ooo`<0
00000P3oool3000000D0oooo0`00000c0?ooo`@000001`3oool3000000D0oooo0`0000020?ooo`<0
00001@3oool300000080oooo0`0000050?ooo`<000008P3oool00<l0oooo0`0000050?ooo`<00000
B`3oool3000000D0oooo0`0000020?ooo`<000001@3oool300000080oooo0`0000050?ooo`<00000
?@3oool3000000D0oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`0000020?ooo`<00000
1@3oool3000003L0oooo0`0000050?ooo`<000001@3oool300000080oooo0`0000050?ooo`<00000
0P3oool3000000D0oooo0`00000d0?ooo`@000001P3oool3000000D0oooo0`0000020?ooo`<00000
1@3oool300000080oooo0`0000050?ooo`<000008P3oool00<l0oooo0`0000050?ooo`<00000D`3o
ool300000080oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`00000m0?ooo`<000001@3o
ool3000000D0oooo0`0000020?ooo`<000001@3oool300000080oooo0`0000050?ooo`<00000=`3o
ool3000000d0oooo0`0000020?ooo`<000001@3oool300000080oooo0`0000050?ooo`<00000=@3o
ool4000000D0oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`0000020?ooo`<000001@3o
ool300000280oooo003?0?ooo`<000001@3oool3000005<0oooo0`0000020?ooo`<000001@3oool3
00000080oooo0`0000050?ooo`<00000?@3oool3000000D0oooo0`0000050?ooo`<000000P3oool3
000000D0oooo0`0000020?ooo`<000001@3oool3000003L0oooo0`00000=0?ooo`<000000P3oool3
000000D0oooo0`0000020?ooo`<000001@3oool3000003H0oooo100000040?ooo`<000001@3oool3
00000080oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`00000R0?ooo`00c`3oool30000
00D0oooo0`00001;0?ooo`<00000103oool400000080oooo0`0000050?ooo`<000000P3oool30000
00D0oooo0`00000m0?ooo`<000001@3oool3000000D0oooo0`0000020?ooo`<000001@3oool30000
0080oooo0`0000050?ooo`<00000=`3oool3000000D0oooo0`0000040?ooo`@000000P3oool30000
00D0oooo0`0000020?ooo`<000001@3oool3000003L0oooo100000030?ooo`<000001@3oool30000
0080oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`00000R0?ooo`00c`3oool3000000D0
oooo0`00001;0?ooo`X000000`3oool3000000D0oooo0`0000020?ooo`<000001@3oool3000003d0
oooo0`0000050?ooo`<000001@3oool300000080oooo0`0000050?ooo`<000000P3oool3000000D0
oooo0`00000g0?ooo`<000001@3oool:000000<0oooo0`0000050?ooo`<000000P3oool3000000D0
oooo0`00000h0?ooo`<000000`3oool3000000D0oooo0`0000020?ooo`<000001@3oool300000080
oooo0`0000050?ooo`<000008P3oool00<l0oooo0`0000050?ooo`<00000C03oool8000000@0oooo
0`0000050?ooo`<000000P3oool3000000D0oooo0`00000i0?ooo`040000003oool0oooo0?ooo`<0
00001@3oool3000000D0oooo0`0000020?ooo`<000001@3oool300000080oooo0`0000050?ooo`<0
0000<`3oool010000000oooo0?ooo`3oool3000000H0oooo200000040?ooo`<000001@3oool30000
0080oooo0`0000050?ooo`<00000>@3oool300000080oooo0`0000050?ooo`<000000P3oool30000
00D0oooo0`0000020?ooo`<000001@3oool300000280oooo003?0?ooo`<000001@3oool3000004`0
oooo1`0000050?ooo`<000001@3oool300000080oooo0`0000050?ooo`<00000>@3oool300000004
0?ooo`0000000000000000D0oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`0000020?oo
o`<000001@3oool3000003<0oooo0`000000103oool000000000000000060?ooo`L000001@3oool3
000000D0oooo0`0000020?ooo`<000001@3oool300000340oooo0`0000050?ooo`<000000P3oool3
000000D0oooo0`0000020?ooo`<000001@3oool300000080oooo0`0000050?ooo`<000008P3oool0
0<l0oooo0`0000050?ooo`<00000C03oool3000000T0oooo0`0000050?ooo`<000000P3oool30000
00D0oooo0`00000i0?ooo`L000001@3oool3000000D0oooo0`0000020?ooo`<000001@3oool30000
0080oooo0`0000050?ooo`<00000<`3oool7000000H0oooo0`0000090?ooo`<000001@3oool30000
0080oooo0`0000050?ooo`<00000<@3oool3000000D0oooo0`0000020?ooo`<000001@3oool30000
0080oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`00000R0?ooo`00d03oool3000000<0
oooo0`00001=0?ooo`<000002P3oool3000000<0oooo0`0000040?ooo`<000000`3oool3000003/0
oooo1P0000060?ooo`<000000`3oool3000000@0oooo0`0000030?ooo`<00000103oool3000000<0
oooo0`00000e0?ooo`H000001P3oool3000000X0oooo0`0000030?ooo`<00000103oool3000000<0
oooo0`00000b0?ooo`@000000`3oool4000000<0oooo0`0000030?ooo`<00000103oool3000000<0
oooo0`0000040?ooo`<000000`3oool3000002<0oooo003@0?ooo`T00000C@3oool9000000@0oooo
2@0000040?ooo`T00000?@3oool4000000H0oooo2@0000040?ooo`T00000103oool9000003L0oooo
100000060?ooo`T00000103oool9000000@0oooo2@00000c0?ooo`T00000103oool9000000@0oooo
2@0000040?ooo`T000008`3oool00=40oooo1`00001?0?ooo`P000001@3oool7000000H0oooo1`00
000o0?ooo`<000001`3oool7000000H0oooo1`0000060?ooo`L00000>@3oool3000000L0oooo2000
00050?ooo`L000001P3oool7000003D0oooo200000050?ooo`L000001P3oool7000000H0oooo1`00
000T0?ooo`00dP3oool500000500oooo200000060?ooo`D00000203oool500000400oooo0`000008
0?ooo`D00000203oool5000000P0oooo1@00000j0?ooo`<000001`3oool8000000H0oooo1@000008
0?ooo`D00000=`3oool5000000P0oooo1@0000080?ooo`D00000203oool5000002D0oooo003o0?oo
ool0ooooW`3oool00?l0ooooo`3ooonO0?ooo`00o`3ooooo0?oooil0oooo003o0?ooool0ooooW`3o
ool00?l0ooooo`3ooonO0?ooo`00o`3ooooo0?oooil0oooo003o0?ooool0ooooW`3oool00<X0oooo
o`00002`000002@0oooo003:0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool04@3o
ool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool04@3o
ool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool04@3o
ool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool04@3o
ool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0403o
ool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool04@3o
ool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool04@3o
ool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo0P00000T0?ooo`00^03oool5
000000d0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0
oooo0140oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0
oooo0140oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0
oooo0140oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0
oooo0140oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0
oooo0140oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0
oooo0140oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0
oooo0140oooo00<000000?ooo`3oool04@3oool2000002@0oooo002g0?ooo`L00000303oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo0640oooo00<000000?ooo`3oool0H@3oool00`00
0000oooo0?ooo`1P0?ooo`030000003oool0oooo0640oooo00<000000?ooo`3oool04P3oool00`00
0000oooo0?ooo`0R0?ooo`00]P3oool9000000/0oooo00<000000?ooo`3oool01`3oool00`1oOgl0
oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0
oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool00;H0oooo
0`0000030?ooo`<000002`3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo0640oooo
00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo
00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`00]@3oool3000000D0oooo0`00
000:0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?oo
o`1Q0?ooo`0307moO`3oool0oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?oo
o`0B0?ooo`030000003oool0oooo0280oooo002e0?ooo`<000001@3oool3000000X0oooo1000003o
07moOjP0Ogmo1000000T0?ooo`00]@3oool3000000D0oooo0`00000:0?ooo`030000003oool0oooo
00L0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo
0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo
0280oooo002e0?ooo`<000001@3oool3000000X0oooo00<000000?ooo`3oool01`3oool00`1oOgl0
oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0
oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool00;D0oooo
0`0000050?ooo`<000002P3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo04h0oooo
o`3<I/a;0<aVc0T0oooo00<000000?ooo`3oool08P3oool00;D0oooo0`0000050?ooo`<000002P3o
ool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo03/0oooo4`3<I/`C0?ooo`0307moO`3o
ool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3o
ool0oooo0180oooo00<000000?ooo`3oool08P3oool00;D0oooo0`0000050?ooo`<000002P3oool0
0`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo01H0oooo9@3<I/`V0?ooo`0307moO`3oool0
oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0
oooo0180oooo00<000000?ooo`3oool08P3oool00;D0oooo0`0000050?ooo`<000002P3oool00`00
0000oooo0?ooo`070?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H@3oool00`1o
Ogl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`00
0000oooo0?ooo`0R0?ooo`00]@3oool3000000D0oooo0`00000:0?ooo`030000003oool0oooo00L0
oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0600
oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo0280
oooo002f0?ooo`<000000`3oool3000000/0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo
0?ooo`1Q0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo
0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool00;H0oooo2@00
000;0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?oo
o`1Q0?ooo`0307moO`3oool0oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?oo
o`0B0?ooo`030000003oool0oooo0280oooo002g0?ooo`L00000303oool00`000000oooo0?ooo`07
0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1P
0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R
0?ooo`00^03oool5000000d0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`1Q0?oo
o`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?oo
o`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool00<X0oooo0`0000070?ooo`03
07moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1P0?ooo`03
07moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0403oool3000002@0oooo003:0?ooo`030000
003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307mo
O`3oool0oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000
003oool0oooo0280oooo003:0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool0H@3o
ool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0600oooo00<0Ogmo0?ooo`3oool0H@3o
ool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo0280oooo003:0?ooo`030000003oool0
oooo00L0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0
oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0
oooo0280oooo003:0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1o
Ogl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1o
Ogl0oooo0?ooo`0B0?ooo`030000003oool0oooo0280oooo003:0?ooo`030000003oool0oooo00L0
oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0600
oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo0280
oooo003:0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo
0?ooo`1Q0?ooo`0307moO`3oool0oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo
0?ooo`0B0?ooo`030000003oool0oooo0280oooo003:0?ooo`030000003oool0oooo00L0oooo00<0
Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0600oooo00<0
Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo0280oooo003:
0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1Q
0?ooo`0307moO`3oool0oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B
0?ooo`030000003oool0oooo0280oooo003:0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?oo
o`3oool0H@3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0600oooo00<0Ogmo0?oo
o`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo0280oooo003:0?ooo`03
0000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1Q0?ooo`03
07moO`3oool0oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`03
0000003oool0oooo0280oooo003:0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool0
H@3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0600oooo00<0Ogmo0?ooo`3oool0
H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo0280oooo003:0?ooo`030000003o
ool0oooo00L0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3o
ool0oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003o
ool0oooo0280oooo003:0?ooo`<000001`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0
oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0
oooo0100oooo0`00000T0?ooo`00bP3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo
0640oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo
0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`00bP3oool00`000000
oooo0?ooo`070?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0
oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000
oooo0?ooo`0R0?ooo`00bP3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo0640oooo
00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo
00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`00bP3oool00`000000oooo0?oo
o`070?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?oo
o`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?oo
o`0R0?ooo`00bP3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo
0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo
0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`00bP3oool00`000000oooo0?ooo`070?oo
o`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1P0?oo
o`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?oo
o`00bP3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3o
ool0H@3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3o
ool04P3oool00`000000oooo0?ooo`0R0?ooo`00bP3oool00`000000oooo0?ooo`070?ooo`0307mo
O`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307mo
O`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`00bP3o
ool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H@3o
ool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3o
ool00`000000oooo0?ooo`0R0?ooo`00bP3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0
oooo0640oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0
oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`00bP3oool00`00
0000oooo0?ooo`070?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H@3oool00`1o
Ogl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`00
0000oooo0?ooo`0R0?ooo`00bP3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo0640
oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640
oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`00bP3oool3000000L0oooo
00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0600oooo
00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0@0?ooo`<00000903oool00<X0oooo00<0
00000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0640oooo00<0
Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo00<0
00000?ooo`3oool08P3oool00<X0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`1Q
0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q
0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool00<X0oooo00<000000?oo
o`3oool01`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?oo
o`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo00<000000?oo
o`3oool08P3oool00<X0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`03
07moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`03
07moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool00<X0oooo00<000000?ooo`3oool0
1`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0
H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool0
8P3oool00<X0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3o
ool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3o
ool0oooo0180oooo00<000000?ooo`3oool08P3oool00<X0oooo00<000000?ooo`3oool01`3oool0
0`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool0
0`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool0
08d0oooo2`0000050?ooo`D00000203oool5000000P0oooo1@00000>0?ooo`030000003oool0oooo
00L0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo
0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo
0280oooo002=0?ooo`/00000103oool7000000H0oooo1`0000060?ooo`L000003@3oool00`000000
oooo0?ooo`070?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0
oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000
oooo0?ooo`0R0?ooo`00SP3oool:000000<0oooo2@0000040?ooo`T00000103oool9000000`0oooo
00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0640oooo
00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo
00<000000?ooo`3oool08P3oool008h0oooo100000090?ooo`<000000`3oool3000000@0oooo0`00
00030?ooo`<00000103oool3000000<0oooo0`00000<0?ooo`030000003oool0oooo00L0oooo00<0
Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0600oooo00<0
Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo0280oooo002?
0?ooo`@000001`3oool3000000D0oooo0`0000020?ooo`<000001@3oool300000080oooo0`000005
0?ooo`<000002`3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo
0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo
0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`00T03oool4000000H0oooo0`0000050?oo
o`<000000P3oool3000000D0oooo0`0000020?ooo`<000001@3oool3000000/0oooo1000003o07mo
OjP0Ogmo1000000T0?ooo`00T@3oool4000000D0oooo0`0000050?ooo`<000000P3oool3000000D0
oooo0`0000020?ooo`<000001@3oool3000000/0oooo00<000000?ooo`3oool01`3oool00`1oOgl0
oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0
oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool00980oooo
100000040?ooo`<000001@3oool300000080oooo0`0000050?ooo`<000000P3oool3000000D0oooo
0`00000;0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool05P3oool00`1Vc6H0oooo
0?ooo`180?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo
0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool009<0oooo1000
00030?ooo`<000001@3oool300000080oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`00
000;0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool05P3oool00`1Vc6H0oooo0?oo
o`180?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?oo
o`1Q0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool009@0oooo0`000003
0?ooo`<000001@3oool300000080oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`00000;
0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool05P3oool00`1Vc6H0oooo06K<IP18
0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q
0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool009D0oooo0`0000020?oo
o`<000001@3oool300000080oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`00000;0?oo
o`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool05P3oool0101Vc6H0oooo06K<IP1Vc6I7
0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q
0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool008d0oooo0`0000050?oo
o`<000000P3oool3000000D0oooo0`0000020?ooo`<000001@3oool300000080oooo0`0000050?oo
o`<000002`3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo01H0oooo00@0I/aV0?oo
o`1Vc6H0I/aVA`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0600oooo00<0Ogmo
0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo0280oooo002=0?oo
o`<000001@3oool300000080oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`0000020?oo
o`<000001@3oool3000000/0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`0F0?oo
o`0506K<IP3oool0I/aV0?ooo`1Vc6H0AP3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0
oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0
oooo0280oooo002=0?ooo`@000000`3oool4000000<0oooo0`0000030?ooo`<00000103oool30000
00<0oooo0`0000040?ooo`<000000`3oool3000000`0oooo00<000000?ooo`3oool01`3oool00`1o
Ogl0oooo0?ooo`0G0?ooo`80I/aV0P3oool406K<IT80oooo00<0Ogmo0?ooo`3oool0H@3oool00`1o
Ogl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`00
0000oooo0?ooo`0R0?ooo`00SP3oool9000000@0oooo2@0000040?ooo`T00000103oool9000000`0
oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`0G0?ooo`0306K<IP3oool0oooo00D0
oooo1P1Vc6Hl0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0
oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool008l0oooo
200000050?ooo`L000001P3oool7000000H0oooo1`00000=0?ooo`030000003oool0oooo00L0oooo
00<0Ogmo0?ooo`3oool05`3oool00`1Vc6H0oooo0?ooo`0;0?ooo`L0I/aV0`3oool00`1Vc6H0oooo
06K<IP0206K<IRd0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307mo
O`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`00T03o
ool5000000P0oooo1@0000080?ooo`D00000203oool5000000h0oooo00<000000?ooo`3oool01`3o
ool00`1oOgl0oooo0?ooo`0G0?ooo`0306K<IP3oool0oooo0180oooo101Vc6H0103oool0I/aV0?oo
o`3oool606K<IRL0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307mo
O`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`00bP3o
ool3000000L0oooo00<0Ogmo0?ooo`3oool05P3oool00`1VI_l0I/aV0?ooo`0F0?ooo`0306K<IP3o
ool0I/aV00P0oooo1P1Vc6HQ0?ooo`0307moO`3oool0oooo0080I/aVG`3oool00`1oOgl0oooo0?oo
o`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0403oool3000002@0oooo003:
0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool05P3oool00`1VI_l0I/aV0?ooo`0F
0?ooo`0306K<IP3oool0I/aV00h0oooo1`1Vc6HJ0?ooo`0507moO`3oool0oooo06K<IP3oool00P1V
c6IM0?ooo`0307moO`3oool0oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?oo
o`0B0?ooo`030000003oool0oooo0280oooo003:0?ooo`030000003oool0oooo00L0oooo00<0Ogmo
0?ooo`3oool05P3oool00`1VI_l0oooo0?ooo`0F0?ooo`0306K<IP3oool0I/aV01D0oooo1P1Vc6HD
0?ooo`0407moO`3oool0oooo06K<IP<0oooo0P1Vc6IK0?ooo`0307moO`3oool0oooo0600oooo00<0
Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo0280oooo003:
0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool05P3oool00`1VI_l0oooo0?ooo`0G
0?ooo`0306K<IP3oool0oooo01X0oooo1`1Vc6H90?ooo`H0I/aV00<0oooo06K<IP3oool0103oool6
06K<IUD0oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0
oooo0180oooo00<000000?ooo`3oool08P3oool00<X0oooo00<000000?ooo`3oool01`3oool00`1o
Ogl0oooo0?ooo`0F0?ooo`0306IVo`3oool0oooo01L0oooo00<0I/aV0?ooo`3oool08@3oool906K<
IP@0oooo00@0Ogmo06K<IP3oool0I/aV2`3oool:06K<IT/0oooo00<0Ogmo0?ooo`3oool00P3oool2
06K<IU`0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0
oooo0280oooo003:0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool05P3oool00`1V
I_l0oooo0?ooo`0G0?ooo`0306K<IP3oool0oooo02h0oooo00<0Ogmo06K<IP1Vc6H05P3oool?06K<
IS`0oooo00<0Ogmo0?ooo`3oool00P3oool0101Vc6H0oooo06K<IP1Vc6IJ0?ooo`0307moO`3oool0
oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`00bP3oool00`00
0000oooo0?ooo`070?ooo`0307moO`3oool0oooo01H0oooo00<0IVKo0?ooo`3oool05`3oool00`1V
c6H0oooo0?ooo`0^0?ooo`0307moO`1Vc6H0I/aV02D0oooo501Vc6HX0?ooo`@0I/aV00<0oooo06K<
IP3oool00P3oool606K<IU@0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`080?oo
o`0306K<IP3oool0oooo00L0oooo00<000000?ooo`3oool08P3oool00<X0oooo00<000000?ooo`3o
ool01`3oool00`1oOgl0oooo0?ooo`0F0?ooo`0306IVo`3oool0oooo01L0oooo00<0I/aV0?ooo`3o
ool0;P3oool00`1oOgl0oooo06K<IP0i0?oooa@0I/aV3P3oool606K<IP0607moO`3oool0oooo06K<
IP3oool0I/aV2@3oool:06K<ITX0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`03
0?ooo`@0I/aV00<0oooo06K<IP3oool0203oool00`000000oooo0?ooo`0R0?ooo`00bP3oool00`00
0000oooo0?ooo`070?ooo`0307moO`3oool0oooo01H0oooo00<0IVKo0?ooo`3oool0B03oool00`1o
Ogl0oooo06K<IP1=0?ooo`h0I/aV1P3oool00`1oOgl0oooo0?ooo`0206K<IQ@0oooo3`1Vc6Hk0?oo
o`0307moO`3oool0oooo00<0oooo2@1Vc6IE0?ooo`H0I/aV0`3oool00`1Vc6H0oooo06K<IP090?oo
o`030000003oool0oooo0280oooo003:0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3o
ool05P3oool00`1VI_l0oooo0?ooo`180?ooo`0307moO`3oool0I/aV0640oooo00D0Ogmo0?ooo`3o
ool0oooo06K<IP0S0?ooobL0I/aV403oool:06K<IPT0oooo2P1Vc6HT0?oooa<0I/aV3P3oool606K<
IP0307moO`3oool0oooo00H0oooo0P1Vc6H:0?ooo`030000003oool0oooo0280oooo003:0?ooo`03
0000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool05P3oool00`1VI_l0oooo0?ooo`180?ooo`03
07moO`3oool0oooo0640oooo00D0Ogmo0?ooo`3oool0oooo06K<IP1:0?oooa00I/aV103oool00`1o
Ogl0oooo0?ooo`0F0?ooob@0I/aV4`3oool>06K<IPH0oooo00<0Ogmo0?ooo`3oool01`3oool00`1V
c6H0oooo0?ooo`080?ooo`030000003oool0oooo0280oooo003:0?ooo`030000003oool0oooo00L0
oooo00<0Ogmo0?ooo`3oool05P3oool00`1VI_l0oooo0?ooo`180?ooo`0307moO`3oool0oooo0640
oooo00D0Ogmo0?ooo`3oool0oooo06K<IP1N0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?oo
o`3oool01`3oool00`1Vc6H0oooo0?ooo`080?ooo`030000003oool0oooo0280oooo003:0?ooo`03
0000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool05P3oool00`1VI_l0oooo0?ooo`180?ooo`03
07moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`03
07moO`3oool0oooo00L0oooo00<0I/aV0?ooo`3oool0203oool00`000000oooo0?ooo`0R0?ooo`00
bP3oool3000000L0oooo00<0Ogmo0?ooo`3oool05P3oool00`1VI_l0oooo0?ooo`180?ooo`0307mo
O`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307mo
O`3oool0oooo0100oooo0`00000T0?ooo`00bP3oool00`000000oooo0?ooo`070?ooo`0307moO`3o
ool0oooo01H0oooo00<0IVKo0?ooo`3oool0B03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3o
ool0oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003o
ool0oooo0280oooo003:0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool05P3oool0
0`1VI_l0oooo0?ooo`180?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool0
0`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool0
0<X0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`0F0?ooo`0306IVo`3oool0oooo
04P0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo
0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`00bP3oool00`000000
oooo0?ooo`070?ooo`0307moO`3oool0oooo01H0oooo00<0IVKo0?ooo`3oool0B03oool00`1oOgl0
oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0
oooo0?ooo`0B0?ooo`030000003oool0oooo0280oooo003:0?ooo`030000003oool0oooo00L0oooo
00<0Ogmo0?ooo`3oool05P3oool00`1VI_l0oooo0?ooo`180?ooo`0307moO`3oool0oooo0640oooo
00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo
00<000000?ooo`3oool08P3oool00<X0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?oo
o`0G0?ooo`0306IVo`3oool0oooo04L0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?oo
o`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?oo
o`0R0?ooo`00bP3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo01L0oooo00<0IVKo
0?ooo`3oool0A`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0600oooo00<0Ogmo
0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo0280oooo003:0?oo
o`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool05P3oool00`3oIVH0IVKo0?ooo`180?oo
o`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?oo
o`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool00<X0oooo00<000000?ooo`3o
ool01`3oool00`1oOgl0oooo0?ooo`0F0?ooo`030?mVIP1VI_l0oooo04P0oooo00<0Ogmo0?ooo`3o
ool0H@3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3o
ool04P3oool00`000000oooo0?ooo`0R0?ooo`00bP3oool00`000000oooo0?ooo`070?ooo`0307mo
O`3oool0oooo01H0oooo00<0ofIV06IVo`1VI_l0B03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307mo
O`3oool0oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000
003oool0oooo0280oooo003:0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool05P3o
ool00`3oIVH0IVKo06IVo`180?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03o
ool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3o
ool00<X0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`0F0?ooo`040?mVIP1VI_l0
IVKo06IVodL0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3o
ool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`00bP3oool3
000000L0oooo00<0Ogmo0?ooo`3oool05P3oool0103oIVH0IVKo06IVo`1VI_m70?ooo`0307moO`3o
ool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3o
ool0oooo0100oooo0`00000T0?ooo`00bP3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0
oooo01H0oooo00D0ofIV06IVo`1VI_l0oooo06IVo`160?ooo`0307moO`3oool0oooo0640oooo00<0
Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo00<0
00000?ooo`3oool08P3oool00<X0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`0F
0?ooo`050?mVIP1VI_l0IVKo0?ooo`1VI_l0AP3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3o
ool0oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003o
ool0oooo0280oooo003:0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool05P3oool0
0`3oIVH0IVKo06IVo`020?ooo`0306IVo`3oool0oooo04<0oooo00<0Ogmo0?ooo`3oool0H@3oool0
0`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool0
0`000000oooo0?ooo`0R0?ooo`00bP3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo
01H0oooo00<0ofIV06IVo`3oool00P3oool00`1VI_l0oooo0?ooo`130?ooo`0307moO`3oool0oooo
0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo
0180oooo00<000000?ooo`3oool08P3oool00<X0oooo00<000000?ooo`3oool01`3oool00`1oOgl0
oooo0?ooo`0F0?ooo`030?mVIP1VI_l0oooo00<0oooo00<0IVKo0?ooo`3oool0@P3oool00`1oOgl0
oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0
oooo0?ooo`0B0?ooo`030000003oool0oooo0280oooo003:0?ooo`030000003oool0oooo00L0oooo
00<0Ogmo0?ooo`3oool05P3oool00`3oIVH0IVKo0?ooo`040?ooo`80IVKo@P3oool00`1oOgl0oooo
0?ooo`1Q0?ooo`0307moO`3oool0oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo
0?ooo`0B0?ooo`030000003oool0oooo0280oooo003:0?ooo`030000003oool0oooo00L0oooo00<0
Ogmo0?ooo`3oool05P3oool00`3oIVH0IVKo0?ooo`060?ooo`0306IVo`3oool0oooo03l0oooo00<0
Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo00<0
Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`00T`3oool3000000L0oooo1@000008
0?ooo`D00000203oool5000000h0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`0F
0?ooo`030?mVIP1VI_l0oooo00L0oooo00<0IVKo0?ooo`3oool0?P3oool00`1oOgl0oooo0?ooo`1Q
0?ooo`0307moO`3oool0oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B
0?ooo`030000003oool0oooo0280oooo002C0?ooo`<000001P3oool7000000H0oooo1`0000060?oo
o`L000003@3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo01H0oooo00<0ofIV06IV
o`3oool0203oool206IVoch0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1P0?oo
o`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?oo
o`00T`3oool3000000D0oooo2@0000040?ooo`T00000103oool9000000`0oooo00<000000?ooo`3o
ool01`3oool00`1oOgl0oooo0?ooo`0F0?ooo`030?mVIP1VI_l0oooo00X0oooo00<0IVKo0?ooo`3o
ool0>`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0600oooo00<0Ogmo0?ooo`3o
ool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo0280oooo002<0?ooo``00000
0`3oool3000000<0oooo0`0000040?ooo`<000000`3oool3000000@0oooo0`0000030?ooo`<00000
303oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo01H0oooo00<0ofIV0?ooo`3oool0
2`3oool00`1VI_l0oooo0?ooo`0j0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0
H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool0
8P3oool008`0oooo300000020?ooo`<000001@3oool300000080oooo0`0000050?ooo`<000000P3o
ool3000000D0oooo0`00000;0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool05P3o
ool00`3oIVH0oooo0?ooo`0<0?ooo`80IVKo>P3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3o
ool0oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003o
ool0oooo0280oooo002<0?ooo``000000P3oool3000000D0oooo0`0000020?ooo`<000001@3oool3
00000080oooo0`0000050?ooo`<000002`3oool4000001l0Ogmo00<0ofIV07moO`1oOgl03P1oOgl0
0`1VI_l0Ogmo07moO`3o07moOgD0Ogmo1000000T0?ooo`00S03oool3000000@0oooo0`0000040?oo
o`<000001@3oool300000080oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`00000;0?oo
o`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool05P3oool00`3oIVH0oooo0?ooo`0?0?oo
o`0306IVo`3oool0oooo03H0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1P0?oo
o`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?oo
o`00S@3oool3000000<0oooo0`0000040?ooo`<000001@3oool300000080oooo0`0000050?ooo`<0
00000P3oool3000000D0oooo0`00000;0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3o
ool05P3oool00`3oIVH0oooo0?ooo`0@0?ooo`80IVKo=P3oool00`1oOgl0oooo0?ooo`1Q0?ooo`03
07moO`3oool0oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`03
0000003oool0oooo0280oooo002>0?ooo`<000000P3oool3000000@0oooo0`0000050?ooo`<00000
0P3oool3000000D0oooo0`0000020?ooo`<000001@3oool3000000/0oooo00<000000?ooo`3oool0
1`3oool00`1oOgl0oooo0?ooo`0F0?ooo`030?mVIP3oool0oooo0180oooo00D0IVKo0?ooo`3oool0
oooo06IVo`0a0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0
oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool008h0oooo
0`0000020?ooo`<00000103oool3000000D0oooo0`0000020?ooo`<000001@3oool300000080oooo
0`0000050?ooo`<000002`3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo01H0oooo
00<0ofIV0?ooo`3oool04`3oool0101VI_l0oooo0?ooo`1VI_la0?ooo`0307moO`3oool0oooo0640
oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180
oooo00<000000?ooo`3oool08P3oool008l0oooo0`000000103oool000000000000000040?ooo`<0
00001@3oool300000080oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`00000;0?ooo`03
0000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool05P3oool00`3oIVH0oooo0?ooo`020?ooo`03
0?mVIP3oool0oooo00l0oooo0`1VI_l00`3oool0IVKo0?ooo`0^0?ooo`0307moO`3oool0oooo0640
oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180
oooo00<000000?ooo`3oool08P3oool00900oooo1P0000040?ooo`<000001@3oool300000080oooo
0`0000050?ooo`<000000P3oool3000000D0oooo0`00000;0?ooo`030000003oool0oooo00L0oooo
00<0Ogmo0?ooo`3oool05`3oool01@3oIVH0oooo0?ooo`3oool0ofIV01<0oooo00<0IVKo0?ooo`1V
I_l00P1VI_l]0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0
oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool00900oooo
1P0000040?ooo`<000001@3oool300000080oooo0`0000050?ooo`<000000P3oool3000000D0oooo
0`00000;0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool05`3oool0103oIVH0oooo
0?ooo`3oool20?mVIQ80oooo00<0IVKo0?ooo`1VI_l00P3oool206IVob/0oooo00<0Ogmo0?ooo`3o
ool0H@3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3o
ool04P3oool00`000000oooo0?ooo`0R0?ooo`00T@3oool5000000D0oooo0`0000030?ooo`<00000
103oool3000000<0oooo0`0000040?ooo`<000000`3oool3000000`0oooo00<000000?ooo`3oool0
1`3oool00`1oOgl0oooo0?ooo`0G0?ooo`060?mVIP3oool0oooo0?mVIP3oool0ofIV4P3oool00`1V
I_l0oooo06IVo`040?ooo`80IVKo:@3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo
0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo
0280oooo002B0?ooo`@000001@3oool9000000@0oooo2@0000040?ooo`T00000303oool00`000000
oooo0?ooo`070?ooo`0307moO`3oool0oooo01L0oooo00H0ofIV0?ooo`3oool0ofIV0?ooo`3oIVHB
0?ooo`0306IVo`3oool0IVKo00H0oooo0P1VI_lW0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo
0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo00<00000
0?ooo`3oool08P3oool00980oooo100000060?ooo`L000001P3oool7000000H0oooo1`00000=0?oo
o`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool05`3oool0103oIVH0oooo0?ooo`3oIVH2
0?ooo`030?mVIP3oool0oooo00l0oooo00<0IVKo0?ooo`1VI_l0203oool206IVobD0oooo00<0Ogmo
0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo
0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`00T`3oool3000000L0oooo1@0000080?oo
o`D00000203oool5000000h0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`0G0?oo
o`040?mVIP3oool0oooo0?mVIP80oooo00<0ofIV0?ooo`3oool0403oool00`1VI_l0oooo0?ooo`09
0?ooo`80IVKo8`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0600oooo00<0Ogmo
0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo0280oooo003:0?oo
o`<000001`3oool00`1oOgl0oooo0?ooo`0G0?ooo`030?mVIP3oool0ofIV00<0oooo00<0ofIV0?oo
o`3oool0403oool00`1VI_l0oooo0?ooo`0;0?ooo`80IVKo8@3oool00`1oOgl0oooo0?ooo`1Q0?oo
o`0307moO`3oool0oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0@0?oo
o`<00000903oool00<X0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`0G0?ooo`03
0?mVIP3oool0ofIV00@0oooo00<0ofIV0?ooo`3oool03`3oool00`1VI_l0oooo0?ooo`0=0?ooo`80
IVKo7`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0600oooo00<0Ogmo0?ooo`3o
ool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo0280oooo003:0?ooo`030000
003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool05`3oool00`3oIVH0oooo0?mVIP040?ooo`030?mV
IP3oool0oooo00l0oooo00<0IVKo0?ooo`3oool03`3oool306IVoa`0oooo00<0Ogmo0?ooo`3oool0
H@3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0
4P3oool00`000000oooo0?ooo`0R0?ooo`00bP3oool00`000000oooo0?ooo`070?ooo`0307moO`3o
ool0oooo01L0oooo00<0ofIV0?ooo`3oIVH01@3oool00`3oIVH0oooo0?ooo`0>0?ooo`0306IVo`3o
ool0oooo0180oooo0`1VI_lI0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03o
ool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3o
ool00<X0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`0G0?ooo`80ofIV1P3oool0
0`3oIVH0oooo0?ooo`0>0?ooo`0306IVo`3oool0oooo01D0oooo0`1VI_lF0?ooo`0307moO`3oool0
oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0
oooo0180oooo00<000000?ooo`3oool08P3oool00<X0oooo00<000000?ooo`3oool01`3oool00`1o
Ogl0oooo0?ooo`0G0?ooo`80ofIV1P3oool00`3oIVH0oooo0?ooo`0>0?ooo`0306IVo`3oool0oooo
01P0oooo0P1VI_lD0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1o
Ogl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool00<X0
oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`0G0?ooo`80ofIV1`3oool00`3oIVH0
oooo0?ooo`0Z0?ooo`<0IVKo4@3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0600
oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo0280
oooo000E0?ooo`<00000103oool3000000@0oooo0`0000040?ooo`<00000303oool<000000L0oooo
1P0000080?ooo`<00000403oool7000005H0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo
0?ooo`0G0?ooo`80ofIV1`3oool00`3oIVH0oooo0?ooo`0]0?ooo`<0IVKo3@3oool306IVof80oooo
00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo
00<000000?ooo`3oool08P3oool001D0oooo0`0000040?ooo`<00000103oool3000000@0oooo0`00
000<0?ooo``000001@3oool:000000H0oooo0`00000>0?ooo`X00000E@3oool00`000000oooo0?oo
o`070?ooo`0307moO`3oool0oooo01L0oooo0P3oIVH70?ooo`030?mVIP3oool0oooo0300oooo0`1V
I_l40?ooo`H0IVKo00@0oooo07moO`1VI_l0oooo2@1VI_mH0?ooo`0307moO`3oool0oooo0600oooo
00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo0280oooo
000E0?ooo`<000000`3oool5000000<0oooo0`0000040?ooo`<00000303oool<000000@0oooo3000
00050?ooo`<000003@3oool<000005@0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?oo
o`0G0?ooo`80ofIV203oool00`3oIVH0oooo0?ooo`0b0?ooo`@0IVKo1`3oool0101oOgl0IVKo0?oo
o`1VI_l80?ooo`<0IVKoE@3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo
00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`005@3oool3000000<0oooo1@00
00030?ooo`<00000103oool3000000`0oooo0`00000<0?ooo`D00000103oool5000000@0oooo0`00
000=0?ooo`@000001@3oool4000005<0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?oo
o`0G0?ooo`80ofIV203oool00`3oIVH0oooo0?ooo`0m0?ooo`0407moO`1VI_l0oooo06IVo`/0oooo
101VI_mA0?ooo`0307moO`3oool0oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo
0?ooo`0B0?ooo`030000003oool0oooo0280oooo000E0?ooo`<000000`3oool5000000<0oooo0`00
00040?ooo`<00000303oool3000000`0oooo0`0000080?ooo`<00000103oool3000000`0oooo1000
00070?ooo`<00000D`3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo01L0oooo00<0
ofIV0?ooo`3oool01`3oool00`3oIVH0oooo0?ooo`0m0?ooo`0407moO`1VI_l0oooo06IVo`l0oooo
0`1VI_m>0?ooo`0307moO`3oool0oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo
0?ooo`0B0?ooo`030000003oool0oooo0280oooo000E0?ooo`<000000`3oool2000000030?ooo`00
0000000000<0oooo0`0000040?ooo`<00000303oool3000000/0oooo100000080?ooo`@000000`3o
ool3000000`0oooo0`0000080?ooo`<00000D`3oool00`000000oooo0?ooo`070?ooo`0307moO`3o
ool0oooo01L0oooo00<0ofIV0?ooo`3oool0203oool00`3oIVH0oooo0?ooo`0l0?ooo`0407moO`1V
I_l0oooo06IVoa80oooo0`1VI_m;0?ooo`0307moO`3oool0oooo0080oooo00<0IVKo0?ooo`3oool0
F`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool0
8P3oool001D0oooo0`0000020?ooo`<0000000@0oooo00000000000000000P3oool3000000@0oooo
0`00000<0?ooo`<000002`3oool3000000X0oooo0`0000030?ooo`X000003`3oool4000005<0oooo
0`0000070?ooo`0307moO`3oool0oooo01L0oooo00<0ofIV0?ooo`3oool0203oool00`3oIVH0oooo
0?ooo`0l0?ooo`0407moO`1VI_l0oooo06IVoaD0oooo101VI_m70?ooo`0307moO`3oool0oooo0080
oooo0P1VI_mL0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0403oool3000002@0
oooo000E0?ooo`<000000P3oool3000000040?ooo`000000000000000080oooo0`0000040?ooo`/0
0000103oool3000000/0oooo0`00000:0?ooo`<000000`3oool;000000/0oooo1P00001D0?ooo`03
0000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool05`3oool00`3oIVH0oooo0?ooo`080?ooo`03
0?mVIP3oool0oooo03`0oooo00<0Ogmo0?ooo`1VI_l06P3oool306IVod@0oooo00<0Ogmo0?ooo`3o
ool00P3oool0101VI_l0oooo06IVo`1VI_mJ0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?oo
o`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`005@3oool300000080oooo0`000000103oool0
00000000000000020?ooo`<00000103oool;000000@0oooo0`00000;0?ooo`<000002P3oool30000
00<0oooo300000070?ooo`T00000E03oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo
01L0oooo00<0ofIV0?ooo`3oool02@3oool00`3oIVH0oooo0?ooo`0k0?ooo`0307moO`3oool0IVKo
01d0oooo101VI_m00?ooo`0307moO`3oool0oooo0080oooo00D0IVKo0?ooo`3oool0oooo06IVo`1I
0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R
0?ooo`005@3oool300000080oooo0P0000030?ooo`8000000P3oool3000000@0oooo2`0000040?oo
o`<000002`3oool3000000X0oooo0`0000030?ooo`<000001P3oool4000000@0oooo2@00001F0?oo
o`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool05`3oool00`3oIVH0oooo0?ooo`090?oo
o`030?mVIP3oool0oooo03/0oooo00<0Ogmo0?ooo`1VI_l08@3oool406IVoc`0oooo00<0Ogmo0?oo
o`3oool00P3oool00`1VI_l0oooo0?ooo`020?ooo`80IVKoE`3oool00`1oOgl0oooo0?ooo`1Q0?oo
o`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool001D0oooo0`000000103oool0
00000000000000030?ooo`<0000000@0oooo0000000000000000103oool3000000`0oooo0`00000;
0?ooo`<000002P3oool3000000<0oooo0`0000070?ooo`<000000`3oool7000005T0oooo00<00000
0?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`0T0?ooo`030?mVIP3oool0oooo03X0oooo00<0Ogmo
0?ooo`1VI_l09@3oool406IVocP0oooo00<0Ogmo0?ooo`3oool00P3oool00`1VI_l0oooo0?ooo`04
0?ooo`<0IVKoE03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo00P0oooo00<0IVKo
0?ooo`3oool01`3oool00`000000oooo0?ooo`0R0?ooo`005@3oool3000000040?ooo`0000000000
000000<0oooo0`000000103oool000000000000000040?ooo`<00000303oool3000000/0oooo1000
00080?ooo`@000000`3oool3000000L0oooo0`0000030?ooo`@00000G03oool00`000000oooo0?oo
o`070?ooo`0307moO`3oool0oooo02@0oooo00<0ofIV0?ooo`3oool0>P3oool00`1oOgl0oooo06IV
o`0Y0?ooo`@0IVKo=03oool406IVo`030?ooo`1VI_l0oooo00P0oooo101VI_m@0?ooo`0307moO`3o
ool0oooo0640oooo00<0Ogmo0?ooo`3oool0203oool00`1VI_l0oooo0?ooo`070?ooo`030000003o
ool0oooo0280oooo000E0?ooo`<0000000<0oooo0000000000001@3oool2000000040?ooo`000000
0000000000@0oooo0`00000<0?ooo`<00000303oool3000000P0oooo0`0000040?ooo`<000001`3o
ool3000000<0oooo0`0000060?ooo`<00000E03oool00`000000oooo0?ooo`070?ooo`0307moO`3o
ool0oooo02@0oooo00<0ofIV0?ooo`3oool0>P3oool00`1oOgl0oooo06IVo`0]0?ooo``0IVKo8P3o
ool606IVo`0607moO`3oool0oooo06IVo`3oool0IVKo3@3oool306IVodd0oooo00<0Ogmo0?ooo`3o
ool0H@3oool00`1oOgl0oooo0?ooo`080?ooo`0306IVo`3oool0oooo00L0oooo00<000000?ooo`3o
ool08P3oool001D0oooo1P0000050?ooo`H00000103oool3000000`0oooo0`00000<0?ooo`D00000
103oool5000000@0oooo0`0000060?ooo`@000000`3oool4000000@0oooo1000001D0?ooo`030000
003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool09@3oool00`3oIVH0oooo0?ooo`0i0?ooo`0307mo
O`3oool0IVKo03T0oooo8P1VI_l60?ooo`0307moO`3oool0oooo0080IVKo4@3oool306IVodX0oooo
00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`050?ooo`80IVKo00<0oooo06IVo`3oool0
203oool00`000000oooo0?ooo`0R0?ooo`005@3oool6000000D0oooo1P0000040?ooo``000000`3o
ool3000000d0oooo300000050?ooo``000001@3oool:000005D0oooo00<000000?ooo`3oool01`3o
ool00`1oOgl0oooo0?ooo`0U0?ooo`030?mVIP3oool0oooo03T0oooo00<0Ogmo0?ooo`3oool0H@3o
ool00`1oOgl0oooo0?ooo`0206IVoa@0oooo101VI_m60?ooo`0307moO`3oool0oooo0640oooo00<0
Ogmo0?ooo`3oool00`3oool206IVo`040?ooo`1VI_l0oooo06IVo`T0oooo00<000000?ooo`3oool0
8P3oool001D0oooo1@0000070?ooo`D00000103oool<000000<0oooo0`00000>0?ooo`X000001P3o
ool<000000D0oooo2P00001E0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool09@3o
ool00`3oIVH0oooo0?ooo`0i0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool00P1V
I_lH0?oooaX0IVKo;03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0407moO`3oool0oooo0?ooo`80IVKo
0`3oool00`1VI_l0oooo06IVo`090?ooo`030000003oool0oooo0280oooo000E0?ooo`D000001`3o
ool5000000@0oooo300000030?ooo`<00000403oool6000000P0oooo2P0000090?ooo`H00000E`3o
ool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo02H0oooo00<0ofIV0?ooo`3oool0>03o
ool00`1oOgl0oooo0?ooo`1Q0?ooo`0507moO`3oool0oooo0?ooo`1VI_l0<P3oool:06IVob80oooo
00<0Ogmo0?ooo`3oool0H@3oool0101oOgl0IVKo06IVo`1VI_l50?ooo`0306IVo`3oool0IVKo00T0
oooo00<000000?ooo`3oool08P3oool00<X0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo
0?ooo`0V0?ooo`030?mVIP3oool0oooo03P0oooo00<0Ogmo0?ooo`3oool0H@3oool01@1oOgl0oooo
0?ooo`3oool0IVKo03`0oooo<@1VI_mD0?ooo`80IVKo203oool206IVo`X0oooo00<000000?ooo`3o
ool08P3oool00<X0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`0V0?ooo`030?mV
IP3oool0oooo00P0oooo00<0ofIV0?ooo`3oool0;@3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0507mo
O`3oool0oooo0?ooo`1VI_l0GP3oool00`1oOgl0oooo0?ooo`0<0?ooo`X0IVKoA`3oool306IVo`03
0?ooo`1oOgl0oooo00L0oooo0P1VI_l:0?ooo`030000003oool0oooo0280oooo003:0?ooo`<00000
1`3oool00`1oOgl0oooo0?ooo`0W0?ooo`030?mVIP3oool0oooo00L0oooo00<0ofIV0?ooo`3oool0
;@3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0507moO`3oool0oooo0?ooo`1VI_l0GP3oool00`1oOgl0
oooo0?ooo`0F0?ooo`l0IVKo5@3oool>06IVoa<0oooo0P1VI_l40?ooo`0307moO`3oool0oooo00L0
oooo00<0IVKo0?ooo`3oool01P3oool3000002@0oooo003:0?ooo`030000003oool0oooo00L0oooo
00<0Ogmo0?ooo`3oool09`3oool00`3oIVH0oooo0?ooo`070?ooo`80ofIV;P3oool00`1oOgl0oooo
0?ooo`1Q0?ooo`0507moO`3oool0oooo0?ooo`1VI_l0GP3oool00`1oOgl0oooo0?ooo`0U0?oooaD0
IVKo3P3oool:06IVo`L0oooo0P1VI_l60?ooo`0307moO`3oool0oooo00L0oooo00<0IVKo0?ooo`3o
ool0203oool00`000000oooo0?ooo`0R0?ooo`00bP3oool00`000000oooo0?ooo`070?ooo`0307mo
O`3oool0oooo02L0oooo00<0ofIV0?ooo`3oool01`3oool20?mVIRh0oooo00<0Ogmo0?ooo`3oool0
H@3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0580oooo1`1VI_l80?ooo`0307mo
O`3oool0oooo00L0oooo00<0IVKo0?ooo`3oool0203oool00`000000oooo0?ooo`0R0?ooo`00bP3o
ool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo02P0oooo00<0ofIV0?ooo`3oool01P3o
ool20?mVIRh0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3o
ool0oooo0640oooo00<0Ogmo0?ooo`3oool01`3oool00`1VI_l0oooo0?ooo`080?ooo`030000003o
ool0oooo0280oooo003:0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool0:03oool0
0`3oIVH0oooo0?ooo`060?ooo`030?mVIP3oool0ofIV02d0oooo00<0Ogmo0?ooo`3oool0H@3oool0
0`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool01`3oool0
0`1VI_l0oooo0?ooo`080?ooo`030000003oool0oooo0280oooo003:0?ooo`030000003oool0oooo
00L0oooo00<0Ogmo0?ooo`3oool0:@3oool00`3oIVH0oooo0?ooo`050?ooo`030?mVIP3oool0ofIV
02d0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo
0640oooo00<0Ogmo0?ooo`3oool01`3oool00`1VI_l0oooo0?ooo`080?ooo`030000003oool0oooo
0280oooo003:0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool0:@3oool00`3oIVH0
oooo0?ooo`050?ooo`040?mVIP3oool0oooo0?mVIR`0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1o
Ogl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`00
0000oooo0?ooo`0R0?ooo`00bP3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo02T0
oooo00<0ofIV0?ooo`3oool01@3oool0103oIVH0oooo0?ooo`3oIVH/0?ooo`0307moO`3oool0oooo
0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo
0180oooo00<000000?ooo`3oool08P3oool00900oooo1@0000080?ooo`D00000203oool5000000P0
oooo1@00000>0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool0:P3oool00`3oIVH0
oooo0?ooo`040?ooo`040?mVIP3oool0oooo0?mVIR`0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1o
Ogl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`00
0000oooo0?ooo`0R0?ooo`00S`3oool7000000H0oooo1`0000060?ooo`L000001P3oool7000000d0
oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`0Z0?ooo`030?mVIP3oool0oooo00@0
oooo00D0ofIV0?ooo`3oool0oooo0?mVIP0[0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?oo
o`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo00<000000?oo
o`3oool08P3oool008h0oooo2@0000040?ooo`T00000103oool9000000@0oooo2@00000<0?ooo`03
0000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool0:P3oool00`3oIVH0oooo0?ooo`040?ooo`05
0?mVIP3oool0oooo0?ooo`3oIVH0:`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo
0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo
0280oooo002>0?ooo`<000000`3oool4000000<0oooo0`0000030?ooo`<00000103oool3000000<0
oooo0`0000040?ooo`<000000`3oool3000000`0oooo00<000000?ooo`3oool01`3oool00`1oOgl0
oooo0?ooo`0[0?ooo`030?mVIP3oool0oooo00<0oooo00D0ofIV0?ooo`3oool0oooo0?mVIP0[0?oo
o`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?oo
o`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool008d0oooo0`0000050?ooo`<0
00000P3oool3000000D0oooo0`0000020?ooo`<000001@3oool300000080oooo0`0000050?ooo`<0
00002`3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo02/0oooo00<0ofIV0?ooo`3o
ool00`3oool00`3oIVH0oooo0?ooo`020?ooo`030?mVIP3oool0oooo02P0oooo00<0Ogmo0?ooo`3o
ool0H@3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3o
ool04P3oool00`000000oooo0?ooo`0R0?ooo`00S@3oool3000000D0oooo0`0000020?ooo`<00000
1@3oool300000080oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`00000;0?ooo`@00000
=01oOgl00`3oIVH0Ogmo07moO`0307moO`030?mVIP1oOgl0Ogmo0080Ogmo00<0ofIV07moO`1oOgl0
o`1oOgmV07moO`@00000903oool008d0oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`00
00020?ooo`<000001@3oool300000080oooo0`0000050?ooo`<000002`3oool00`000000oooo0?oo
o`070?ooo`0307moO`3oool0oooo02`0oooo00<0ofIV0?ooo`3oool00P3oool00`3oIVH0oooo0?oo
o`030?ooo`030?mVIP3oool0oooo02L0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?oo
o`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?oo
o`0R0?ooo`00S@3oool4000000<0oooo100000020?ooo`<000001@3oool300000080oooo0`000005
0?ooo`<000000P3oool3000000D0oooo0`00000;0?ooo`030000003oool0oooo00L0oooo00<0Ogmo
0?ooo`3oool0;03oool00`3oIVH0oooo0?ooo`020?ooo`030?mVIP3oool0oooo00<0oooo00<0ofIV
0?ooo`3oool09`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0600oooo00<0Ogmo
0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo0280oooo002=0?oo
o`X000000`3oool3000000D0oooo0`0000020?ooo`<000001@3oool300000080oooo0`0000050?oo
o`<000002`3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo02`0oooo00<0ofIV0?oo
o`3oool00P3oool00`3oIVH0oooo0?ooo`030?ooo`030?mVIP3oool0oooo02L0oooo00<0Ogmo0?oo
o`3oool0H@3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?oo
o`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`00S@3oool:000000<0oooo0`0000050?ooo`<0
00000P3oool3000000D0oooo0`0000020?ooo`<000001@3oool3000000/0oooo00<000000?ooo`3o
ool01`3oool00`1oOgl0oooo0?ooo`0]0?ooo`050?mVIP3oool0ofIV0?ooo`3oIVH01P3oool00`3o
IVH0oooo0?ooo`0V0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1o
Ogl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool008d0
oooo0`0000000`3oool0000000000002000000D0oooo0`0000050?ooo`<000000P3oool3000000D0
oooo0`0000020?ooo`<000001@3oool3000000/0oooo00<000000?ooo`3oool01`3oool00`1oOgl0
oooo0?ooo`0]0?ooo`050?mVIP3oool0ofIV0?ooo`3oIVH01P3oool00`3oIVH0oooo0?ooo`0V0?oo
o`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?oo
o`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool008d0oooo0`00000:0?ooo`<0
00001@3oool300000080oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`00000;0?ooo`03
0000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool0;P3oool20?mVIP030?ooo`3oIVH0oooo00D0
oooo00<0ofIV0?ooo`3oool09P3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0600
oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo0280
oooo002=0?ooo`<000001@3oool300000080oooo0`0000050?ooo`<000000P3oool3000000D0oooo
0`0000020?ooo`<000001@3oool3000000/0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo
0?ooo`0^0?ooo`80ofIV00<0oooo0?mVIP3oool01P3oool00`3oIVH0oooo0?ooo`0U0?ooo`0307mo
O`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307mo
O`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool008h0oooo0`0000030?ooo`@000000`3o
ool3000000<0oooo0`0000040?ooo`<000000`3oool3000000@0oooo0`0000030?ooo`<00000303o
ool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo02h0oooo0P3oIVH00`3oool0ofIV0?oo
o`060?ooo`030?mVIP3oool0oooo02D0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?oo
o`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?oo
o`0R0?ooo`00SP3oool9000000@0oooo2@0000040?ooo`T00000103oool9000000`0oooo00<00000
0?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`0_0?ooo`80ofIV203oool00`3oIVH0oooo0?ooo`0U
0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q
0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool008l0oooo200000050?oo
o`L000001P3oool7000000H0oooo1`00000=0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?oo
o`3oool0;`3oool20?mVIPT0oooo00<0ofIV0?ooo`3oool0903oool00`1oOgl0oooo0?ooo`1Q0?oo
o`0307moO`3oool0oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?oo
o`030000003oool0oooo0280oooo002A0?ooo`@00000203oool5000000P0oooo1@0000080?ooo`D0
00003P3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo02l0oooo0P3oIVH90?ooo`03
0?mVIP3oool0oooo02@0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1P0?ooo`03
07moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`00
bP3oool3000000L0oooo00<0Ogmo0?ooo`3oool0;`3oool20?mVIPX0oooo00<0ofIV0?ooo`3oool0
8`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0600oooo00<0Ogmo0?ooo`3oool0
H@3oool00`1oOgl0oooo0?ooo`0@0?ooo`<00000903oool00<X0oooo00<000000?ooo`3oool01`3o
ool00`1oOgl0oooo0?ooo`0_0?ooo`80ofIV2P3oool00`3oIVH0oooo0?ooo`0S0?ooo`0307moO`3o
ool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3o
ool0oooo0180oooo00<000000?ooo`3oool08P3oool00<X0oooo00<000000?ooo`3oool01`3oool0
0`1oOgl0oooo0?ooo`0_0?ooo`80ofIV2P3oool00`3oIVH0oooo0?ooo`0S0?ooo`0307moO`3oool0
oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0
oooo0180oooo00<000000?ooo`3oool08P3oool00<X0oooo00<000000?ooo`3oool01`3oool00`1o
Ogl0oooo0?ooo`0_0?ooo`80ofIV2`3oool00`3oIVH0oooo0?ooo`0R0?ooo`0307moO`3oool0oooo
0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo
0180oooo00<000000?ooo`3oool08P3oool00<X0oooo00<000000?ooo`3oool01`3oool00`1oOgl0
oooo0?ooo`0_0?ooo`80ofIV2`3oool00`3oIVH0oooo0?ooo`0R0?ooo`0307moO`3oool0oooo0640
oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180
oooo00<000000?ooo`3oool08P3oool00<X0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo
0?ooo`0_0?ooo`80ofIV2`3oool00`3oIVH0oooo0?ooo`0R0?ooo`0307moO`3oool0oooo0640oooo
00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo
00<000000?ooo`3oool08P3oool00<X0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?oo
o`0`0?ooo`030?mVIP3oool0oooo00X0oooo00<0ofIV0?ooo`3oool08@3oool00`1oOgl0oooo0?oo
o`1Q0?ooo`0307moO`3oool0oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?oo
o`0B0?ooo`030000003oool0oooo0280oooo003:0?ooo`030000003oool0oooo00L0oooo00<0Ogmo
0?ooo`3oool0<03oool00`3oIVH0oooo0?ooo`0:0?ooo`030?mVIP3oool0oooo0240oooo00<0Ogmo
0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo
0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`00bP3oool00`000000oooo0?ooo`070?oo
o`0307moO`3oool0oooo0300oooo00<0ofIV0?ooo`3oool02P3oool00`3oIVH0oooo0?ooo`0Q0?oo
o`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?oo
o`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool00<X0oooo00<000000?ooo`3o
ool01`3oool00`1oOgl0oooo0?ooo`0`0?ooo`030?mVIP3oool0oooo00/0oooo00<0ofIV0?ooo`3o
ool0803oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0600oooo00<0Ogmo0?ooo`3o
ool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo0280oooo003:0?ooo`030000
003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool0<03oool00`3oIVH0oooo0?ooo`0;0?ooo`030?mV
IP3oool0oooo0200oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307mo
O`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`00bP3o
ool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo0300oooo00<0ofIV0?ooo`3oool0303o
ool00`3oIVH0oooo0?ooo`0O0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03o
ool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3o
ool00<X0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`0`0?ooo`030?mVIP3oool0
oooo00`0oooo00<0ofIV0?ooo`3oool07`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0
oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0
oooo0280oooo003:0?ooo`<000001`3oool00`1oOgl0oooo0?ooo`0`0?ooo`030?mVIP3oool0oooo
00`0oooo00<0ofIV0?ooo`3oool07`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo
0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0@0?ooo`<00000903oool00<X0
oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`0`0?ooo`030?mVIP3oool0oooo00d0
oooo00<0ofIV0?ooo`3oool07P3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0600
oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo0280
oooo003:0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool0<03oool00`3oIVH0oooo
0?ooo`0=0?ooo`030?mVIP3oool0oooo01h0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo
0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo
0?ooo`0R0?ooo`00bP3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo0300oooo00<0
ofIV0?ooo`3oool03@3oool00`3oIVH0oooo0?ooo`0C0?ooo`80ofIV2@3oool00`1oOgl0oooo0?oo
o`1Q0?ooo`0307moO`3oool0oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?oo
o`0B0?ooo`030000003oool0oooo0280oooo003:0?ooo`030000003oool0oooo00L0oooo00<0Ogmo
0?ooo`3oool0<03oool00`3oIVH0oooo0?ooo`0>0?ooo`030?mVIP3oool0oooo00l0oooo0`3oIVH2
0?ooo`030?mVIP3oool0oooo00H0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1P
0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R
0?ooo`00bP3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo0300oooo00<0ofIV0?oo
o`3oool03P3oool00`3oIVH0oooo0?ooo`0<0?ooo`<0ofIV1P3oool00`3oIVH0oooo0?ooo`050?oo
o`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?oo
o`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool00<X0oooo00<000000?ooo`3o
ool01`3oool00`1oOgl0oooo0?ooo`0`0?ooo`030?mVIP3oool0oooo00l0oooo00<0ofIV0?ooo`3o
ool02@3oool20?mVIPX0oooo00<0ofIV0?ooo`3oool0103oool00`1oOgl0oooo0?ooo`1Q0?ooo`03
07moO`3oool0oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`03
0000003oool0oooo0280oooo003:0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool0
@P3oool00`3oIVH0oooo0?ooo`060?ooo`<0ofIV303oool00`3oIVH0oooo0?ooo`040?ooo`0307mo
O`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307mo
O`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool00<X0oooo00<000000?ooo`3oool01`3o
ool00`1oOgl0oooo0?ooo`120?ooo`030?mVIP3oool0oooo00<0oooo0`3oIVH@0?ooo`030?mVIP3o
ool0oooo00<0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3o
ool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`00bP3oool0
0`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo04<0oooo00<0ofIV0?ooo`3oIVH00P3oIVHD
0?ooo`030?mVIP3oool0oooo0080oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1P
0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R
0?ooo`00bP3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo04<0oooo0P3oIVHH0?oo
o`050?mVIP3oool0oooo0?ooo`1oOgl0H`3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0
oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`00bP3oool00`00
0000oooo0?ooo`070?ooo`0307moO`3oool0oooo05h0oooo00@0ofIV0?ooo`3oool0OgmoH`3oool0
0`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool0
0`000000oooo0?ooo`0R0?ooo`00bP3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo
05l0oooo00<0ofIV0?ooo`1oOgl0H`3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo
0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`00bP3oool3000000L0
oooo00<0Ogmo0?ooo`3oool0G`3oool00`3oIVH0oooo07moO`1S0?ooo`0307moO`3oool0oooo0600
oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0@0?ooo`<00000903oool00<X0oooo
00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`1P0?ooo`030?mVIP1oOgl0oooo0680oooo
00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo
00<000000?ooo`3oool08P3oool00<X0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?oo
o`1Q0?ooo`030?mVIP3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?oo
o`1Q0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool00<X0oooo00<00000
0?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oIVH0oooo0640oooo00<0Ogmo
0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo00<00000
0?ooo`3oool08P3oool00<X0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`1Q0?oo
o`0307moO`3oIVH0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?oo
o`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool00<X0oooo00<000000?ooo`3o
ool01`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oIVH0oooo0640oooo00<0Ogmo0?ooo`3o
ool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3o
ool08P3oool00<X0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307mo
O`3oIVH0oooo0640oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307mo
O`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool00<X0oooo00<000000?ooo`3oool01`3o
ool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oIVH0oooo0640oooo00<0Ogmo0?ooo`3oool0H03o
ool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3o
ool00900oooo1@0000080?ooo`D00000203oool5000000P0oooo1@00000>0?ooo`030000003oool0
oooo00L0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0ofIV0?ooo`1Q0?ooo`0307moO`3oool0
oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0
oooo0280oooo002>0?ooo`T000001@3oool7000000H0oooo1`0000060?ooo`L000003@3oool00`00
0000oooo0?ooo`070?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?mVIP3oool0H@3oool00`1o
Ogl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`00
0000oooo0?ooo`0R0?ooo`00SP3oool9000000@0oooo2@0000040?ooo`T00000103oool9000000`0
oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oIVH0oooo0640
oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180
oooo00<000000?ooo`3oool08P3oool008d0oooo100000030?ooo`@000000`3oool3000000<0oooo
0`0000040?ooo`<000000`3oool3000000@0oooo0`0000030?ooo`<00000303oool00`000000oooo
0?ooo`070?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?mVIP3oool0;`3oool00`3oIVH0oooo
0?ooo`0_0?ooo`0307moO`3oool0oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo
0?ooo`0B0?ooo`030000003oool0oooo0280oooo002=0?ooo`<000001@3oool300000080oooo0`00
00050?ooo`<000000P3oool3000000D0oooo0`0000020?ooo`<000001@3oool3000000/0oooo00<0
00000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oIVH0oooo02h0oooo00<0
ofIV0?ooo`3oIVH0<03oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo00<0
Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`00S@3oool3000000D0oooo0`000002
0?ooo`<000001@3oool300000080oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`00000;
0?ooo`@00000K01oOgl00`3oIVH0Ogmo07moO`0Z07moO`80ofIV0P1oOgl00`3oIVH0Ogmo07moO`3o
07moO`P0Ogmo1000000T0?ooo`00S@3oool4000000<0oooo100000020?ooo`<000001@3oool30000
0080oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`00000;0?ooo`030000003oool0oooo
00L0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?mVIP0[0?ooo`030?mVIP3oool0oooo
00<0oooo00<0ofIV0?ooo`3oool0;@3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo
0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`00SP3oool9000000<0
oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`0000020?ooo`<000001@3oool3000000/0
oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0ofIV00P0
ofIV8P3oool00`3oIVH0oooo0?ooo`040?ooo`030?mVIP3oool0oooo02d0oooo00<0Ogmo0?ooo`3o
ool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3o
ool08P3oool008l0oooo1`0000040?ooo`<000001@3oool300000080oooo0`0000050?ooo`<00000
0P3oool3000000D0oooo0`00000;0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool0
H@3oool0101oOgl0oooo0?mVIP3oIVH70?ooo`80ofIV7P3oool20?mVIPP0oooo00<0ofIV0?ooo`3o
ool0;03oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3o
ool04P3oool00`000000oooo0?ooo`0R0?ooo`00SP3oool9000000<0oooo0`0000050?ooo`<00000
0P3oool3000000D0oooo0`0000020?ooo`<000001@3oool3000000/0oooo00<000000?ooo`3oool0
1`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0407moO`3oool0ofIV0?mVIPT0oooo0P3oIVHK0?ooo`03
0?mVIP3oool0oooo00T0oooo00<0ofIV0?ooo`3oool0:`3oool00`1oOgl0oooo0?ooo`1P0?ooo`03
07moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`00
S@3oool4000000<0oooo100000020?ooo`<000001@3oool300000080oooo0`0000050?ooo`<00000
0P3oool3000000D0oooo0`00000;0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool0
H@3oool00`1oOgl0oooo0?mVIP0<0?ooo`80ofIV603oool00`3oIVH0oooo0?ooo`0:0?ooo`030?mV
IP3oool0oooo02/0oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307mo
O`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool008d0oooo0`0000050?ooo`<000000P3o
ool3000000D0oooo0`0000020?ooo`<000001@3oool300000080oooo0`0000050?ooo`<000002`3o
ool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oIVH03P3o
ool20?mVIQ@0oooo0P3oIVH>0?ooo`030?mVIP3oool0oooo02X0oooo00<0Ogmo0?ooo`3oool0H03o
ool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3o
ool008d0oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`0000020?ooo`<000001@3oool3
00000080oooo0`0000050?ooo`<000002`3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0
oooo0640oooo00<0Ogmo0?ooo`3oIVH0403oool00`3oIVH0oooo0?ooo`0@0?ooo`030?mVIP3oool0
oooo00h0oooo00<0ofIV0?ooo`3oool0:P3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0
oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`00S@3oool40000
00<0oooo100000030?ooo`<000000`3oool3000000@0oooo0`0000030?ooo`<00000103oool30000
00<0oooo0`00000<0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1o
Ogl0oooo0?mVIP0A0?ooo`80ofIV3`3oool00`3oIVH0oooo0?ooo`0@0?ooo`030?mVIP3oool0oooo
02T0oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo
0180oooo00<000000?ooo`3oool08P3oool008h0oooo2@0000040?ooo`T00000103oool9000000@0
oooo2@00000<0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0
oooo0?ooo`0C0?ooo`80ofIV2`3oool20?mVIQ@0oooo00<0ofIV0?ooo`3oool0:03oool00`1oOgl0
oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000
oooo0?ooo`0R0?ooo`00SP3oool9000000D0oooo1`0000060?ooo`L000001P3oool7000000d0oooo
00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo01D0oooo
0P3oIVH80?ooo`030?mVIP3oool0oooo01@0oooo00<0ofIV0?ooo`3oool0:03oool00`1oOgl0oooo
0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo
0?ooo`0R0?ooo`00T03oool5000000P0oooo1@0000080?ooo`D00000203oool5000000h0oooo00<0
00000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo01L0oooo0P3o
IVH50?ooo`030?mVIP3oool0oooo01H0oooo00<0ofIV0?ooo`3oool09`3oool00`1oOgl0oooo0?oo
o`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?oo
o`0R0?ooo`00bP3oool3000000L0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0I
0?ooo`80ofIV00<0oooo0?mVIP3oIVH06@3oool00`3oIVH0oooo0?ooo`0W0?ooo`0307moO`3oool0
oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0@0?ooo`<00000903oool0
0<X0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo
01/0oooo00<0ofIV0?ooo`3oool06P3oool00`3oIVH0oooo0?ooo`0V0?ooo`0307moO`3oool0oooo
0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0B0?ooo`030000003oool0oooo
0280oooo003:0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0
oooo0?ooo`0i0?ooo`030?mVIP3oool0oooo02D0oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0
oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo00<000000?ooo`3oool08P3oool00<X0oooo
00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo03T0oooo
00<0ofIV0?ooo`3oool09@3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo
00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`00bP3oool00`000000oooo0?oo
o`070?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0>P3oool00`3oIVH0oooo0?oo
o`0T0?ooo`0307moO`3oool0oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?oo
o`0B0?ooo`030000003oool0oooo0280oooo003:0?ooo`030000003oool0oooo00L0oooo00<0Ogmo
0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0k0?ooo`030?mVIP3oool0oooo02<0oooo00<0Ogmo
0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0180oooo00<00000
0?ooo`3oool08P3oool00<X0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`1Q0?oo
o`0307moO`3oool0oooo03/0oooo00<0ofIV0?ooo`3oool08`3oool00`1oOgl0oooo0?ooo`1P0?oo
o`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0203oool00`3oIVH0oooo0?ooo`070?oo
o`030000003oool0oooo0280oooo003:0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3o
ool0H@3oool00`1oOgl0oooo0?ooo`0l0?ooo`030?mVIP3oool0oooo0280oooo00<0Ogmo0?ooo`3o
ool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo00P0oooo00<0ofIV0?ooo`3o
ool01`3oool00`000000oooo0?ooo`0R0?ooo`00bP3oool00`000000oooo0?ooo`070?ooo`0307mo
O`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0?03oool00`3oIVH0oooo0?ooo`0R0?ooo`0307mo
O`3oool0oooo0600oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`080?ooo`030?mV
IP3oool0oooo00L0oooo00<000000?ooo`3oool08P3oool00<X0oooo00<000000?ooo`3oool01`3o
ool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo03d0oooo00<0ofIV0?ooo`3oool08@3o
ool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0203o
ool00`3oIVH0oooo0?ooo`070?ooo`030000003oool0oooo0280oooo003:0?ooo`030000003oool0
oooo00L0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0n0?ooo`030?mVIP3oool0
oooo0200oooo00<0Ogmo0?ooo`3oool0H03oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0
oooo00P0oooo00<0ofIV0?ooo`3oool01`3oool00`000000oooo0?ooo`0R0?ooo`00bP3oool00`00
0000oooo0?ooo`070?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0?P3oool00`3o
IVH0oooo0?ooo`0P0?ooo`0307moO`3oool0oooo00L0oooo00<0ofIV0?ooo`3oool0EP3oool00`1o
Ogl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo00P0oooo00<0ofIV0?ooo`3oool01`3oool00`00
0000oooo0?ooo`0R0?ooo`00bP3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo0640
oooo00<0Ogmo0?ooo`3oool0?`3oool00`3oIVH0oooo0?ooo`0O0?ooo`0307moO`3oool0oooo00H0
oooo00<0ofIV0?ooo`3oIVH0E`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo00P0
oooo00<0ofIV0?ooo`3oool01`3oool00`000000oooo0?ooo`0R0?ooo`00bP3oool3000000L0oooo
00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0o0?ooo`030?mVIP3oool0oooo01l0oooo
00<0Ogmo0?ooo`3oool01P3oool0103oIVH0oooo0?ooo`3oIVIF0?ooo`0307moO`3oool0oooo0640
oooo00<0Ogmo0?ooo`3oool0203oool00`3oIVH0oooo0?ooo`050?ooo`<00000903oool00<X0oooo
00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0400oooo
00<0ofIV0?ooo`3oool07P3oool00`1oOgl0oooo0?ooo`050?ooo`050?mVIP3oool0oooo0?ooo`3o
IVH0EP3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo00H0oooo00<0ofIV0?ooo`3o
IVH02@3oool00`000000oooo0?ooo`0R0?ooo`00bP3oool00`000000oooo0?ooo`070?ooo`0307mo
O`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0@@3oool00`3oIVH0oooo0?ooo`0M0?ooo`0307mo
O`3oool0oooo00@0oooo00<0ofIV0?ooo`3oool00`3oool00`3oIVH0oooo0?ooo`1C0?ooo`0307mo
O`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool01@3oool20?mVIP030?ooo`3oIVH0oooo00P0oooo
00<000000?ooo`3oool08P3oool00<X0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?oo
o`1Q0?ooo`0307moO`3oool0oooo0440oooo00<0ofIV0?ooo`3oool07@3oool0101oOgl0oooo0?oo
o`3oIVH30?ooo`030?mVIP3oool0oooo00@0oooo00<0ofIV0?ooo`3oool0DP3oool00`1oOgl0oooo
0?ooo`1Q0?ooo`0307moO`3oool0oooo00@0oooo00D0ofIV0?ooo`3oIVH0oooo0?mVIP090?ooo`03
0000003oool0oooo0280oooo003:0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool0
H@3oool00`1oOgl0oooo0?ooo`120?ooo`030?mVIP3oool0oooo01`0oooo00@0Ogmo0?mVIP3oIVH0
ofIV0P3oool00`3oIVH0oooo0?ooo`060?ooo`030?mVIP3oool0oooo0540oooo00<0Ogmo0?ooo`3o
ool0H@3oool00`1oOgl0oooo0?ooo`040?ooo`050?mVIP3oool0ofIV0?ooo`3oIVH02@3oool00`00
0000oooo0?ooo`0R0?ooo`00bP3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo0640
oooo00<0Ogmo0?ooo`3oool0@P3oool00`3oIVH0oooo0?ooo`0K0?ooo`80ofIV0P3oool0103oIVH0
oooo0?ooo`3oIVH90?ooo`030?mVIP3oool0oooo0500oooo00<0Ogmo0?ooo`3oool0H@3oool00`1o
Ogl0oooo0?ooo`030?ooo`060?mVIP3oool0oooo0?mVIP3oool0ofIV2@3oool00`000000oooo0?oo
o`0R0?ooo`00bP3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo
0?ooo`3oool0@`3oool30?mVIQP0oooo0P3oIVH01`3oool0Ogmo0?ooo`3oool0ofIV0?ooo`3oIVH0
2`3oool00`3oIVH0oooo0?ooo`1?0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0
0P3oool01`3oIVH0oooo0?ooo`3oool0ofIV0?ooo`3oIVH02@3oool00`000000oooo0?ooo`0R0?oo
o`00bP3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3o
ool0AP3oool50?mVIQ40oooo0P3oIVH30?ooo`0607moO`3oool0oooo0?mVIP3oool0ofIV2`3oool0
0`3oIVH0oooo0?ooo`1?0?ooo`0307moO`3oool0oooo0640oooo00D0Ogmo0?ooo`3oool0oooo0?mV
IP040?ooo`80ofIV2P3oool00`000000oooo0?ooo`0R0?ooo`00bP3oool00`000000oooo0?ooo`07
0?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0B`3oool50?mVIPX0oooo0P3oIVH5
0?ooo`0607moO`3oool0oooo0?mVIP3oool0ofIV303oool00`3oIVH0oooo0?ooo`1>0?ooo`0307mo
O`3oool0oooo0640oooo00@0Ogmo0?ooo`3oool0ofIV1@3oool20?mVIPX0oooo00<000000?ooo`3o
ool08P3oool00<X0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307mo
O`3oool0oooo0500oooo1@3oIVH30?ooo`80ofIV1`3oool01P1oOgl0oooo0?ooo`3oIVH0oooo0?mV
IPd0oooo00<0ofIV0?ooo`3oool0C@3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0ofIV
00H0oooo0P3oIVH:0?ooo`030000003oool0oooo0280oooo003:0?ooo`030000003oool0oooo00L0
oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1E0?ooo`<0ofIV2@3oool01P1oOgl0
oooo0?ooo`3oIVH0oooo0?mVIPh0oooo00<0ofIV0?ooo`3oool0C03oool00`1oOgl0oooo0?ooo`1Q
0?ooo`0307moO`3oool0ofIV00H0oooo0P3oIVH:0?ooo`030000003oool0oooo0280oooo003:0?oo
o`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1Q0?oo
o`0407moO`3oool0oooo0?ooo`80ofIV3`3oool00`3oIVH0oooo0?ooo`1;0?ooo`0307moO`3oool0
oooo0640oooo00<0Ogmo0?mVIP3oool01P3oool20?mVIPX0oooo00<000000?ooo`3oool08P3oool0
0<X0oooo00<000000?ooo`3oool01`3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo
0640oooo00D0Ogmo0?ooo`3oool0oooo0?mVIP0A0?ooo`030?mVIP3oool0oooo04X0oooo00<0Ogmo
0?ooo`3oool0H@3oool00`3oIVH0oooo0?ooo`070?ooo`030?mVIP3oool0oooo00P0oooo00<00000
0?ooo`3oool08P3oool00<X0oooo0`0000070?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo0?oo
o`3oool0H@3oool01@1oOgl0oooo0?ooo`3oool0ofIV0140oooo00<0ofIV0?ooo`3oool0BP3oool0
0`1oOgl0oooo0?ooo`1P0?ooo`030?mVIP1oOgl0oooo00P0oooo00<0ofIV0?ooo`3oool01P3oool3
000002@0oooo003:0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1o
Ogl0oooo0?ooo`1Q0?ooo`0507moO`3oool0oooo0?ooo`3oIVH04P3oool00`3oIVH0oooo0?ooo`19
0?ooo`0307moO`3oool0oooo00H0oooo0`3oIVIF0?ooo`030?mVIP3oool0Ogmo00T0oooo00<0ofIV
0?ooo`3oool0203oool00`000000oooo0?ooo`0R0?ooo`00bP3oool00`000000oooo0?ooo`070?oo
o`0307moO`3oool0oooo0640oooo00<0Ogmo0?ooo`3oool0H@3oool01@1oOgl0oooo0?ooo`3oool0
ofIV01<0oooo00<0ofIV0?ooo`3oool0B03oool00`1oOgl0oooo0?ooo`020?ooo`@0ofIV0`3oool3
0?mVIU80oooo00@0ofIV0?ooo`3oool0Ogmo2@3oool00`3oIVH0oooo0?ooo`080?ooo`030000003o
ool0oooo0280oooo003:0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool0H@3oool0
0`1oOgl0oooo0?ooo`1Q0?ooo`0507moO`3oool0oooo0?ooo`3oIVH0503oool00`3oIVH0oooo0?oo
o`170?ooo`0307moO`3oIVH0ofIV0080ofIV2P3oool40?mVITh0oooo00@0ofIV0?ooo`3oool0Ogmo
2@3oool00`3oIVH0oooo0?ooo`080?ooo`030000003oool0oooo0280oooo003:0?ooo`030000003o
ool0oooo00L0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0507moO`3o
ool0oooo0?ooo`3oIVH05@3oool00`3oIVH0oooo0?ooo`0O0?ooo`h0ofIV5P3oool40?mVIQ80oooo
0`3oIVI:0?ooo`050?mVIP3oool0oooo0?ooo`1oOgl02@3oool00`3oIVH0oooo0?ooo`080?ooo`03
0000003oool0oooo0280oooo003:0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool0
H@3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0507moO`3oool0oooo0?ooo`3oIVH05P3oool00`3oIVH0
oooo0?ooo`0B0?ooo``0ofIV3P3oool:0?mVIPP0oooo103oIVH30?ooo`0307moO`3oool0oooo01<0
oooo0`3oIVI60?ooo`030?mVIP3oool0oooo0080oooo00<0Ogmo0?ooo`3oool01`3oool00`3oIVH0
oooo0?ooo`080?ooo`030000003oool0oooo0280oooo003:0?ooo`030000003oool0oooo00L0oooo
00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo01P0oooo
00<0ofIV0?ooo`3oool03@3oool50?mVIR@0oooo203oIVH70?ooo`0307moO`3oool0oooo01H0oooo
103oIVI10?ooo`030?mVIP3oool0oooo00<0oooo00<0Ogmo0?ooo`3oool01`3oool00`3oIVH0oooo
0?ooo`080?ooo`030000003oool0oooo0280oooo003:0?ooo`030000003oool0oooo00L0oooo00<0
Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo01T0oooo00<0
ofIV0?ooo`3oool01`3oool50?mVISP0oooo00<0Ogmo0?ooo`3oool06P3oool;0?mVISD0oooo00<0
ofIV0?ooo`3oool0103oool00`1oOgl0oooo0?ooo`070?ooo`030?mVIP3oool0oooo00P0oooo00<0
00000?ooo`3oool08P3oool008D0oooo0`0000080?ooo`D00000203oool5000000P0oooo1@000008
0?ooo`D000003P3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo0640oooo00<0Ogmo
0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`0J0?ooo`040?mVIP3oool0oooo0?ooo`D0ofIV?@3o
ool00`1oOgl0oooo0?ooo`0U0?ooo`l0ofIV9@3oool00`3oIVH0oooo0?ooo`050?ooo`0307moO`3o
ool0oooo00L0oooo00<0ofIV0?ooo`3oool0203oool00`000000oooo0?ooo`0R0?ooo`00Q@3oool3
000000L0oooo1`0000060?ooo`L000001P3oool7000000H0oooo1`00000=0?ooo`030000003oool0
oooo00L0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0
oooo01/0oooo0`3oIVI20?ooo`0307moO`3oool0oooo03@0oooo2P3oIVHK0?ooo`030?mVIP3oool0
oooo00D0oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`00Q@3oool30000
00H0oooo2@0000040?ooo`T00000103oool9000000@0oooo2@00000<0?ooo`030000003oool0oooo
00L0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo
0600oooo00<0Ogmo0?ooo`3oool0?P3oool?0?mVIP/0oooo00<0ofIV0?ooo`3oool01P3oool00`1o
Ogl0oooo0?ooo`0B0?ooo`030000003oool0oooo0280oooo00250?ooo`<000001P3oool3000000<0
oooo0`0000040?ooo`<000000`3oool3000000@0oooo0`0000030?ooo`<00000103oool3000000<0
oooo0`00000<0?ooo`030000003oool0oooo00L0oooo00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0
oooo0?ooo`1Q0?ooo`0307moO`3oool0oooo0600oooo00<0Ogmo0?ooo`3oool0C@3oool;0?mVIPT0
oooo00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`00Q@3oool3000000D0oooo
0`0000050?ooo`<000000P3oool3000000D0oooo0`0000020?ooo`<000001@3oool300000080oooo
0`0000050?ooo`<000002`3oool00`000000oooo0?ooo`070?ooo`0307moO`3oool0oooo0640oooo
00<0Ogmo0?ooo`3oool0H@3oool00`1oOgl0oooo0?ooo`1P0?ooo`0307moO`3oool0oooo0640oooo
00<0Ogmo0?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`00Q@3oool3000000D0oooo0`00
00050?ooo`<000000P3oool3000000D0oooo0`0000020?ooo`<000001@3oool300000080oooo0`00
00050?ooo`<000002`3oool400000?l0OgmoZ01oOgl4000002@0oooo00250?ooo`<000001@3oool3
000000D0oooo0`0000020?ooo`<000001@3oool300000080oooo0`0000050?ooo`<000000P3oool3
000000D0oooo0`00000;0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0H@3oool0
0`000000oooo0?ooo`1Q0?ooo`030000003oool0oooo0600oooo00<000000?ooo`3oool0H@3oool0
0`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0280oooo00250?ooo`<000001@3oool30000
00D0oooo0`0000020?ooo`<000001@3oool300000080oooo0`0000050?ooo`<000000P3oool30000
00D0oooo0`00000;0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool04@3oool00`00
0000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool04@3oool00`00
0000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool04@3oool00`00
0000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool04@3oool00`00
0000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0403oool00`00
0000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool04@3oool00`00
0000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool04@3oool00`00
0000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo0P00000T0?ooo`00Q@3oool3000000D0
oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`0000020?ooo`<000001@3oool300000080
oooo0`0000050?ooo`<000002`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0140
oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140
oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140
oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140
oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140
oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140
oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0A0?ooo`800000903oool008D0oooo
0`0000050?ooo`<000001@3oool300000080oooo0`0000050?ooo`<000000P3oool3000000D0oooo
0`0000020?ooo`<000001@3oool3000000/0ooooo`00002`000002@0oooo00210?ooo`040000003o
ool0oooo0?ooo`<000001@3oool3000000D0oooo0`0000020?ooo`<000001@3oool300000080oooo
0`0000050?ooo`<000000P3oool3000000D0oooo0`00003o0?oooml0oooo00210?ooo`<0000000@0
oooo00000000000000001@3oool3000000D0oooo0`0000020?ooo`<000001@3oool300000080oooo
0`0000050?ooo`<000000P3oool3000000D0oooo0`00003o0?oooml0oooo00210?ooo`L000001@3o
ool3000000D0oooo0`0000020?ooo`<000001@3oool300000080oooo0`0000050?ooo`<000000P3o
ool3000000D0oooo0`00003o0?oooml0oooo00220?ooo`H000001P3oool3000000<0oooo0`000004
0?ooo`<000000`3oool3000000@0oooo0`0000030?ooo`<00000103oool3000000<0oooo0`00003o
0?ooon00oooo00240?ooo`@000001P3oool9000000@0oooo2@0000040?ooo`T00000103oool90000
0?l0ooooh03oool008D0oooo0`0000070?ooo`L000001P3oool7000000H0oooo1`0000060?ooo`L0
0000o`3ooooQ0?ooo`00Q@3oool3000000P0oooo1@0000080?ooo`D00000203oool5000000P0oooo
1@00003o0?ooon80oooo003o0?ooool0ooooW`3oool00?l0ooooo`3oool90?ooo`@00000TP3oool0
0?l0ooooo`3oool90?ooo`H00000T03oool00?l0ooooo`3oool90?ooo`H00000T03oool00?l0oooo
o`3oool<0?ooo`@00000S`3oool00?l0ooooo`3oool=0?ooo`<00000S`3oool00;/0oooo0`000004
0?ooo`<00000103oool3000000@0oooo0`00000<0?ooo``000001`3oool6000000P0oooo0`00000@
0?ooo`L000005@3oool5000000L0oooo0`0000050?ooo`<000002`3oool500000080oooo0`00000;
0?ooo`@000002@3oool5000000P0oooo1@00000=0?ooo`<00000103oool3000000@0oooo0`000004
0?ooo`<000001`3oool3000000<0oooo2P00000>0?ooo`<00000603oool3000000D0oooo0`000007
0?ooo`D000000P3oool3000000D0oooo0`00000D0?ooo`<000001@3oool4000000@0oooo1@000002
0?ooo`<000000P3oool3000000D0oooo0`0000040?ooo`<000001P3oool400000080oooo0`000004
0?ooo`<00000103oool3000000@0oooo0`0000090?ooo`/000005@3oool00;/0oooo0`0000040?oo
o`<00000103oool3000000@0oooo0`00000<0?ooo``000001@3oool:000000H0oooo0`00000>0?oo
o`X000004P3oool9000000D0oooo0`0000050?ooo`<000002P3oool:000000X0oooo200000060?oo
o`L000001P3oool7000000`0oooo0`0000040?ooo`<00000103oool3000000@0oooo0`0000070?oo
o`<000000`3oool:000000h0oooo0`00000H0?ooo`<000001@3oool3000000H0oooo2P0000050?oo
o`D000004`3oool3000000@0oooo1@0000030?ooo`X000000`3oool3000000D0oooo0`0000040?oo
o`<000001@3oool:000000@0oooo0`0000040?ooo`<00000103oool3000000T0oooo2`00000E0?oo
o`00^`3oool3000000<0oooo1@0000030?ooo`<00000103oool3000000`0oooo300000040?ooo``0
00001@3oool3000000d0oooo3000000@0?ooo`/00000103oool3000000D0oooo0`0000090?ooo`@0
00000`3oool4000000X0oooo2@0000040?ooo`T00000103oool9000000/0oooo0`0000030?ooo`D0
00000`3oool3000000@0oooo0`0000070?ooo`<000000`3oool:000000h0oooo0`00000G0?ooo`D0
00000`3oool5000000@0oooo100000030?ooo`@000001@3oool5000001<0oooo0`0000040?ooo`D0
00000P3oool4000000<0oooo100000030?ooo`<000001@3oool3000000@0oooo0`0000040?ooo`/0
0000103oool3000000@0oooo0`0000040?ooo`<000002P3oool:000001D0oooo002k0?ooo`<00000
0`3oool5000000<0oooo0`0000040?ooo`<00000303oool3000000`0oooo1@0000040?ooo`D00000
103oool3000000d0oooo100000050?ooo`@000003`3oool4000000<0oooo100000040?ooo`<00000
1@3oool3000000T0oooo0`0000050?ooo`<000002@3oool4000000<0oooo0`0000040?ooo`<00000
0`3oool3000000@0oooo0`0000030?ooo`<000002`3oool3000000<0oooo1@0000030?ooo`<00000
103oool3000000L0oooo0`0000040?ooo`<000003@3oool<000001D0oooo1@0000030?ooo`D00000
103oool3000000D0oooo0`0000040?ooo`L000004P3oool3000000@0oooo0`0000040?ooo`<00000
1@3oool3000000<0oooo0`0000050?ooo`<00000103oool3000000@0oooo100000030?ooo`@00000
103oool3000000@0oooo0`0000040?ooo`<000002P3oool4000001/0oooo002k0?ooo`<000000`3o
ool5000000<0oooo0`0000040?ooo`<00000303oool3000000`0oooo0`0000080?ooo`<00000103o
ool3000000`0oooo100000070?ooo`<000003P3oool4000000D0oooo100000030?ooo`<000001@3o
ool3000000T0oooo0`0000050?ooo`<000002@3oool3000000D0oooo0`0000020?ooo`<000001@3o
ool300000080oooo0`0000050?ooo`<000002P3oool3000000<0oooo1@0000030?ooo`<00000103o
ool3000000L0oooo0`0000050?ooo`<00000303oool<000001D0oooo0P0000000`3oool000000000
00030?ooo`80000000<0oooo000000000000103oool3000000D0oooo0`0000040?ooo`<0000000@0
oooo00000000000000004P3oool3000000@0oooo0`0000040?ooo`<000001@3oool3000000<0oooo
0`0000050?ooo`<00000103oool3000000@0oooo0`0000050?ooo`<00000103oool3000000@0oooo
0`0000040?ooo`<000002`3oool4000001X0oooo002k0?ooo`<000000`3oool2000000030?ooo`00
0000000000<0oooo0`0000040?ooo`<00000303oool3000000/0oooo100000080?ooo`@000000`3o
ool3000000`0oooo0`0000080?ooo`<000003P3oool3000000L0oooo0`0000030?ooo`<000001@3o
ool3000000X0oooo1@0000020?ooo`<000004@3oool300000080oooo0`0000050?ooo`<000000P3o
ool3000000D0oooo0`00000:0?ooo`<000000`3oool2000000030?ooo`000000000000<0oooo0`00
00040?ooo`<000001`3oool3000000H0oooo0`00000;0?ooo``000000P3oool>000000@0oooo0`00
00001@3oool00000000000000000oooo00<0000000@0oooo0000000000000000103oool500000080
oooo0`0000040?ooo`<0000000@0oooo00000000000000004P3oool3000000@0oooo0`0000050?oo
o`D000000P3oool3000000<0oooo0`0000050?ooo`<00000103oool3000000@0oooo0`0000050?oo
o`<00000103oool3000000@0oooo0`0000040?ooo`<00000303oool4000001T0oooo002k0?ooo`<0
00000P3oool3000000040?ooo`000000000000000080oooo0`0000040?ooo`<00000303oool30000
00/0oooo0`00000:0?ooo`<000000`3oool:000000l0oooo1000000>0?ooo`<000001`3oool30000
00<0oooo0`0000050?ooo`<000002`3oool9000000`0oooo10000000103oool00000000000000002
0?ooo`<000001@3oool300000080oooo0`0000050?ooo`<000002P3oool300000080oooo0`000000
103oool000000000000000020?ooo`<00000103oool3000000L0oooo0`0000060?ooo`@000002P3o
ool3000000@0oooo0`0000040?ooo`h00000103oool3000000050?ooo`00000000000000003oool0
0`000000103oool000000000000000050?ooo`T000000`3oool3000000<0oooo0`00000A0?ooo`<0
0000103oool3000000H0oooo2@0000030?ooo`<000001@3oool3000000@0oooo0`0000040?ooo`<0
00001@3oool3000000@0oooo0`0000040?ooo`<00000103oool3000000d0oooo1000000H0?ooo`00
^`3oool300000080oooo0`000000103oool000000000000000020?ooo`<00000103oool;000000@0
oooo0`00000;0?ooo`<000002P3oool3000000<0oooo2`00000;0?ooo`H000003`3oool3000000L0
oooo0`0000030?ooo`<000001@3oool3000000h0oooo1P00000:0?ooo`X000000P3oool3000000D0
oooo0`0000020?ooo`<000001@3oool3000000X0oooo0`0000020?ooo`<0000000@0oooo00000000
000000000P3oool3000000@0oooo3@0000070?ooo`<000002`3oool3000000<0oooo0`00000F0?oo
o`<0000000D0oooo00000000000000000?ooo`03000000040?ooo`0000000000000000P0oooo1P00
00030?ooo`<000000`3oool300000140oooo0`0000040?ooo`<000002@3oool6000000<0oooo0`00
00050?ooo`<00000103oool3000000@0oooo0`0000050?ooo`<00000103oool3000000@0oooo0`00
00040?ooo`<000003P3oool4000001L0oooo002k0?ooo`<000000P3oool3000000040?ooo`000000
000000000080oooo0`0000040?ooo`/00000103oool3000000/0oooo0`00000:0?ooo`<000000`3o
ool<000000L0oooo2@00000?0?ooo`@000001@3oool4000000<0oooo0`0000050?ooo`<000004@3o
ool3000000X0oooo2P0000020?ooo`<000001@3oool300000080oooo0`0000050?ooo`<000002P3o
ool300000080oooo0`000000103oool000000000000000020?ooo`<00000103oool=000000P0oooo
0`00000;0?ooo`<000000P3oool3000001D0oooo0`0000030?ooo`D000000`3oool3000000X0oooo
0`0000020?ooo`@000000`3oool300000140oooo0`0000040?ooo`<00000303oool3000000<0oooo
0`0000050?ooo`<00000103oool3000000@0oooo0`0000050?ooo`<00000103oool3000000@0oooo
0`0000040?ooo`<000003`3oool4000001H0oooo002k0?ooo`<000000P3oool2000000<0oooo0P00
00020?ooo`<00000103oool;000000@0oooo0`00000;0?ooo`<000002P3oool3000000<0oooo0`00
00060?ooo`@00000103oool900000180oooo100000030?ooo`@00000103oool4000000<0oooo1000
00090?ooo`<000001@3oool3000000T0oooo100000030?ooo`@000000P3oool3000000D0oooo0`00
00020?ooo`<000001@3oool3000000X0oooo0`0000020?ooo`8000000`3oool200000080oooo0`00
00040?ooo`d000002@3oool3000000X0oooo0`0000020?ooo`<000005@3oool3000000<0oooo1@00
00030?ooo`<000000P3oool3000000D0oooo0`0000020?ooo`<000001@3oool300000100oooo0`00
00040?ooo`<00000103oool3000000D0oooo0`0000030?ooo`@000000`3oool4000000@0oooo0`00
00040?ooo`<000001@3oool3000000@0oooo100000030?ooo`@000000`3oool300000100oooo0`00
000F0?ooo`00^`3oool3000000040?ooo`0000000000000000<0oooo0`000000103oool000000000
000000040?ooo`<00000303oool3000000/0oooo0`00000:0?ooo`<000000`3oool3000000L0oooo
0`0000030?ooo`L000005@3oool;000000@0oooo2`00000:0?ooo`<000000`3oool4000000T0oooo
0`0000050?ooo`<000000P3oool3000000D0oooo0`0000020?ooo`<000001@3oool3000000X0oooo
0`000000103oool000000000000000030?ooo`<0000000@0oooo0000000000000000103oool30000
00L0oooo0`0000030?ooo`X000002P3oool3000000040?ooo`0000000000000001D0oooo0`000003
0?ooo`D000000`3oool3000000<0oooo0`0000030?ooo`@000000P3oool3000000D0oooo0`00000@
0?ooo`<000000P3oool7000000<0oooo0`0000030?ooo`@000000`3oool;000000@0oooo0`000004
0?ooo`<000001@3oool3000000@0oooo4@00000A0?ooo`<000005@3oool00;/0oooo0`000000103o
ool000000000000000030?ooo`<0000000@0oooo0000000000000000103oool3000000`0oooo0`00
000;0?ooo`@00000203oool4000000<0oooo0`0000070?ooo`<000000`3oool4000001T0oooo2@00
00050?ooo`X000002`3oool9000000X0oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`00
00020?ooo`<000001@3oool3000000X0oooo0`000000103oool000000000000000030?ooo`<00000
00@0oooo0000000000000000103oool3000000L0oooo0`0000030?ooo`X000002`3oool6000001@0
oooo100000040?ooo`<000001@3oool300000080oooo2@0000020?ooo`<000001`3oool3000000l0
oooo0`0000020?ooo`L000000`3oool9000000@0oooo2P0000050?ooo`<00000103oool3000000D0
oooo0`0000040?ooo`T0000000<0oooo0000000000001000000:0?ooo`<000001@3oool3000001D0
oooo002k0?ooo`<0000000<0oooo0000000000001@3oool2000000040?ooo`0000000000000000@0
oooo0`00000<0?ooo`<00000303oool3000000P0oooo0`0000040?ooo`<000001`3oool3000000<0
oooo0`0000060?ooo`<000004`3oool5000000L0oooo0`0000020?ooo`@000003P3oool6000000/0
oooo0`0000050?ooo`<000000P3oool3000000D0oooo0`0000020?ooo`<000001@3oool3000000X0
oooo0`0000000`3oool00000000000050?ooo`80000000@0oooo0000000000000000103oool30000
00L0oooo0`0000030?ooo`X000002`3oool6000001@0oooo0`0000050?ooo`<000001@3oool30000
00@0oooo1P0000030?ooo`<000001`3oool3000000l0oooo0`0000020?ooo`L000001@3oool60000
00D0oooo0`0000020?ooo`@000001P3oool3000000@0oooo0`0000050?ooo`<00000103oool30000
00030?ooo`0000000000008000000`3oool4000000/0oooo0`0000050?ooo`<000005@3oool00;/0
oooo1P0000050?ooo`H00000103oool3000000`0oooo0`00000<0?ooo`D00000103oool5000000@0
oooo0`0000060?ooo`@000000`3oool4000000@0oooo100000170?ooo`@000000`3oool3000000@0
oooo0`0000030?ooo`<00000103oool3000000<0oooo0`00000;0?ooo`H000001@3oool6000000@0
oooo0`0000070?ooo`<000006@3oool500000500oooo0`0000040?ooo`<00000D@3oool4000000<0
oooo1000000E0?ooo`00^`3oool6000000D0oooo1P0000040?ooo``000000`3oool3000000d0oooo
300000050?ooo``000001@3oool:000004T0oooo2@0000040?ooo`T00000103oool9000000/0oooo
1P0000050?ooo`H00000103oool3000000L0oooo0`00000J0?ooo`@00000D03oool3000000@0oooo
0`00000Q0?ooo`<00000;P3oool9000001H0oooo002k0?ooo`D000001`3oool5000000@0oooo3000
00030?ooo`<000003P3oool:000000H0oooo300000050?ooo`X00000BP3oool7000000H0oooo1`00
00060?ooo`L00000303oool5000000L0oooo1@0000040?ooo`<000001`3oool3000001X0oooo1000
001@0?ooo`<000001@3oool200000240oooo0`00000_0?ooo`P000005P3oool00;/0oooo1@000007
0?ooo`D00000103oool<000000<0oooo0`00000@0?ooo`H00000203oool:000000T0oooo1P00001=
0?ooo`D00000203oool5000000P0oooo1@00000=0?ooo`D000001`3oool5000000@0oooo0`000007
0?ooo`<000006`3oool300000500oooo0`0000060?ooo`030000003oool0oooo01l0oooo0`00000`
0?ooo`D00000603oool00?l0ooooo`3ooonO0?ooo`00o`3ooooo0?oooil0oooo003o0?ooool0oooo
W`3oool00?l0ooooo`3ooonO0?ooo`00o`3ooooo0?oooil0oooo003o0?ooool0ooooW`3oool00?l0
ooooo`3ooonO0?ooo`00o`3ooooo0?oooil0oooo003o0?ooool0ooooW`3oool00?l0ooooo`3ooonO
0?ooo`00o`3ooooo0?oooil0oooo003o0?ooool0ooooW`3oool00?l0ooooo`3ooonO0?ooo`00o`3o
oooo0?oooil0oooo003o0?ooool0ooooW`3oool00?l0ooooo`3ooonO0?ooo`00o`3ooooo0?oooil0
oooo003o0?ooool0ooooW`3oool00?l0ooooo`3ooonO0?ooo`00o`3ooooo0?oooil0oooo003o0?oo
ool0ooooW`3oool00?l0ooooo`3ooonO0?ooo`00o`3ooooo0?oooil0oooo003o0?ooool0ooooW`3o
ool00?l0ooooo`3ooonO0?ooo`00o`3ooooo0?oooil0oooo003o0?ooool0ooooW`3oool00?l0oooo
o`3ooonO0?ooo`00o`3ooooo0?oooil0oooo003o0?ooool0ooooW`3oool00?l0ooooo`3ooonO0?oo
o`00\
\>"],
  ImageRangeCache->{{{0, 668.938}, {473.938, 0}} -> {-1.05079, -0.83546, \
0.00471252, 0.00471252}, {{16.125, 652.75}, {404.688, 11.25}} -> {-1066.77, \
-5995.99, 7.5219, 58.801}, {{149.813, 595.438}, {462.625, 418.063}} -> \
{-1.72717, -0.69912, 0.0162919, 0.0794398}, {{166.375, 258.438}, {459.188, \
440.313}} -> {-22.5068, -2.56294, 0.158942, 0.158942}, {{273.188, 365.25}, \
{459.188, 440.313}} -> {-33.8248, -2.56294, 0.158942, 0.158942}, {{379.938, \
472.063}, {459.188, 440.313}} -> {-45.1395, -2.56294, 0.158942, 0.158942}, \
{{486.75, 578.813}, {459.188, 440.313}} -> {-56.4542, -2.56294, 0.158942, \
0.158942}}],

Cell["\<\
We did similar analyses for numerical linear algebra operations such as \
LinearSolve, Eigenvalues, SingularValueDecomposition\
\>", "Text"]
}, Open  ]],

Cell["", "Section",
  CellTags->{"PlatformSupport", "LastSlide"}]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 723}},
ScreenStyleEnvironment->"SlideShow",
WindowSize->{1016, 696},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
Magnification->1.5,
StyleDefinitions -> "HelpBrowser.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "DenseLinearAlgebra"->{
    Cell[3373, 96, 83, 1, 76, "Section",
      CellTags->"DenseLinearAlgebra"]},
  "PlatformSupport"->{
    Cell[308288, 6679, 65, 1, 76, "Section",
      CellTags->{"PlatformSupport", "LastSlide"}]},
  "LastSlide"->{
    Cell[308288, 6679, 65, 1, 76, "Section",
      CellTags->{"PlatformSupport", "LastSlide"}]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"DenseLinearAlgebra", 309055, 6700},
  {"PlatformSupport", 309159, 6703},
  {"LastSlide", 309273, 6706}
  }
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 1594, 41, 20, "SlideShowNavigationBar"],

Cell[CellGroupData[{
Cell[3373, 96, 83, 1, 76, "Section",
  CellTags->"DenseLinearAlgebra"],
Cell[3459, 99, 391, 8, 87, "Text"],
Cell[3853, 109, 112, 3, 37, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4002, 117, 1594, 41, 20, "SlideShowNavigationBar"],
Cell[5599, 160, 231, 8, 44, "MathCaption"],
Cell[5833, 170, 69, 2, 51, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5939, 177, 1594, 41, 20, "SlideShowNavigationBar"],
Cell[7536, 220, 122, 3, 43, "MathCaption"],
Cell[7661, 225, 494, 13, 103, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8192, 243, 1594, 41, 20, "SlideShowNavigationBar"],
Cell[9789, 286, 106, 2, 43, "MathCaption"],
Cell[9898, 290, 358, 9, 78, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10293, 304, 1594, 41, 20, "SlideShowNavigationBar"],
Cell[11890, 347, 256, 4, 87, "Text"],
Cell[12149, 353, 417, 7, 112, "Text"],
Cell[12569, 362, 156016, 3438, 529, 43401, 2042, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[168622, 3805, 1594, 41, 20, "SlideShowNavigationBar"],
Cell[170219, 3848, 680, 12, 162, "Text"],
Cell[170902, 3862, 137218, 2809, 496, 30220, 1482, "GraphicsData", \
"PostScript", "Graphics"],
Cell[308123, 6673, 150, 3, 62, "Text"]
}, Open  ]],
Cell[308288, 6679, 65, 1, 76, "Section",
  CellTags->{"PlatformSupport", "LastSlide"}]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

