(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     27403,        960]*)
(*NotebookOutlinePosition[     28146,        985]*)
(*  CellTagsIndexPosition[     28102,        981]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
  "Preview of New and Improved Features\nfor \n",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 5.0"
}], "Chapter",
  TextAlignment->Center],

Cell[CellGroupData[{

Cell["", "SubsectionIcon"],

Cell[CellGroupData[{

Cell[TextData[{
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 5.0 Input Cell Initializations"
}], "SubsubsectionIcon"],

Cell["\<\
Set the directory to be $Presentation/Data where we have data files and where \
we can write things. \
\>", "MathCaption"],

Cell[BoxData[
    \(\($PresentationDirectory = "\<C:\\Documents and Settings\\rknapp\\My \
Documents\\Demos\\Mathematica 5\>";\)\)], "Input",
  CellLabel->"In[1]:="],

Cell[BoxData[
    \(\($Path\  = \ 
        Join[$Path, \ {$PresentationDirectory <> "\<\\Packages\>"}];\)\)], \
"Input",
  CellLabel->"In[2]:="],

Cell[BoxData[
    \(\(SetDirectory[\ 
        ToFileName[$PresentationDirectory, \ "\<Data\>"]\ ];\)\)], "Input",
  CellLabel->"In[3]:="],

Cell["Avoid our favourite message ...", "MathCaption"],

Cell[BoxData[{
    \(\(Off[\ General::spell1\ ];\)\), "\[IndentingNewLine]", 
    \(\(Off[\ General::spell\ ];\)\)}], "Input",
  CellLabel->"In[4]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Parallel Computing Toolkit Initialization", "SubsubsectionIcon"],

Cell["This loads Parallel Computing Toolkit", "MathCaption"],

Cell[BoxData[
    \(<< Parallel`\)], "Input",
  CellLabel->"In[6]:="],

Cell[CellGroupData[{

Cell[BoxData[
    \(M50\  = \ 
      LaunchSlave["\<localhost\>", \ \[IndentingNewLine]"\<\"C:\\Program \
Files\\Wolfram Research\\Mathematica\\5.0\\MathKernel\" -mathlink\>"\ ]\)], \
"Input",
  CellLabel->"In[7]:="],

Cell[BoxData[
    \(LinkObject[
      "C:\\Program Files\\Wolfram Research\\Mathematica\\5.0\\MathKernel", 2, 
      2]\)], "Output",
  CellLabel->"Out[7]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(M42\  = \ 
      LaunchSlave["\<localhost\>", \ \[IndentingNewLine]"\<\"C:\\Program \
Files\\Wolfram Research\\Mathematica\\4.2\\MathKernel\" -mathlink\>"\ ]\)], \
"Input",
  CellLabel->"In[8]:="],

Cell[BoxData[
    \(LinkObject[
      "C:\\Program Files\\Wolfram Research\\Mathematica\\4.2\\MathKernel", 3, 
      3]\)], "Output",
  CellLabel->"Out[8]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\((R\  = \ RemoteEvaluate)\)[$Version]\)], "Input",
  CellLabel->"In[9]:="],

Cell[BoxData[
    \({"5.0 for Microsoft Windows (June 10, 2003)", 
      "4.2 for Microsoft Windows (August 23, 2002)"}\)], "Output",
  CellLabel->"Out[9]="]
}, Open  ]],

Cell["\<\
Need to do some setup for 4.2 in order to compare performance.\
\>", "MathCaption"],

Cell[BoxData[
    \(\(RemoteEvaluate[\ 
        SetAttributes[\ AbsoluteTiming, \ HoldAll\ ], \ {M42}];\)\)], "Input",\

  CellLabel->"In[10]:="],

Cell[BoxData[
    \(\(RemoteEvaluate[\ \(AbsoluteTiming[\ 
              expr_\ ]\  := \[IndentingNewLine]Module[\ {t\  = \ 
                  AbsoluteTime[], \ e}, \[IndentingNewLine]e\  = \ 
                expr; \[IndentingNewLine]{AbsoluteTime[] - t, \ 
                e}\[IndentingNewLine]];\), \ {M42}];\)\)], "Input",
  CellLabel->"In[11]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Numerics", "SubsectionIcon"],

Cell[TextData[{
  "In the area of numerical computation, ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 5 brings improvements not only in speed, but also in the depth and \
breadth of algorithms."
}], "Text"],

Cell[CellGroupData[{

Cell["\<\
Dense Numerical Linear Algebra has been converted to use LAPACK/BLAS\
\>", "SubsubsectionIcon",
  FontColor->RGBColor[0.500008, 0, 0]],

Cell[TextData[{
  "For numerical linear algebra, LAPACK (Linear Algebra PACKage) has become \
the library of choice.  LAPACK evolved from LINPACK which is what numerical \
linear algebra in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 4.2 and earlier is based on.  A basic idea of LAPACK was to build the \
library on top of  small basic commonly needed operations which were simple \
enough that they could be optimized for a particular machine architecture.  \
These Basic Linear Algebra Subroutines (BLAS) form the core of LAPACK.  \
LAPACK further improves on LINPACK by using block algorithms to get better \
performance and by using more robust numerical algorithms.  ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s higher level linear algebra functions provide a platform independent \
interface to LAPACK which runs with special machine optimized BLAS libraries \
for the many platforms we support. [",
  ButtonBox["More...",
    ButtonData:>{"Dense Linear Algebra.nb", None},
    ButtonStyle->"Hyperlink"],
  "]"
}], "Text"],

Cell["\<\
We have taken the ideas from machine number LAPACK/BLAS a step farther and \
optimized a few very basic routines for arbitrary precision numbers.  This \
translates into improved robustness and modest speed improvements for \
arbitrary precision matrices.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Additional Linear Algebra Functions", "SubsubsectionIcon"],

Cell[TextData[{
  "\[Bullet] New function ",
  StyleBox["MatrixRank", "MR"],
  " for computing the rank of an exact or approximate matrix."
}], "Text"],

Cell[TextData[{
  "\[Bullet] New function ",
  StyleBox["Norm", "MR"],
  " for computing the norm of a vector or matrix."
}], "Text"],

Cell[TextData[{
  "\[Bullet] New function ",
  StyleBox["SingularValueList", "MR"],
  " for computing the singular values only."
}], "Text"],

Cell[TextData[{
  "\[Bullet] New function ",
  StyleBox["SingularValueDecomposition", "MR"],
  " for computing the full SVD."
}], "Text"],

Cell[TextData[{
  "\[Bullet] New function ",
  StyleBox["CholeskyDecomposition", "MR"],
  "."
}], "Text"],

Cell[TextData[{
  "\[Bullet] Generalized ",
  StyleBox["Eigenvalues", "MR"],
  ", ",
  StyleBox["Eigenvectors", "MR"],
  " and ",
  StyleBox["Eigensystem", "MR"],
  ". "
}], "Text"],

Cell[TextData[{
  "\[Bullet] Generalized ",
  StyleBox["SchurDecomposition", "MR"],
  ". "
}], "Text"],

Cell[TextData[{
  "\[Bullet] Generalized ",
  StyleBox["SingularValueDecomposition", "MR"],
  ". "
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["New Fast Sparse Numerical Linear Algebra", "SubsubsectionIcon"],

Cell[TextData[{
  "In ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 5.0, a new data structure which supports sparse matrices has been \
introduced.  ",
  StyleBox["SparseArray", "MR"],
  " objects provide a sparse representation for arrays of any dimensions.  As \
a new piece in the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " language, the representation has been integrated into the system in a \
thorough way. [",
  ButtonBox["More...",
    ButtonData:>{"Sparse.nb", None},
    ButtonStyle->"Hyperlink"],
  "]"
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Large Scale Linear Programming", "SubsubsectionIcon"],

Cell[TextData[{
  "A way of solving large resource allocation problems that is typically used \
in finance and engineering.  ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " has had a LinearProgramming command in the past based on the simplex and \
revised simplex algorithms.  ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 5.0 includes an interior point algorithm with which it is realistic to \
solve sparse problems with a million variables. [",
  ButtonBox["Examples...",
    ButtonData:>{"LinearProgramming.nb", None},
    ButtonStyle->"Hyperlink"],
  "]"
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Options for monitoring progress of numerical algorithms", \
"SubsubsectionIcon"],

Cell[TextData[{
  "In ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 5.0 the ",
  StyleBox["EvaluationMonitor", "MR"],
  " and ",
  StyleBox["StepMonitor", "MR"],
  " options have been added to several functions to allow you to follow the \
progress of various numerical algorithms. The table shows which functions \
support these options.  [",
  ButtonBox["More...",
    ButtonData:>{"Monitors.nb", None},
    ButtonStyle->"Hyperlink"],
  "]  \n",
  Cell[BoxData[
      FormBox[GridBox[{
            {"Function", 
              StyleBox["EvaluationMonitor",
                "MR"], 
              StyleBox["StepMonitor",
                "MR"]},
            {
              StyleBox["FindFit",
                "MR"], "\[FilledSquare]", "\[FilledSquare]"},
            {
              StyleBox["FindMaximum",
                "MR"], "\[FilledSquare]", "\[FilledSquare]"},
            {
              StyleBox["FindMinimum",
                "MR"], "\[FilledSquare]", "\[FilledSquare]"},
            {
              StyleBox["FindRoot",
                "MR"], "\[FilledSquare]", "\[FilledSquare]"},
            {
              StyleBox["NDSolve",
                "MR"], "\[FilledSquare]", "\[FilledSquare]"},
            {
              StyleBox["NIntegrate",
                "MR"], "\[FilledSquare]", " "},
            {
              StyleBox["NMaximize",
                "MR"], "\[FilledSquare]", "\[FilledSquare]"},
            {
              StyleBox["NMinimize",
                "MR"], "\[FilledSquare]", "\[FilledSquare]"},
            {
              StyleBox["NProduct",
                "MR"], "\[FilledSquare]", " "},
            {
              StyleBox["NSum",
                "MR"], "\[FilledSquare]", " "}
            },
          GridFrame->True,
          RowLines->True,
          ColumnLines->True], TraditionalForm]]]
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Array valued variables supported in several numerical functions\
\>", "SubsubsectionIcon"],

Cell[TextData[{
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 5.0 includes many new state of the art algorithms.  Some of these are \
really only effective for large scale problems.  Sometimes the most natural \
way to pose large problems are through (black box) functions which operate on \
vectors or arrays.  In ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 5.0, some of the numerical functions are able to interpret some or all of \
their variables as vectors, based on initial conditions. [",
  ButtonBox["More...",
    ButtonData:>{"VectorVariables.nb", None},
    ButtonStyle->"Hyperlink"],
  "]"
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Finite difference computation of gradients and Jacobian matrices\
\>", "SubsubsectionIcon"],

Cell[TextData[{
  "In differential equations and optimization problems, it is frequently \
necessary to find the derivative with respect to all of the problem \
variables.  ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " typically does this symbolically, but there are cases where the symbolic \
computation is very time consuming or symbolic derivatives do not exist.  In \
these cases it is useful to use finite differences to approximate needed \
derivatives.   When the sparse structure of a Jacobian or Hessian matrix can \
be determined, a sparse algorithm is used which substantially reduces the \
number of function evaluations required to approximate the derivative.  For \
PDE solutions, where the Jacobian is typically very sparse, this has a \
dramatic effect on execution time."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Constrained global optimization functions NMinimize and NMaximize\
\>", "SubsubsectionIcon"],

Cell[TextData[{
  StyleBox["NMinimize", "MR"],
  " and ",
  StyleBox["NMaximize", "MR"],
  " solve constrained or unconstrained global optimization problems.  For \
linear problems, they use linear programming.  For general nonlinear \
problems, the methods Nelder-Mead, Differential Evolution, Simulated \
Annealing and Random Search are employed."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Substantial improvements in NDSolve for numerical solutions of differential \
equations.\
\>", "SubsubsectionIcon"],

Cell[TextData[{
  "For ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 5.0 the ",
  StyleBox["NDSolve", "MR"],
  " command was completely restructured into a new framework.  There are \
numerous advantages, including speed improvements, especially for large \
systems, many new methods, and flexibility both in user capability of adding \
specialized methods and in controlling iterations.  Included in these changes \
are the ability to solve differential-algebraic equations (DAEs) and 1+n \
dimensional intial-boundary value problem partial differential equations.  \
[",
  ButtonBox["More...",
    ButtonData:>{"NDSolve.nb", None},
    ButtonStyle->"Hyperlink"],
  "]"
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Faster arbitrary precision and large integer arithmetic", \
"SubsubsectionIcon"],

Cell[TextData[{
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " has always had the ability to compute with arbitrary precision reals and \
very large integers.  The algorithms in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 1.0 were reasonably good at small sizes but asymptotically had quadratic \
complexity.  In later versions, the complexity was reduced to O(n log n).  \
Similar to the LAPACK/BLAS approach used for numerical linear algebra, ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 5.0 uses the GMP (Gnu Multiprecision Library) which provides machine \
optimized support for the fundamental low-level operations required for big \
number operations.  In the 10 - 1000 digit range, this gives a speed \
improvement of up to a factor of 3.  For more digits, the algorithms have the \
same asymptotic complexity, but run somewhat faster and use much less memory \
than those used by ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 4.2.  The speed increases propagate through all of the elementary and \
special functions which are based ultimately on arithmetic."
}], "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Symbolics", "SubsectionIcon"],

Cell[CellGroupData[{

Cell["\<\
Reduce can now solve equations, inequalities and quantifiers over different \
domains\
\>", "SubsubsectionIcon"],

Cell[TextData[{
  StyleBox["Reduce", "MR"],
  " solves equations and inequalities over different ",
  StyleBox["domains",
    FontSlant->"Italic"],
  ".",
  StyleBox[" Reduce", "MR"],
  " generates ",
  StyleBox["complete solution",
    FontSlant->"Italic"],
  " sets even when they are infinite such as may happen for transcendental \
equations over the Complexes or Reals and polynomial equations over the \
Integers.",
  StyleBox[" Reduce", "MR"],
  " allows you to use ",
  StyleBox["existential and universal quantifiers",
    FontSlant->"Italic"],
  " in the equation specification.   ",
  StyleBox["Reduce", "MR"],
  " extended input allows you to use equations, inequations, inequalities, \
existential quantifiers, universal quantifiers, domain specifications.  [",
  ButtonBox["Examples...",
    ButtonData:>{"Reduce.nb", None},
    ButtonStyle->"Hyperlink"],
  "]"
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["New function FindInstance", "SubsubsectionIcon"],

Cell[TextData[{
  StyleBox["FindInstance", "MR"],
  " is a new function that takes the same input as ",
  StyleBox["Reduce", "MR"],
  " and will produce instances of variables satisfying the equations and \
inequalities.  [",
  ButtonBox["Examples...",
    ButtonData:>{"FindInstance.nb", None},
    ButtonStyle->"Hyperlink"],
  "]"
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
New functions Minimize and Maximize for symbolic optimization\
\>", "SubsubsectionIcon"],

Cell[TextData[{
  StyleBox["Minimize", "MR"],
  " and ",
  StyleBox["Maximize", "MR"],
  " optimize a function over a region specified by equations, inequalities \
and domain specifications.  [",
  ButtonBox["Examples...",
    ButtonData:>{"Minimize.nb", None},
    ButtonStyle->"Hyperlink"],
  "]"
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
New function RSolve for solving difference and q-difference equations\
\>", "SubsubsectionIcon"],

Cell[TextData[{
  StyleBox["RSolve", "MR"],
  " is a comprehensive solver for recurrence or difference equations. It ...\n\
\[Bullet] solves single linear or nonlinear difference equations. \n\[Bullet] \
solves systems of linear and nonlinear difference equations.\n\[Bullet] \
solves difference algebraic equations. \n\[Bullet] solves partial difference \
equations. \n\[Bullet] solves q-difference equations or divide-and-conquer \
equations. \n[",
  ButtonBox["Examples...",
    ButtonData:>{"RSolve.nb", None},
    ButtonStyle->"Hyperlink"],
  "]"
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Improved DSolve support for systems of ODEs and DAEs", \
"SubsubsectionIcon"],

Cell[TextData[{
  StyleBox["DSolve", "MR"],
  " can now find all rational function solutions to systems of linear \
equations with rational coefficients.   Furthermore, ",
  StyleBox["DSolve", "MR"],
  " can now solve all linear systems of constant coefficient differential \
algebraic equations. [",
  ButtonBox["Examples...",
    ButtonData:>{"DSolve.nb", None},
    ButtonStyle->"Hyperlink"],
  "]"
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["New mechanism for propagating assumptions", "SubsubsectionIcon"],

Cell[TextData[{
  "The new command ",
  StyleBox["Refine", "MR"],
  " is essentially symbolic evaluation with assumptions.   The new command \
ing ",
  StyleBox["Assuming", "MR"],
  " works by giving assumptions to all functions which take the ",
  StyleBox["Assumptions", "MR"],
  " option. [",
  ButtonBox["Examples...",
    ButtonData:>{"Assumptions.nb", None},
    ButtonStyle->"Hyperlink"],
  "]"
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
New function for quantifier elimination over different domains\
\>", "SubsubsectionIcon"],

Cell[TextData[{
  "Quantifier elimination is a general and convenient way to formulate many \
types of problems.  The new function ",
  StyleBox["Resolve", "MR"],
  " eliminates quantifiers from formulae to produce results free of \
quantifiers. [",
  ButtonBox["Examples...",
    ButtonData:>{"Resolve.nb", None},
    ButtonStyle->"Hyperlink"],
  "]"
}], "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Import and Export", "SubsectionIcon"],

Cell[CellGroupData[{

Cell["Graphics Formats", "SubsubsectionIcon"],

Cell[TextData[{
  "\[Bullet] Support for DICOM biomedical imaging format.  [",
  ButtonBox["Example...",
    ButtonData:>{"DICOM.nb", None},
    ButtonStyle->"Hyperlink"],
  "]"
}], "Text"],

Cell["\<\
\[Bullet] Support for PNG raster graphics format.\
\>", "Text"],

Cell[TextData[{
  "\[Bullet] Support for SVG vector graphics format.   [",
  ButtonBox["Example...",
    ButtonData:>{"SVG.nb", None},
    ButtonStyle->"Hyperlink"],
  "]"
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Sparse data formats", "SubsubsectionIcon"],

Cell["\<\
\[Bullet] Support for MatrixMarket sparse matrix format. \
\>", "Text"],

Cell["\<\
\[Bullet] Support for Harwell-Boeing sparse matrix formats.\
\>", "Text"],

Cell["\<\
\[Bullet] Support for MPS linear programming format.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Performance improvements", "SubsubsectionIcon"],

Cell["\<\
\[Bullet] Improved export of XHTML, now generating cascading style sheets.\
\>", "Text"],

Cell[TextData[{
  "\[Bullet] Performance improvement to tabular ",
  StyleBox["Import", "MR"],
  " and ",
  StyleBox["Export", "MR"],
  ". "
}], "Text"],

Cell[TextData[{
  "\[Bullet] Performance improvement to raster graphics ",
  StyleBox["Import", "MR"],
  " and ",
  StyleBox["Export", "MR"],
  ". "
}], "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["System and Language", "SubsectionIcon"],

Cell[TextData[{
  "\[Bullet] New ",
  StyleBox["Sow", "MR"],
  " and ",
  StyleBox["Reap", "MR"],
  " functional programming constructs. "
}], "Text"],

Cell[TextData[{
  "\[Bullet] ",
  StyleBox["SparseArray", "MR"],
  " support integrated into the language. "
}], "Text"],

Cell[TextData[{
  "\[Bullet] New ",
  StyleBox["AbsoluteTiming", "MR"],
  " function for wall clock timing."
}], "Text"],

Cell[TextData[{
  "\[Bullet] Improved resolution for ",
  StyleBox["AbsoluteTime", "MR"],
  " and ",
  StyleBox["Pause", "MR"],
  "."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["MathLink", "SubsectionIcon"],

Cell[CellGroupData[{

Cell[TextData[{
  "Performance improvement for ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " over TCPIP."
}], "SubsubsectionIcon"],

Cell[TextData[{
  "This affects you when you are running remote kernels, e.g. grid",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " relies heavily on this.  The new TCPIP communication device, means ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " communicates at the speed of the underlying network. Typical speedup on a \
lightly loaded 100Base-T network are 10x ",
  StyleBox["bandwidth",
    FontSlant->"Italic"],
  " improvement and 200x ",
  StyleBox["latency",
    FontSlant->"Italic"],
  " improvement. Even greater speedups are expected to increase on a gigabit \
network (our measurements for that case are not done yet)."
}], "Text"],

Cell[TextData[{
  "Bandwidth speedups on 100Mb/s network. Command: ",
  StyleBox["Plot3D[Sin[x y],{x,0,2Pi},{y,0,-2Pi},PlotPoints\[Rule]500, Mesh\
\[Rule]False]//AbsoluteTiming", "Program"]
}], "MathCaption"],

Cell[BoxData[GridBox[{
        {\(FE/Kernel\), "Windows", "Linux", "MacOSX"},
        {"Windows", 
          "\[Placeholder]", \(140.2/15.6 = 8.9\), \(136.0/15.1 = 9.0\)},
        {"Linux", \(96.8/14.2 = 6.8\), "\[Placeholder]", \(63.3/9.8 = 6.4\)},
        {"MacOSX", \(123.3/14.1 = 8.7\), \(134.5/12.6 = 10.6\), 
          "\[Placeholder]"}
        },
      GridFrame->True,
      RowLines->True,
      ColumnLines->True]], "Text"],

Cell[TextData[{
  "Latency improvements on 100Mb/s network. Command: ",
  StyleBox["Do[RemoteEvaluate[$MachineName],{10000}]//AbsoluteTiming", 
    "Program"]
}], "MathCaption"],

Cell[BoxData[GridBox[{
        {\(Kernel/Kernel\), "Windows", "Linux", "MacOSX"},
        {"Windows", 
          "\[Placeholder]", \(1999.7/10.0 = 199.9\), \(2009.9/9.0 = 223.3\)},
        {"Linux", \(2001.3/11.7 = 171.0\), 
          "\[Placeholder]", \(1981.2/11.4 = 173.7\)},
        {"MacOSX", \(2013.4/9.0 = 223.7\), \(1999.9/9.9 = 202.010101010101`\),
           "\[Placeholder]"}
        },
      GridFrame->True,
      RowLines->True,
      ColumnLines->True]], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "Performance improvement for ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " on Windows."
}], "SubsubsectionIcon"],

Cell[TextData[{
  "Major performance improvement with new ",
  StyleBox["SharedMemory", "MR"],
  " protocol for Windows platforms. Bandwidth 10x improvement."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "New ",
  StyleBox[".NETLink", "SR",
    FontSlant->"Italic"],
  " binding of ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " ",
  "for Microsoft .NET"
}], "SubsubsectionIcon"],

Cell[TextData[{
  "New ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " framework to support .net in a similar way that we support Java.   \
Similar to using J/Link with Java, it allows you to load any .net object into \
",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " and extend it. With minimal extra effort, you can call any DLL and COM \
object from within ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "."
}], "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["User Interface", "SubsectionIcon"],

Cell["\<\
\[Bullet] Improved HelpBrowser design. \
\>", "Text"],

Cell["\<\
\[Bullet] Improved Copy and Paste on Windows.\
\>", "Text"],

Cell["\[Bullet] Improved Slide show support.", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["64 bit Platform Support", "SubsectionIcon"],

Cell[TextData[{
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " is now optimized for large number of 64-bit platforms. \n\[Bullet] Linux \
IA64\n\[Bullet] Sun Solaris UltraSPARC\n\[Bullet] HP HP-UX PA-RISC\n\[Bullet] \
IBM AIX POWER\n\[Bullet] HP Tru64 UNIX Alpha\n\[Bullet] Linux Alpha"
}], "Text"],

Cell[TextData[{
  "The single most important feature that a 64-bit platform offer a ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " user is clearly that it makes is possible to work on problems larger then \
2 GB in size. In fact with ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 5.0 it is not very hard to hit the 2GB memory limit imposed by 32-bit \
platforms. "
}], "Text"],

Cell["\<\
In addition there are a few performance improvements as well. For instance \
big number arithmetic does speed up with 64-bit platforms.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Summary", "SubsectionIcon"],

Cell[TextData[{
  "There are three different themes running through many of the features I \
have demonstrated above.  First, we are actively looking into the use of \
highly optimized libraries to get the best  speed possible for basic \
operations on the wide variety of platforms we support.  Second, we are \
actively pursuing algorithm development which will allow you to pursue large \
and large sized problems with efficient, robust methods.  Finally, we are \
making many of the building blocks for the higher level algorithms, such as \
SparseArrays, available as features to ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " users.  In the future, you should expect to see similar development in \
further areas with successive versions of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ".  "
}], "Text"]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 723}},
ScreenStyleEnvironment->"Brackets",
WindowSize->{1016, 696},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
Magnification->1.5,
StyleDefinitions -> "New In 5.0 Style.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 166, 6, 216, "Chapter"],

Cell[CellGroupData[{
Cell[1967, 63, 26, 0, 58, "SubsectionIcon"],

Cell[CellGroupData[{
Cell[2018, 67, 128, 4, 35, "SubsubsectionIcon"],
Cell[2149, 73, 132, 3, 64, "MathCaption"],
Cell[2284, 78, 165, 3, 66, "Input"],
Cell[2452, 83, 144, 4, 40, "Input"],
Cell[2599, 89, 137, 3, 40, "Input"],
Cell[2739, 94, 54, 0, 64, "MathCaption"],
Cell[2796, 96, 150, 3, 66, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[2983, 104, 70, 0, 27, "SubsubsectionIcon"],
Cell[3056, 106, 60, 0, 58, "MathCaption"],
Cell[3119, 108, 69, 2, 40, "Input"],

Cell[CellGroupData[{
Cell[3213, 114, 216, 5, 92, "Input"],
Cell[3432, 121, 157, 4, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3626, 130, 216, 5, 92, "Input"],
Cell[3845, 137, 157, 4, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4039, 146, 95, 2, 40, "Input"],
Cell[4137, 150, 157, 3, 70, "Output"]
}, Open  ]],
Cell[4309, 156, 93, 2, 58, "MathCaption"],
Cell[4405, 160, 145, 4, 40, "Input"],
Cell[4553, 166, 350, 6, 144, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[4952, 178, 34, 0, 39, "SubsectionIcon"],
Cell[4989, 180, 220, 6, 56, "Text"],

Cell[CellGroupData[{
Cell[5234, 190, 144, 3, 35, "SubsubsectionIcon"],
Cell[5381, 195, 1060, 21, 210, "Text"],
Cell[6444, 218, 279, 5, 78, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[6760, 228, 64, 0, 27, "SubsubsectionIcon"],
Cell[6827, 230, 151, 4, 35, "Text"],
Cell[6981, 236, 133, 4, 35, "Text"],
Cell[7117, 242, 140, 4, 35, "Text"],
Cell[7260, 248, 137, 4, 35, "Text"],
Cell[7400, 254, 105, 4, 35, "Text"],
Cell[7508, 260, 181, 8, 35, "Text"],
Cell[7692, 270, 102, 4, 35, "Text"],
Cell[7797, 276, 110, 4, 35, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[7944, 285, 69, 0, 27, "SubsubsectionIcon"],
Cell[8016, 287, 554, 17, 101, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[8607, 309, 59, 0, 27, "SubsubsectionIcon"],
Cell[8669, 311, 595, 15, 100, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[9301, 331, 86, 1, 27, "SubsubsectionIcon"],
Cell[9390, 334, 1856, 56, 374, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[11283, 395, 100, 2, 27, "SubsubsectionIcon"],
Cell[11386, 399, 638, 15, 122, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[12061, 419, 101, 2, 27, "SubsubsectionIcon"],
Cell[12165, 423, 814, 14, 166, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[13016, 442, 102, 2, 27, "SubsubsectionIcon"],
Cell[13121, 446, 361, 8, 79, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[13519, 459, 125, 3, 50, "SubsubsectionIcon"],
Cell[13647, 464, 695, 17, 123, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[14379, 486, 86, 1, 27, "SubsubsectionIcon"],
Cell[14468, 489, 1129, 22, 232, "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[15646, 517, 35, 0, 39, "SubsectionIcon"],

Cell[CellGroupData[{
Cell[15706, 521, 122, 3, 58, "SubsubsectionIcon"],
Cell[15831, 526, 887, 25, 147, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[16755, 556, 54, 0, 27, "SubsubsectionIcon"],
Cell[16812, 558, 344, 10, 57, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[17193, 573, 98, 2, 27, "SubsubsectionIcon"],
Cell[17294, 577, 310, 10, 57, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[17641, 592, 106, 2, 27, "SubsubsectionIcon"],
Cell[17750, 596, 563, 12, 215, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[18350, 613, 83, 1, 27, "SubsubsectionIcon"],
Cell[18436, 616, 413, 11, 80, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[18886, 632, 70, 0, 27, "SubsubsectionIcon"],
Cell[18959, 634, 413, 13, 80, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[19409, 652, 99, 2, 27, "SubsubsectionIcon"],
Cell[19511, 656, 363, 10, 79, "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[19923, 672, 43, 0, 39, "SubsectionIcon"],

Cell[CellGroupData[{
Cell[19991, 676, 45, 0, 35, "SubsubsectionIcon"],
Cell[20039, 678, 189, 6, 34, "Text"],
Cell[20231, 686, 73, 2, 34, "Text"],
Cell[20307, 690, 183, 6, 34, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[20527, 701, 48, 0, 27, "SubsubsectionIcon"],
Cell[20578, 703, 81, 2, 34, "Text"],
Cell[20662, 707, 83, 2, 34, "Text"],
Cell[20748, 711, 76, 2, 34, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[20861, 718, 53, 0, 27, "SubsubsectionIcon"],
Cell[20917, 720, 98, 2, 34, "Text"],
Cell[21018, 724, 152, 6, 35, "Text"],
Cell[21173, 732, 160, 6, 35, "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[21382, 744, 45, 0, 39, "SubsectionIcon"],
Cell[21430, 746, 150, 6, 35, "Text"],
Cell[21583, 754, 120, 4, 35, "Text"],
Cell[21706, 760, 120, 4, 35, "Text"],
Cell[21829, 766, 145, 6, 35, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[22011, 777, 34, 0, 39, "SubsectionIcon"],

Cell[CellGroupData[{
Cell[22070, 781, 140, 5, 35, "SubsubsectionIcon"],
Cell[22213, 788, 660, 16, 122, "Text"],
Cell[22876, 806, 208, 4, 87, "MathCaption"],
Cell[23087, 812, 433, 10, 112, "Text"],
Cell[23523, 824, 177, 4, 86, "MathCaption"],
Cell[23703, 830, 477, 11, 112, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[24217, 846, 140, 5, 27, "SubsubsectionIcon"],
Cell[24360, 853, 170, 4, 57, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[24567, 862, 202, 9, 28, "SubsubsectionIcon"],
Cell[24772, 873, 451, 14, 78, "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[25272, 893, 40, 0, 39, "SubsectionIcon"],
Cell[25315, 895, 63, 2, 34, "Text"],
Cell[25381, 899, 69, 2, 34, "Text"],
Cell[25453, 903, 54, 0, 34, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[25544, 908, 49, 0, 39, "SubsectionIcon"],
Cell[25596, 910, 307, 6, 214, "Text"],
Cell[25906, 918, 401, 10, 78, "Text"],
Cell[26310, 930, 159, 3, 56, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[26506, 938, 33, 0, 39, "SubsectionIcon"],
Cell[26542, 940, 833, 16, 166, "Text"]
}, Closed]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

