(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     29312,        869]*)
(*NotebookOutlinePosition[     30534,        910]*)
(*  CellTagsIndexPosition[     30386,        902]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[TextData[Cell[BoxData[GridBox[{
        {
          ButtonBox[
            StyleBox["\[FirstPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageFirst"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"First Slide"], 
          ButtonBox[
            StyleBox["\[LeftPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPagePrevious"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Previous Slide"], 
          ButtonBox[
            StyleBox["\[RightPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageNext"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Next Slide"], 
          ButtonBox[
            StyleBox["\[LastPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageLast"]}],
            ButtonStyle->"SlideHyperlink"], " ", 
          ButtonBox[
            StyleBox[
              RowBox[{
                CounterBox["SlideShowNavigationBar"], \(\(\ \)\(of\)\(\ \)\), 
                
                CounterBox["SlideShowNavigationBar", "LastSlide"]}],
              "SR"],
            ButtonFrame->"None"]}
        }]]]], "SlideShowNavigationBar"],

Cell[CellGroupData[{

Cell["Sparse Representations of Arrays", "Section",
  CellTags->"DenseLinearAlgebra"],

Cell[TextData[{
  "The ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " function SparseArray can be used to convert from ordinary arrays or as a \
constructor from index coordinate rules."
}], "Text"],

Cell["\<\
Here a pattern rule is used to construct a tridiagonal matrix.\
\>", "MathCaption"],

Cell[BoxData[
    \(SparseArray[{{i_, \ i_} \[Rule] 
          i, \ {i_, j_}\  /; \ Abs[i - j]\  \[Equal] \ \ 1 \[Rule] 
          1/\((i\  + \ j)\)}, \ {8, 8}]\)], "Input",
  CellLabel->"In[14]:="],

Cell["\<\
The output form is abbreviated because they use a very efficient (Compressed \
Sparse Row) representation for the representation.\
\>", "Text"],

Cell["You can see the structure with MatrixForm", "MathCaption"],

Cell[BoxData[
    \(MatrixForm[%]\)], "Input",
  CellLabel->"In[15]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[GridBox[{
        {
          ButtonBox[
            StyleBox["\[FirstPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageFirst"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"First Slide"], 
          ButtonBox[
            StyleBox["\[LeftPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPagePrevious"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Previous Slide"], 
          ButtonBox[
            StyleBox["\[RightPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageNext"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Next Slide"], 
          ButtonBox[
            StyleBox["\[LastPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageLast"]}],
            ButtonStyle->"SlideHyperlink"], " ", 
          ButtonBox[
            StyleBox[
              RowBox[{
                CounterBox["SlideShowNavigationBar"], \(\(\ \)\(of\)\(\ \)\), 
                
                CounterBox["SlideShowNavigationBar", "LastSlide"]}],
              "SR"],
            ButtonFrame->"None"]}
        }]]]], "SlideShowNavigationBar"],

Cell["\<\
The real advantage of the sparse representation shows when the matrix or \
array dimensions are large but there are relatively few nonzero entries.\
\>", "Text"],

Cell["\<\
This imports a large sparse matrix from a file in Harwell-Boeing format.\
\>", "MathCaption"],

Cell[BoxData[
    \(AbsoluteTiming[
      S\  = \ Import["\<finan512.rsa\>", \ "\<HarwellBoeing\>"]]\)], "Input",
  CellLabel->"In[16]:="],

Cell["\<\
A function MatrixPlot has been added to show the structure of matrices.\
\>", "MathCaption"],

Cell[BoxData[
    \(Needs["\<LinearAlgebra`MatrixManipulation`\>"]\)], "Input",
  CellLabel->"In[17]:="],

Cell[BoxData[
    \(\(MatrixPlot[S];\)\)], "Input",
  CellLabel->"In[18]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[GridBox[{
        {
          ButtonBox[
            StyleBox["\[FirstPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageFirst"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"First Slide"], 
          ButtonBox[
            StyleBox["\[LeftPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPagePrevious"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Previous Slide"], 
          ButtonBox[
            StyleBox["\[RightPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageNext"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Next Slide"], 
          ButtonBox[
            StyleBox["\[LastPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageLast"]}],
            ButtonStyle->"SlideHyperlink"], " ", 
          ButtonBox[
            StyleBox[
              RowBox[{
                CounterBox["SlideShowNavigationBar"], \(\(\ \)\(of\)\(\ \)\), 
                
                CounterBox["SlideShowNavigationBar", "LastSlide"]}],
              "SR"],
            ButtonFrame->"None"]}
        }]]]], "SlideShowNavigationBar"],

Cell[TextData[{
  "The SparseArray data structure is well integrated into the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " system, so you do arithmetic, structural operations, and linear algebra \
operations just as with ordinary arrays."
}], "Text"],

Cell["\<\
This computes the matrix product of the sparse matrix with itself.\
\>", "MathCaption"],

Cell[BoxData[
    \(Timing[S . S]\)], "Input",
  CellLabel->"In[19]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[GridBox[{
        {
          ButtonBox[
            StyleBox["\[FirstPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageFirst"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"First Slide"], 
          ButtonBox[
            StyleBox["\[LeftPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPagePrevious"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Previous Slide"], 
          ButtonBox[
            StyleBox["\[RightPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageNext"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Next Slide"], 
          ButtonBox[
            StyleBox["\[LastPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageLast"]}],
            ButtonStyle->"SlideHyperlink"], " ", 
          ButtonBox[
            StyleBox[
              RowBox[{
                CounterBox["SlideShowNavigationBar"], \(\(\ \)\(of\)\(\ \)\), 
                
                CounterBox["SlideShowNavigationBar", "LastSlide"]}],
              "SR"],
            ButtonFrame->"None"]}
        }]]]], "SlideShowNavigationBar"],

Cell["\<\
Several algorithms, both direct and iterative are available for solving \
linear systems.\
\>", "Text"],

Cell["\<\
This computes a sparse LU decomposition using a multifrontal method.\
\>", "MathCaption"],

Cell[BoxData[
    \(Timing[lu\  = \ LinearSolve[S]]\)], "Input",
  CellLabel->"In[20]:="],

Cell["\<\
This generates a vector for which it is easy to verify the correctness of the \
solution\
\>", "MathCaption"],

Cell[BoxData[
    \(\(b\  = \ S . Range[Length[S]];\)\)], "Input",
  CellLabel->"In[21]:="],

Cell["\<\
This shows the Norm of the relative error in the solution found by \
backsubstituting \
\>", "MathCaption"],

Cell[BoxData[
    \(Timing[Norm[1\  - \ lu[b]/Range[Length[S]]]]\)], "Input",
  CellLabel->"In[22]:="],

Cell["\<\
You could, of course, just use LinearSolve[S,b] if you don't expect to use \
the LU decomposition more than once.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[GridBox[{
        {
          ButtonBox[
            StyleBox["\[FirstPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageFirst"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"First Slide"], 
          ButtonBox[
            StyleBox["\[LeftPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPagePrevious"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Previous Slide"], 
          ButtonBox[
            StyleBox["\[RightPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageNext"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Next Slide"], 
          ButtonBox[
            StyleBox["\[LastPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageLast"]}],
            ButtonStyle->"SlideHyperlink"], " ", 
          ButtonBox[
            StyleBox[
              RowBox[{
                CounterBox["SlideShowNavigationBar"], \(\(\ \)\(of\)\(\ \)\), 
                
                CounterBox["SlideShowNavigationBar", "LastSlide"]}],
              "SR"],
            ButtonFrame->"None"]}
        }]]]], "SlideShowNavigationBar"],

Cell["\<\
This solves the system using a preconditioned iterative method and shows the \
relative error.\
\>", "MathCaption"],

Cell[BoxData[
    \(Timing[
      Norm[1\  - \ 
          LinearSolve[S, \ b, \ 
              Method \[Rule] {"\<Krylov\>", \ "\<Preconditioner\>" -> \
"\<ILU\>", \ Method -> "\<ConjugateGradient\>"}]/
            Range[Length[S]]]]\)], "Input",
  CellLabel->"In[23]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[GridBox[{
        {
          ButtonBox[
            StyleBox["\[FirstPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageFirst"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"First Slide"], 
          ButtonBox[
            StyleBox["\[LeftPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPagePrevious"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Previous Slide"], 
          ButtonBox[
            StyleBox["\[RightPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageNext"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Next Slide"], 
          ButtonBox[
            StyleBox["\[LastPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageLast"]}],
            ButtonStyle->"SlideHyperlink"], " ", 
          ButtonBox[
            StyleBox[
              RowBox[{
                CounterBox["SlideShowNavigationBar"], \(\(\ \)\(of\)\(\ \)\), 
                
                CounterBox["SlideShowNavigationBar", "LastSlide"]}],
              "SR"],
            ButtonFrame->"None"]}
        }]]]], "SlideShowNavigationBar"],

Cell[TextData[{
  "For a very large sparse matrix, it is not typically practical to find all \
of the eigvenvalues and/or vectors.  However, ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 5.0 has included algorithms to find collections of them and a natural \
syntax to specify which ones."
}], "Text"],

Cell["This computes the 8 eigenvalues with the largest magnitude.", \
"MathCaption"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[Eigenvalues[S, \ 8]]\)], "Input",
  CellLabel->"In[46]:=",
  Evaluatable->False],

Cell[BoxData[
    \({58.043000000000006`\ Second, {28.183319247597563`, 
        27.934657401976665`, 27.278130526541233`, 26.92170947681477`, 
        25.859939000927707`, 25.64175020887573`, 25.51756701874552`, 
        25.400320840335674`}}\)], "Output",
  CellLabel->"Out[46]="]
}, Closed]],

Cell["This computes the eigenvalue with the smallest magnitude", "MathCaption"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[Eigenvalues[S, \ \(-1\)]]\)], "Input",
  CellLabel->"In[28]:=",
  Evaluatable->False],

Cell[BoxData[
    \({71.303`\ Second, {0.9474684469629978`}}\)], "Output",
  CellLabel->"Out[28]=",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[GridBox[{
        {
          ButtonBox[
            StyleBox["\[FirstPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageFirst"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"First Slide"], 
          ButtonBox[
            StyleBox["\[LeftPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPagePrevious"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Previous Slide"], 
          ButtonBox[
            StyleBox["\[RightPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageNext"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Next Slide"], 
          ButtonBox[
            StyleBox["\[LastPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageLast"]}],
            ButtonStyle->"SlideHyperlink"], " ", 
          ButtonBox[
            StyleBox[
              RowBox[{
                CounterBox["SlideShowNavigationBar"], \(\(\ \)\(of\)\(\ \)\), 
                
                CounterBox["SlideShowNavigationBar", "LastSlide"]}],
              "SR"],
            ButtonFrame->"None"]}
        }]]]], "SlideShowNavigationBar"],

Cell["\<\
To conclude, I'll show you an example which is simple, yet shows how you can \
take advantage of several of the sparse array properties to make \
transformations.\
\>", "Text"],

Cell["\<\
This defines a function Update[c_, n_] which updates a cell based on its \
current value c and the sum of its neighbors values n according to the simple \
game of life cellular automaton rules\
\>", "MathCaption"],

Cell[BoxData[{
    \(\(SetAttributes[CellUpdate, \ Listable];\)\), "\[IndentingNewLine]", 
    \(\(CellUpdate[1, \ 2]\  := \ 1;\)\), "\[IndentingNewLine]", 
    \(\(CellUpdate[_, \ 3]\  := \ 1;\)\), "\[IndentingNewLine]", 
    \(\(CellUpdate[_, \ _]\  := \ 0;\)\)}], "Input",
  CellLabel->"In[52]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[GridBox[{
        {
          ButtonBox[
            StyleBox["\[FirstPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageFirst"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"First Slide"], 
          ButtonBox[
            StyleBox["\[LeftPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPagePrevious"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Previous Slide"], 
          ButtonBox[
            StyleBox["\[RightPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageNext"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Next Slide"], 
          ButtonBox[
            StyleBox["\[LastPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageLast"]}],
            ButtonStyle->"SlideHyperlink"], " ", 
          ButtonBox[
            StyleBox[
              RowBox[{
                CounterBox["SlideShowNavigationBar"], \(\(\ \)\(of\)\(\ \)\), 
                
                CounterBox["SlideShowNavigationBar", "LastSlide"]}],
              "SR"],
            ButtonFrame->"None"]}
        }]]]], "SlideShowNavigationBar"],

Cell["\<\
The Update function has set to be listable so that it can work with a matrix \
of cells at one go.\
\>", "Text"],

Cell["\<\
This defines a function which updates a matrix of cells (with periodic \
repetition)\
\>", "MathCaption"],

Cell[BoxData[
    RowBox[{\(MatrixUpdate[mat_?MatrixQ]\), " ", ":=", " ", 
      "\[IndentingNewLine]", 
      RowBox[{"CellUpdate", "[", 
        RowBox[{"mat", ",", 
          RowBox[{"Total", "[", 
            RowBox[{
              RowBox[{"(", GridBox[{
                    {\(RotateRight[mat, {1, 1}]\), \(RotateRight[
                        mat, {0, 1}]\), \(RotateRight[mat, {1, \(-1\)}]\)},
                    {\(RotateRight[mat, 1]\), 
                      "0", \(RotateRight[mat, \(-1\)]\)},
                    {\(RotateRight[mat, {\(-1\), 1}]\), \(RotateRight[
                        mat, {0, \(-1\)}]\), \(RotateRight[
                        mat, {\(-1\), \(-1\)}]\)}
                    }], ")"}], ",", " ", "2"}], "]"}]}], 
        "\[IndentingNewLine]", "]"}]}]], "Input",
  CellLabel->"In[56]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[GridBox[{
        {
          ButtonBox[
            StyleBox["\[FirstPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageFirst"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"First Slide"], 
          ButtonBox[
            StyleBox["\[LeftPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPagePrevious"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Previous Slide"], 
          ButtonBox[
            StyleBox["\[RightPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageNext"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Next Slide"], 
          ButtonBox[
            StyleBox["\[LastPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageLast"]}],
            ButtonStyle->"SlideHyperlink"], " ", 
          ButtonBox[
            StyleBox[
              RowBox[{
                CounterBox["SlideShowNavigationBar"], \(\(\ \)\(of\)\(\ \)\), 
                
                CounterBox["SlideShowNavigationBar", "LastSlide"]}],
              "SR"],
            ButtonFrame->"None"]}
        }]]]], "SlideShowNavigationBar"],

Cell["Here is a random set of cells", "MathCaption"],

Cell[BoxData[
    \(ListDensityPlot[
      mat\  = \ Table[Random[Integer], \ {100}, {100}]]\)], "Input",
  CellLabel->"In[57]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[GridBox[{
        {
          ButtonBox[
            StyleBox["\[FirstPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageFirst"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"First Slide"], 
          ButtonBox[
            StyleBox["\[LeftPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPagePrevious"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Previous Slide"], 
          ButtonBox[
            StyleBox["\[RightPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageNext"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Next Slide"], 
          ButtonBox[
            StyleBox["\[LastPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageLast"]}],
            ButtonStyle->"SlideHyperlink"], " ", 
          ButtonBox[
            StyleBox[
              RowBox[{
                CounterBox["SlideShowNavigationBar"], \(\(\ \)\(of\)\(\ \)\), 
                
                CounterBox["SlideShowNavigationBar", "LastSlide"]}],
              "SR"],
            ButtonFrame->"None"]}
        }]]]], "SlideShowNavigationBar"],

Cell["This iterates the rule for 10 iterations", "MathCaption"],

Cell[BoxData[
    \(Do[ListDensityPlot[mat\  = \ MatrixUpdate[mat]], {10}]\)], "Input",
  CellLabel->"In[58]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[GridBox[{
        {
          ButtonBox[
            StyleBox["\[FirstPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageFirst"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"First Slide"], 
          ButtonBox[
            StyleBox["\[LeftPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPagePrevious"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Previous Slide"], 
          ButtonBox[
            StyleBox["\[RightPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageNext"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Next Slide"], 
          ButtonBox[
            StyleBox["\[LastPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageLast"]}],
            ButtonStyle->"SlideHyperlink"], " ", 
          ButtonBox[
            StyleBox[
              RowBox[{
                CounterBox["SlideShowNavigationBar"], \(\(\ \)\(of\)\(\ \)\), 
                
                CounterBox["SlideShowNavigationBar", "LastSlide"]}],
              "SR"],
            ButtonFrame->"None"]}
        }]]]], "SlideShowNavigationBar"],

Cell["\<\
Of course, there is nothing in the definitions which prevent us from having \
the input be a sparse array.  Using a SparseArray allows us to have an \
essentially infinite zero background on which you can see if the evolution \
generally spreads or not.\
\>", "Text"],

Cell["\<\
This sets up an initial pattern of cells which is a filled in square in the \
center of a large background.\
\>", "MathCaption"],

Cell[BoxData[{
    \(\(n\  = \ 100;\)\), "\[IndentingNewLine]", 
    \(\(m\  = \ 5;\)\), "\[IndentingNewLine]", 
    \(\(S\  = \ 
        PadRight[
          SparseArray[Table[1, \ {n}, \ {n}]], \ \((2\ m + 1)\) n {1, 1}, \ 
          0, \ m\ n {1, 1}];\)\), "\[IndentingNewLine]", 
    \(ListDensityPlot[S, \ Mesh \[Rule] False]; S\)}], "Input",
  CellLabel->"In[66]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[GridBox[{
        {
          ButtonBox[
            StyleBox["\[FirstPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageFirst"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"First Slide"], 
          ButtonBox[
            StyleBox["\[LeftPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPagePrevious"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Previous Slide"], 
          ButtonBox[
            StyleBox["\[RightPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageNext"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Next Slide"], 
          ButtonBox[
            StyleBox["\[LastPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageLast"]}],
            ButtonStyle->"SlideHyperlink"], " ", 
          ButtonBox[
            StyleBox[
              RowBox[{
                CounterBox["SlideShowNavigationBar"], \(\(\ \)\(of\)\(\ \)\), 
                
                CounterBox["SlideShowNavigationBar", "LastSlide"]}],
              "SR"],
            ButtonFrame->"None"]}
        }]]]], "SlideShowNavigationBar"],

Cell["Now each iteration is fast enough to do several per plot", "Text"],

Cell["\<\
This iterates the game of life rule on the starting pattern given by S, \
showing a plot once for every 10 iterations.\
\>", "MathCaption"],

Cell[BoxData[
    \(Do[ListDensityPlot[Do[S\  = \ SparseArray[MatrixUpdate[S]], {10}]; S, \ 
        Mesh \[Rule] False], {80}]\)], "Input",
  CellLabel->"In[70]:="],

Cell["", "Text",
  CellOpen->False,
  CellTags->{"PlatformSupport", "LastSlide"}]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 723}},
ScreenStyleEnvironment->"SlideShow",
WindowSize->{1016, 696},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
Magnification->1.5,
StyleDefinitions -> "HelpBrowser.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "DenseLinearAlgebra"->{
    Cell[3373, 96, 85, 1, 76, "Section",
      CellTags->"DenseLinearAlgebra"]},
  "PlatformSupport"->{
    Cell[29215, 864, 81, 2, 28, "Text",
      CellOpen->False,
      CellTags->{"PlatformSupport", "LastSlide"}]},
  "LastSlide"->{
    Cell[29215, 864, 81, 2, 28, "Text",
      CellOpen->False,
      CellTags->{"PlatformSupport", "LastSlide"}]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"DenseLinearAlgebra", 30010, 887},
  {"PlatformSupport", 30114, 890},
  {"LastSlide", 30246, 894}
  }
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 1594, 41, 20, "SlideShowNavigationBar"],

Cell[CellGroupData[{
Cell[3373, 96, 85, 1, 76, "Section",
  CellTags->"DenseLinearAlgebra"],
Cell[3461, 99, 210, 6, 62, "Text"],
Cell[3674, 107, 93, 2, 43, "MathCaption"],
Cell[3770, 111, 198, 4, 77, "Input"],
Cell[3971, 117, 153, 3, 62, "Text"],
Cell[4127, 122, 64, 0, 43, "MathCaption"],
Cell[4194, 124, 71, 2, 51, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4302, 131, 1594, 41, 20, "SlideShowNavigationBar"],
Cell[5899, 174, 171, 3, 62, "Text"],
Cell[6073, 179, 103, 2, 43, "MathCaption"],
Cell[6179, 183, 138, 3, 51, "Input"],
Cell[6320, 188, 102, 2, 43, "MathCaption"],
Cell[6425, 192, 104, 2, 51, "Input"],
Cell[6532, 196, 76, 2, 40, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6645, 203, 1594, 41, 28, "SlideShowNavigationBar"],
Cell[8242, 246, 264, 6, 56, "Text"],
Cell[8509, 254, 97, 2, 58, "MathCaption"],
Cell[8609, 258, 71, 2, 40, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8717, 265, 1594, 41, 28, "SlideShowNavigationBar"],
Cell[10314, 308, 113, 3, 34, "Text"],
Cell[10430, 313, 99, 2, 58, "MathCaption"],
Cell[10532, 317, 89, 2, 40, "Input"],
Cell[10624, 321, 119, 3, 58, "MathCaption"],
Cell[10746, 326, 91, 2, 40, "Input"],
Cell[10840, 330, 117, 3, 58, "MathCaption"],
Cell[10960, 335, 102, 2, 40, "Input"],
Cell[11065, 339, 137, 3, 34, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11239, 347, 1594, 41, 28, "SlideShowNavigationBar"],
Cell[12836, 390, 125, 3, 58, "MathCaption"],
Cell[12964, 395, 271, 7, 92, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13272, 407, 1594, 41, 28, "SlideShowNavigationBar"],
Cell[14869, 450, 314, 7, 56, "Text"],
Cell[15186, 459, 84, 1, 58, "MathCaption"],

Cell[CellGroupData[{
Cell[15295, 464, 107, 3, 40, "Input",
  Evaluatable->False],
Cell[15405, 469, 282, 5, 63, "Output"]
}, Closed]],
Cell[15702, 477, 79, 0, 32, "MathCaption"],

Cell[CellGroupData[{
Cell[15806, 481, 112, 3, 40, "Input",
  Evaluatable->False],
Cell[15921, 486, 151, 4, 38, "Output"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[16121, 496, 1594, 41, 28, "SlideShowNavigationBar"],
Cell[17718, 539, 186, 4, 56, "Text"],
Cell[17907, 545, 223, 4, 80, "MathCaption"],
Cell[18133, 551, 300, 5, 118, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[18470, 561, 1594, 41, 28, "SlideShowNavigationBar"],
Cell[20067, 604, 122, 3, 34, "Text"],
Cell[20192, 609, 115, 3, 58, "MathCaption"],
Cell[20310, 614, 819, 17, 244, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[21166, 636, 1594, 41, 28, "SlideShowNavigationBar"],
Cell[22763, 679, 52, 0, 58, "MathCaption"],
Cell[22818, 681, 130, 3, 40, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[22985, 689, 1594, 41, 28, "SlideShowNavigationBar"],
Cell[24582, 732, 63, 0, 58, "MathCaption"],
Cell[24648, 734, 112, 2, 40, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24797, 741, 1594, 41, 28, "SlideShowNavigationBar"],
Cell[26394, 784, 277, 5, 56, "Text"],
Cell[26674, 791, 138, 3, 58, "MathCaption"],
Cell[26815, 796, 371, 8, 118, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[27223, 809, 1594, 41, 28, "SlideShowNavigationBar"],
Cell[28820, 852, 72, 0, 34, "Text"],
Cell[28895, 854, 149, 3, 58, "MathCaption"],
Cell[29047, 859, 165, 3, 66, "Input"],
Cell[29215, 864, 81, 2, 28, "Text",
  CellOpen->False,
  CellTags->{"PlatformSupport", "LastSlide"}]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

