(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     10735,        348]*)
(*NotebookOutlinePosition[     12051,        391]*)
(*  CellTagsIndexPosition[     11903,        383]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[TextData[Cell[BoxData[GridBox[{
        {
          ButtonBox[
            StyleBox["\[FirstPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageFirst"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"First Slide"], 
          ButtonBox[
            StyleBox["\[LeftPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPagePrevious"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Previous Slide"], 
          ButtonBox[
            StyleBox["\[RightPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageNext"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Next Slide"], 
          ButtonBox[
            StyleBox["\[LastPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageLast"]}],
            ButtonStyle->"SlideHyperlink"], " ", 
          ButtonBox[
            StyleBox[
              RowBox[{
                CounterBox["SlideShowNavigationBar"], \(\(\ \)\(of\)\(\ \)\), 
                
                CounterBox["SlideShowNavigationBar", "LastSlide"]}],
              "SR"],
            ButtonFrame->"None"]}
        }]]]], "SlideShowNavigationBar"],

Cell[CellGroupData[{

Cell["Dense Numerical Linear Algebra", "Section",
  CellTags->"DenseLinearAlgebra"],

Cell["\<\
\[Bullet] Modern implementation based on LAPACK and BLAS for machine \
precision numbers.\
\>", "Text"],

Cell["\<\
\[Bullet] Optimized for modern microprocessors with big speed difference \
between processor and memory subsystem. \
\>", "Text"],

Cell["\<\
\[Bullet] Takes advantage of vector instruction sets (some platforms) and \
multi-threading (some platforms). \
\>", "Text"],

Cell[CellGroupData[{

Cell["Examples", "SubsubsectionIcon"],

Cell[TextData[{
  "This commands uses Parallel Computing Toolkit in order to perform the same \
operation in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 4.2 and 5.0"
}], "MathCaption"],

Cell[CellGroupData[{

Cell[BoxData[
    \($Version\  // \ R\)], "Input",
  CellLabel->"In[20]:="],

Cell[BoxData[
    \({"5.0 for Microsoft Windows (April 3, 2003)", 
      "4.2 for Microsoft Windows (June 5, 2002)"}\)], "Output",
  CellLabel->"Out[20]="]
}, Open  ]],

Cell["\<\
This is probably the most important basic operation for linear algebra. \
\>", "MathCaption"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"R", "[", " ", 
      RowBox[{\(A\  = \ Table[\ Random[], \ {10\^3}, \ {10\^3}]\), ";", " ", 
        RowBox[{"Timing", "[", 
          StyleBox[" ",
            FontColor->RGBColor[1, 0, 0]], 
          RowBox[{
            StyleBox[\(A\  . \ A\),
              FontColor->RGBColor[1, 0, 0]], ";"}], " ", "]"}]}], " ", 
      "]"}]], "Input",
  CellLabel->"In[21]:="],

Cell[BoxData[
    \({{4.406`\ Second, Null}, {36.422`\ Second, Null}}\)], "Output",
  CellLabel->"Out[21]="]
}, Open  ]],

Cell["This is essentially the inverse of the last operation. ", "MathCaption"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"R", "[", " ", 
      RowBox[{\(A\  = \ Table[\ Random[], \ {10\^3}, \ {10\^3}]\), 
        ";", \(b\  = \ Table[\ Random[], \ {10\^3}]\), ";", "  ", 
        RowBox[{"Timing", "[", " ", 
          RowBox[{
            StyleBox[\(LinearSolve[A, \ b]\),
              FontColor->RGBColor[1, 0, 0]], ";"}], " ", "]"}]}], " ", 
      "]"}]], "Input",
  CellLabel->"In[149]:="],

Cell[BoxData[
    \({{2.3119999999999994`\ Second, Null}, {25.906000000000006`\ Second, 
        Null}}\)], "Output",
  CellLabel->"Out[149]="]
}, Open  ]],

Cell["This is another example of greater speedup. ", "MathCaption"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"R", "[", " ", 
      RowBox[{\(A\  = \ Table[\ Random[], \ {10\^3}, \ {10\^3}]\), ";", " ", 
        RowBox[{"Timing", "[", 
          StyleBox[" ",
            FontColor->RGBColor[1, 0, 0]], 
          RowBox[{
            StyleBox[\(Inverse[A]\),
              FontColor->RGBColor[1, 0, 0]], ";"}], " ", "]"}]}], " ", 
      "]"}]], "Input",
  CellLabel->"In[150]:="],

Cell[BoxData[
    \({{5.813000000000001`\ Second, Null}, {113.969`\ Second, 
        Null}}\)], "Output",
  CellLabel->"Out[150]="]
}, Open  ]],

Cell[TextData[{
  "Here is a speedup for ",
  StyleBox["SingularValueDecomposition", "MR"],
  ". "
}], "MathCaption"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"R", "[", " ", 
      RowBox[{
        RowBox[{\(A\  = \ Table[\ Random[], \ {500}, \ {500}]\), ";", " ", 
          RowBox[{"Timing", "[", 
            StyleBox[" ",
              FontColor->RGBColor[1, 0, 0]], 
            RowBox[{
              StyleBox[\(SingularValues[A]\),
                FontColor->RGBColor[1, 0, 0]], ";"}], " ", "]"}]}], ",", " ", 
        "M42"}], " ", "]"}]], "Input",
  CellLabel->"In[151]:="],

Cell[BoxData[
    \({34.531000000000006`\ Second, Null}\)], "Output",
  CellLabel->"Out[151]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"R", "[", " ", 
      RowBox[{
        RowBox[{\(A\  = \ Table[\ Random[], \ {500}, \ {500}]\), ";", " ", 
          RowBox[{"Timing", "[", 
            StyleBox[" ",
              FontColor->RGBColor[1, 0, 0]], 
            RowBox[{
              StyleBox[\(SingularValueDecomposition[A]\),
                FontColor->RGBColor[1, 0, 0]], ";"}], " ", "]"}]}], ",", " ", 
        "M50"}], " ", "]"}]], "Input",
  CellLabel->"In[152]:="],

Cell[BoxData[
    \({6.141000000000002`\ Second, Null}\)], "Output",
  CellLabel->"Out[152]="]
}, Open  ]],

Cell["\<\
In some cases you only need the actual singular values, so through a design \
enhancement we get another improvement. \
\>", "MathCaption"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"R", "[", " ", 
      RowBox[{
        RowBox[{\(A\  = \ Table[\ Random[], \ {500}, \ {500}]\), ";", " ", 
          RowBox[{"Timing", "[", 
            StyleBox[" ",
              FontColor->RGBColor[1, 0, 0]], 
            RowBox[{
              StyleBox[\(\(SingularValues[A]\)[\([2]\)]\),
                FontColor->RGBColor[1, 0, 0]], ";"}], " ", "]"}]}], ",", " ", 
        "M42"}], " ", "]"}]], "Input",
  CellLabel->"In[151]:="],

Cell[BoxData[
    \({34.531000000000006`\ Second, Null}\)], "Output",
  CellLabel->"Out[151]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"R", "[", " ", 
      RowBox[{
        RowBox[{\(A\  = \ Table[\ Random[], \ {500}, \ {500}]\), ";", " ", 
          RowBox[{"Timing", "[", 
            StyleBox[" ",
              FontColor->RGBColor[1, 0, 0]], 
            RowBox[{
              StyleBox[\(SingularValueList[A]\),
                FontColor->RGBColor[1, 0, 0]], ";"}], " ", "]"}]}], ",", " ", 
        "M50"}], " ", "]"}]], "Input",
  CellLabel->"In[153]:="],

Cell[BoxData[
    \({3.312000000000001`\ Second, Null}\)], "Output",
  CellLabel->"Out[153]="]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[GridBox[{
        {
          ButtonBox[
            StyleBox["\[FirstPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageFirst"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"First Slide"], 
          ButtonBox[
            StyleBox["\[LeftPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPagePrevious"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Previous Slide"], 
          ButtonBox[
            StyleBox["\[RightPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageNext"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Next Slide"], 
          ButtonBox[
            StyleBox["\[LastPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageLast"]}],
            ButtonStyle->"SlideHyperlink"], " ", 
          ButtonBox[
            StyleBox[
              RowBox[{
                CounterBox["SlideShowNavigationBar"], \(\(\ \)\(of\)\(\ \)\), 
                
                CounterBox["SlideShowNavigationBar", "LastSlide"]}],
              "SR"],
            ButtonFrame->"None"]}
        }]]]], "SlideShowNavigationBar",
  CellTags->"LastSlide"],

Cell["", "Text",
  CellOpen->False,
  CellTags->{"PlatformSupport", "LastSlide"}]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 723}},
ScreenStyleEnvironment->"Brackets",
WindowSize->{886, 651},
WindowMargins->{{12, Automatic}, {1, Automatic}},
Magnification->1.5,
StyleDefinitions -> "New In 5.0 Style.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "DenseLinearAlgebra"->{
    Cell[3373, 96, 83, 1, 82, "Section",
      CellTags->"DenseLinearAlgebra"]},
  "LastSlide"->{
    Cell[9016, 299, 1619, 42, 28, "SlideShowNavigationBar",
      CellTags->"LastSlide"],
    Cell[10638, 343, 81, 2, 28, "Text",
      CellOpen->False,
      CellTags->{"PlatformSupport", "LastSlide"}]},
  "PlatformSupport"->{
    Cell[10638, 343, 81, 2, 28, "Text",
      CellOpen->False,
      CellTags->{"PlatformSupport", "LastSlide"}]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"DenseLinearAlgebra", 11437, 366},
  {"LastSlide", 11535, 369},
  {"PlatformSupport", 11763, 375}
  }
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 1594, 41, 28, "SlideShowNavigationBar"],

Cell[CellGroupData[{
Cell[3373, 96, 83, 1, 82, "Section",
  CellTags->"DenseLinearAlgebra"],
Cell[3459, 99, 113, 3, 34, "Text"],
Cell[3575, 104, 139, 3, 56, "Text"],
Cell[3717, 109, 134, 3, 34, "Text"],

Cell[CellGroupData[{
Cell[3876, 116, 37, 0, 35, "SubsubsectionIcon"],
Cell[3916, 118, 199, 6, 65, "MathCaption"],

Cell[CellGroupData[{
Cell[4140, 128, 75, 2, 51, "Input"],
Cell[4218, 132, 155, 3, 75, "Output"]
}, Open  ]],
Cell[4388, 138, 103, 2, 43, "MathCaption"],

Cell[CellGroupData[{
Cell[4516, 144, 394, 10, 78, "Input"],
Cell[4913, 156, 108, 2, 50, "Output"]
}, Open  ]],
Cell[5036, 161, 78, 0, 43, "MathCaption"],

Cell[CellGroupData[{
Cell[5139, 165, 399, 9, 104, "Input"],
Cell[5541, 176, 143, 3, 50, "Output"]
}, Open  ]],
Cell[5699, 182, 67, 0, 43, "MathCaption"],

Cell[CellGroupData[{
Cell[5791, 186, 396, 10, 78, "Input"],
Cell[6190, 198, 131, 3, 50, "Output"]
}, Open  ]],
Cell[6336, 204, 117, 4, 44, "MathCaption"],

Cell[CellGroupData[{
Cell[6478, 212, 449, 11, 77, "Input"],
Cell[6930, 225, 95, 2, 50, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7062, 232, 461, 11, 77, "Input"],
Cell[7526, 245, 94, 2, 50, "Output"]
}, Open  ]],
Cell[7635, 250, 149, 3, 65, "MathCaption"],

Cell[CellGroupData[{
Cell[7809, 257, 462, 11, 77, "Input"],
Cell[8274, 270, 95, 2, 50, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8406, 277, 452, 11, 77, "Input"],
Cell[8861, 290, 94, 2, 50, "Output"]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[9016, 299, 1619, 42, 28, "SlideShowNavigationBar",
  CellTags->"LastSlide"],
Cell[10638, 343, 81, 2, 28, "Text",
  CellOpen->False,
  CellTags->{"PlatformSupport", "LastSlide"}]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

